@@ -7,7 +7,7 @@ cd kmymoney/ && $PREPARETIPS > tips.cpp
 cd ..
 # call xgettext on all source files. If your sources have other filename
 # extensions besides .cpp, and .h, just add them in the find call.
-$XGETTEXT `find . -name \*.cpp -o -name \*.h` -o $podir/kmymoney.pot
+$XGETTEXT `find . -name \*.cpp -o -name \*.h | grep -v '/tests/'` -o $podir/kmymoney.pot
 
 # Remove these two generated files again
 rm kmymoney/tips.cpp rc.cpp
