@@ -22,31 +22,38 @@
 
 #include <QFile>
 #include <QFileInfo>
-#include <QApplication>
 #include <QTextStream>
 #include <QList>
+#include <QSaveFile>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
 
-#include <KLocale>
+#include <KLocalizedString>
 #include <KMessageBox>
-#include <KSaveFile>
-#include <KTemporaryFile>
-#include <kio/netaccess.h>
+#include <QTemporaryFile>
+#include <KXmlGuiWindow>
+#include <KIO/StoredTransferJob>
+#include <KJobWidgets>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
 #include "kmymoneyutils.h"
+#include "mymoneyfile.h"
+#include "mymoneyaccount.h"
+#include "mymoneyexception.h"
+#include "mymoneyenums.h"
 
 MyMoneyTemplate::MyMoneyTemplate() :
-    m_progressCallback(0)
+    m_progressCallback(0),
+    m_accountsRead(0)
 {
 }
 
-MyMoneyTemplate::MyMoneyTemplate(const KUrl& url) :
-    m_progressCallback(0)
+MyMoneyTemplate::MyMoneyTemplate(const QUrl& url) :
+    m_progressCallback(0),
+    m_accountsRead(0)
 {
   loadTemplate(url);
 }
@@ -55,10 +62,10 @@ MyMoneyTemplate::~MyMoneyTemplate()
 {
 }
 
-bool MyMoneyTemplate::loadTemplate(const KUrl& url)
+bool MyMoneyTemplate::loadTemplate(const QUrl &url)
 {
   QString filename;
-
+  bool downloadedFile = false;
   if (!url.isValid()) {
     qDebug("Invalid template URL '%s'", qPrintable(url.url()));
     return false;
@@ -69,15 +76,24 @@ bool MyMoneyTemplate::loadTemplate(const KUrl& url)
     filename = url.toLocalFile();
 
   } else {
-    bool rc;
-    rc = KIO::NetAccess::download(url, filename, KMyMoneyUtils::mainWindow());
-    if (!rc) {
-      KMessageBox::detailedError(KMyMoneyUtils::mainWindow(),
+    bool rc = false;
+    downloadedFile = true;
+    KIO::StoredTransferJob *transferjob = KIO::storedGet (url);
+    KJobWidgets::setWindow(transferjob, KMyMoneyUtils::mainWindow());
+    rc = transferjob->exec();
+    if (! rc) {
+        KMessageBox::detailedError(KMyMoneyUtils::mainWindow(),
                                  i18n("Error while loading file '%1'.", url.url()),
-                                 KIO::NetAccess::lastErrorString(),
+                                 transferjob->errorString(),
                                  i18n("File access error"));
-      return false;
+        return false;
     }
+    QTemporaryFile file;
+    file.setAutoRemove(false);
+    file.open();
+    file.write(transferjob->data());
+    filename = file.fileName();
+    file.close();
   }
 
   bool rc = true;
@@ -105,10 +121,12 @@ bool MyMoneyTemplate::loadTemplate(const KUrl& url)
     rc = false;
   }
 
-  // if a temporary file was constructed by NetAccess::download,
-  // then it will be removed with the next call. Otherwise, it
-  // stays untouched on the local filesystem
-  KIO::NetAccess::removeTempFile(filename);
+  // if a temporary file was downloaded, then it will be removed
+  // with the next call. Otherwise, it stays untouched on the local
+  // filesystem.
+  if (downloadedFile) {
+    QFile::remove(filename);
+  }
   return rc;
 }
 
@@ -142,7 +160,7 @@ bool MyMoneyTemplate::loadDescription()
         m_longDesc = childElement.text();
         validMask |= validLong;
       } else {
-        KMessageBox::error(KMyMoneyUtils::mainWindow(), i18n("<p>Invalid tag <b>%1</b> in template file <b>%2</b></p>", childElement.tagName(), m_source.prettyUrl()));
+        KMessageBox::error(KMyMoneyUtils::mainWindow(), i18n("<p>Invalid tag <b>%1</b> in template file <b>%2</b></p>", childElement.tagName(), m_source.toDisplayString()));
         validMask |= invalid;
       }
       child = child.nextSibling();
@@ -173,29 +191,21 @@ void MyMoneyTemplate::hierarchy(QMap<QString, QTreeWidgetItem*>& list)
   bool rc = !m_accounts.isNull();
   QDomNode accounts = m_accounts;
   while (rc == true && !accounts.isNull() && accounts.isElement()) {
-    QDomElement childElement = accounts.toElement();
-    if (childElement.tagName() == "account"
-        && childElement.attribute("name").isEmpty()) {
-      switch (childElement.attribute("type").toUInt()) {
-        case MyMoneyAccount::Asset:
-          list[i18n("Asset")] = 0;
-          rc = hierarchy(list, i18n("Asset"), childElement.firstChild());
-          break;
-        case MyMoneyAccount::Liability:
-          list[i18n("Liability")] = 0;
-          rc = hierarchy(list, i18n("Liability"), childElement.firstChild());
-          break;
-        case MyMoneyAccount::Income:
-          list[i18n("Income")] = 0;
-          rc = hierarchy(list, i18n("Income"), childElement.firstChild());
-          break;
-        case MyMoneyAccount::Expense:
-          list[i18n("Expense")] = 0;
-          rc = hierarchy(list, i18n("Expense"), childElement.firstChild());
-          break;
-        case MyMoneyAccount::Equity:
-          list[i18n("Equity")] = 0;
-          rc = hierarchy(list, i18n("Equity"), childElement.firstChild());
+    QDomElement rootNode = accounts.toElement();
+    QString name = rootNode.attribute("name");
+    if (rootNode.tagName() == "account") {
+        rootNode = rootNode.firstChild().toElement();
+        eMyMoney::Account::Type type = static_cast<eMyMoney::Account::Type>(accounts.toElement().attribute("type").toUInt());
+        switch (type) {
+        case eMyMoney::Account::Type::Asset:
+        case eMyMoney::Account::Type::Liability:
+        case eMyMoney::Account::Type::Income:
+        case eMyMoney::Account::Type::Expense:
+        case eMyMoney::Account::Type::Equity:
+          if (name.isEmpty())
+            name = MyMoneyAccount::accountTypeToString(type);
+          list[name] = 0;
+          rc = hierarchy(list, name, rootNode);
           break;
 
         default:
@@ -219,34 +229,36 @@ bool MyMoneyTemplate::importTemplate(void(*callback)(int, int, const QString&))
 
   while (rc == true && !m_accounts.isNull() && m_accounts.isElement()) {
     QDomElement childElement = m_accounts.toElement();
-    if (childElement.tagName() == "account"
-        && childElement.attribute("name").isEmpty()) {
+    if (childElement.tagName() == "account") {
       ++m_accountsRead;
       MyMoneyAccount parent;
       switch (childElement.attribute("type").toUInt()) {
-        case MyMoneyAccount::Asset:
+        case (uint)eMyMoney::Account::Type::Asset:
           parent = file->asset();
           break;
-        case MyMoneyAccount::Liability:
+        case (uint)eMyMoney::Account::Type::Liability:
           parent = file->liability();
           break;
-        case MyMoneyAccount::Income:
+        case (uint)eMyMoney::Account::Type::Income:
           parent = file->income();
           break;
-        case MyMoneyAccount::Expense:
+        case (uint)eMyMoney::Account::Type::Expense:
           parent = file->expense();
           break;
-        case MyMoneyAccount::Equity:
+        case (uint)eMyMoney::Account::Type::Equity:
           parent = file->equity();
           break;
 
         default:
-          KMessageBox::error(KMyMoneyUtils::mainWindow(), i18n("<p>Invalid top-level account type <b>%1</b> in template file <b>%2</b></p>", childElement.attribute("type"), m_source.prettyUrl()));
+          KMessageBox::error(KMyMoneyUtils::mainWindow(), i18n("<p>Invalid top-level account type <b>%1</b> in template file <b>%2</b></p>", childElement.attribute("type"), m_source.toDisplayString()));
           rc = false;
       }
 
       if (rc == true) {
-        rc = createAccounts(parent, childElement.firstChild());
+        if (childElement.attribute("name").isEmpty())
+            rc = createAccounts(parent, childElement.firstChild());
+        else
+            rc = createAccounts(parent, childElement);
       }
     } else {
       rc = false;
@@ -281,7 +293,7 @@ bool MyMoneyTemplate::createAccounts(MyMoneyAccount& parent, QDomNode account)
         if (it == subAccountList.constEnd()) {
           // not found, we need to create it
           acc.setName(accountElement.attribute("name"));
-          acc.setAccountType(static_cast<MyMoneyAccount::_accountTypeE>(accountElement.attribute("type").toUInt()));
+          acc.setAccountType(static_cast<eMyMoney::Account::Type>(accountElement.attribute("type").toUInt()));
           setFlags(acc, account.firstChild());
           try {
             MyMoneyFile::instance()->addAccount(acc, parent);
@@ -307,10 +319,15 @@ bool MyMoneyTemplate::setFlags(MyMoneyAccount& acc, QDomNode flags)
         QString value = flagElement.attribute("name");
         if (value == "Tax") {
           acc.setValue(value.toLatin1(), "Yes");
+        } else if (value == "OpeningBalanceAccount") {
+          acc.setValue("OpeningBalanceAccount", "Yes");
         } else {
-          KMessageBox::error(KMyMoneyUtils::mainWindow(), i18n("<p>Invalid flag type <b>%1</b> for account <b>%3</b> in template file <b>%2</b></p>", flagElement.attribute("name"), m_source.prettyUrl(), acc.name()));
+          KMessageBox::error(KMyMoneyUtils::mainWindow(), i18n("<p>Invalid flag type <b>%1</b> for account <b>%3</b> in template file <b>%2</b></p>", flagElement.attribute("name"), m_source.toDisplayString(), acc.name()));
           rc = false;
         }
+        QString currency = flagElement.attribute("currency");
+        if (!currency.isEmpty())
+          acc.setCurrencyId(currency);
       }
     }
     flags = flags.nextSibling();
@@ -337,22 +354,28 @@ bool MyMoneyTemplate::exportTemplate(void(*callback)(int, int, const QString&))
   m_doc.appendChild(mainElement);
 
   QDomElement title = m_doc.createElement("title");
+  QDomText t = m_doc.createTextNode(m_title);
+  title.appendChild(t);
   mainElement.appendChild(title);
 
   QDomElement shortDesc = m_doc.createElement("shortdesc");
+  t = m_doc.createTextNode(m_shortDesc);
+  shortDesc.appendChild(t);
   mainElement.appendChild(shortDesc);
 
   QDomElement longDesc = m_doc.createElement("longdesc");
+  t = m_doc.createTextNode(m_longDesc);
+  longDesc.appendChild(t);
   mainElement.appendChild(longDesc);
 
   QDomElement accounts = m_doc.createElement("accounts");
   mainElement.appendChild(accounts);
 
-  // addAccountStructure(accounts, MyMoneyFile::instance()->asset());
-  // addAccountStructure(accounts, MyMoneyFile::instance()->liability());
-  addAccountStructure(accounts, MyMoneyFile::instance()->income());
+  addAccountStructure(accounts, MyMoneyFile::instance()->asset());
   addAccountStructure(accounts, MyMoneyFile::instance()->expense());
-  // addAccountStructure(accounts, MyMoneyFile::instance()->equity());
+  addAccountStructure(accounts, MyMoneyFile::instance()->income());
+  addAccountStructure(accounts, MyMoneyFile::instance()->liability());
+  addAccountStructure(accounts, MyMoneyFile::instance()->equity());
 
   return true;
 }
@@ -372,6 +395,26 @@ const QString& MyMoneyTemplate::longDescription() const
   return m_longDesc;
 }
 
+void MyMoneyTemplate::setTitle(const QString &s)
+{
+  m_title = s;
+}
+
+void MyMoneyTemplate::setShortDescription(const QString &s)
+{
+  m_shortDesc = s;
+}
+
+void MyMoneyTemplate::setLongDescription(const QString &s)
+{
+  m_longDesc = s;
+}
+
+static bool nameLessThan(MyMoneyAccount &a1, MyMoneyAccount &a2)
+{
+  return a1.name() < a2.name();
+}
+
 bool MyMoneyTemplate::addAccountStructure(QDomElement& parent, const MyMoneyAccount& acc)
 {
   QDomElement account = m_doc.createElement("account");
@@ -381,14 +424,24 @@ bool MyMoneyTemplate::addAccountStructure(QDomElement& parent, const MyMoneyAcco
     account.setAttribute(QString("name"), QString());
   else
     account.setAttribute(QString("name"), acc.name());
-  account.setAttribute(QString("type"), acc.accountType());
+  account.setAttribute(QString("type"), (int)acc.accountType());
 
   // FIXME: add tax flag stuff
+  if (acc.pairs().contains("OpeningBalanceAccount")) {
+    QString openingBalanceAccount = acc.value("OpeningBalanceAccount");
+    if (openingBalanceAccount == "Yes") {
+      QDomElement flag = m_doc.createElement("flag");
+      flag.setAttribute(QString("name"), "OpeningBalanceAccount");
+      flag.setAttribute(QString("currency"), acc.currencyId());
+      account.appendChild(flag);
+    }
+  }
 
   // any child accounts?
   if (acc.accountList().count() > 0) {
     QList<MyMoneyAccount> list;
     MyMoneyFile::instance()->accountList(list, acc.accountList(), false);
+    qSort(list.begin(), list.end(), nameLessThan);
     QList<MyMoneyAccount>::Iterator it;
     for (it = list.begin(); it != list.end(); ++it) {
       addAccountStructure(account, *it);
@@ -397,7 +450,7 @@ bool MyMoneyTemplate::addAccountStructure(QDomElement& parent, const MyMoneyAcco
   return true;
 }
 
-bool MyMoneyTemplate::saveTemplate(const KUrl& url)
+bool MyMoneyTemplate::saveTemplate(const QUrl &url)
 {
   QString filename;
 
@@ -408,33 +461,40 @@ bool MyMoneyTemplate::saveTemplate(const KUrl& url)
 
   if (url.isLocalFile()) {
     filename = url.toLocalFile();
-    KSaveFile qfile(filename/*, 0600*/);
-    if (qfile.open()) {
+    QSaveFile qfile(filename/*, 0600*/);
+    if (qfile.open(QIODevice::WriteOnly)) {
       saveToLocalFile(&qfile);
-      if (!qfile.finalize()) {
+      if (!qfile.commit()) {
         throw MYMONEYEXCEPTION(i18n("Unable to write changes to '%1'", filename));
       }
     } else {
       throw MYMONEYEXCEPTION(i18n("Unable to write changes to '%1'", filename));
     }
   } else {
-    KTemporaryFile tmpfile;
-    KSaveFile qfile(tmpfile.fileName());
-    if (qfile.open()) {
+    QTemporaryFile tmpfile;
+    tmpfile.open();
+    QSaveFile qfile(tmpfile.fileName());
+    if (qfile.open(QIODevice::WriteOnly)) {
       saveToLocalFile(&qfile);
-      if (!qfile.finalize()) {
-        throw MYMONEYEXCEPTION(i18n("Unable to upload to '%1'", url.url()));
+      if (!qfile.commit()) {
+        throw MYMONEYEXCEPTION(i18n("Unable to upload to '%1'", url.toDisplayString()));
       }
     } else {
-      throw MYMONEYEXCEPTION(i18n("Unable to upload to '%1'", url.url()));
+      throw MYMONEYEXCEPTION(i18n("Unable to upload to '%1'", url.toDisplayString()));
     }
-    if (!KIO::NetAccess::upload(tmpfile.fileName(), url, 0))
-      throw MYMONEYEXCEPTION(i18n("Unable to upload to '%1'", url.url()));
+    int permission = -1;
+    QFile file(tmpfile.fileName());
+    file.open(QIODevice::ReadOnly);
+    KIO::StoredTransferJob *putjob = KIO::storedPut(file.readAll(), url, permission, KIO::JobFlag::Overwrite);
+    if (!putjob->exec()) {
+      throw MYMONEYEXCEPTION(i18n("Unable to upload to '%1'.<br />%2", url.toDisplayString(), putjob->errorString()));
+    }
+    file.close();
   }
   return true;
 }
 
-bool MyMoneyTemplate::saveToLocalFile(KSaveFile* qfile)
+bool MyMoneyTemplate::saveToLocalFile(QSaveFile* qfile)
 {
   QTextStream stream(qfile);
   stream.setCodec("UTF-8");
