@@ -21,47 +21,44 @@
 // ----------------------------------------------------------------------------
 // QT Headers
 
-#include <QObject>
-#include <QDateTime>
-#include <QString>
-#include <QStringList>
-#include <QMap>
+#include <QProcess>
+#include <QDate>
 
 // ----------------------------------------------------------------------------
 // KDE Headers
 
-#include <kprocess.h>
-#include <kurl.h>
-
 // ----------------------------------------------------------------------------
 // Project Headers
 
-#include "mymoneymoney.h"
+#include "csv/import/core/csvimportercore.h"
 
+class KJob;
+class QDate;
+class QTextCodec;
 /**
 Helper class to attend the process which is running the script, in the case
 of a local script being used to fetch the quote.
 
 @author Thomas Baumgart <thb@net-bembel.de> & Ace Jones <acejones@users.sourceforge.net>
 */
-class WebPriceQuoteProcess: public KProcess
+class WebPriceQuoteProcess: public QProcess
 {
   Q_OBJECT
 public:
-  WebPriceQuoteProcess(void);
-  inline void setSymbol(const QString& _symbol) {
-    m_symbol = _symbol; m_string.truncate(0);
+  WebPriceQuoteProcess();
+  inline void setWebID(const QString& _webID) {
+    m_webID = _webID; m_string.truncate(0);
   }
 
-public slots:
+public Q_SLOTS:
   void slotReceivedDataFromFilter();
   void slotProcessExited(int exitCode, QProcess::ExitStatus exitStatus);
 
-signals:
+Q_SIGNALS:
   void processExited(const QString&);
 
 private:
-  QString m_symbol;
+  QString m_webID;
   QString m_string;
 };
 
@@ -73,11 +70,11 @@ by the Finance::Quote package, and more user-friendly names.
 
 @author Thomas Baumgart <thb@net-bembel.de> & Ace Jones <acejones@users.sourceforge.net>, Tony B<tonybloom@users.sourceforge.net>
  */
-class FinanceQuoteProcess: public KProcess
+class FinanceQuoteProcess: public QProcess
 {
   Q_OBJECT
 public:
-  FinanceQuoteProcess(void);
+  FinanceQuoteProcess();
   void launch(const QString& scriptPath);
   bool isFinished() const {
     return(m_isDone);
@@ -86,7 +83,7 @@ public:
   const QString crypticName(const QString& niceName) const;
   const QString niceName(const QString& crypticName) const;
 
-public slots:
+public Q_SLOTS:
   void slotReceivedDataFromFilter();
   void slotProcessExited();
 
@@ -104,18 +101,21 @@ private:
   * for stock prices or currency exchange rates.
   */
 struct WebPriceQuoteSource {
-  WebPriceQuoteSource() {}
+  enum identifyBy {Symbol, IdentificationNumber, Name};
+  WebPriceQuoteSource() : m_skipStripping(false) {}
   explicit WebPriceQuoteSource(const QString& name);
-  WebPriceQuoteSource(const QString& name, const QString& url, const QString& sym, const QString& price, const QString& date, const QString& dateformat, bool skipStripping = false);
+  WebPriceQuoteSource(const QString& name, const QString& url, const QString& csvUrl, const QString& id, const identifyBy idBy, const QString& price, const QString& date, const QString& dateformat, bool skipStripping = false);
   ~WebPriceQuoteSource() {}
 
-  void write(void) const;
+  void write() const;
   void rename(const QString& name);
-  void remove(void) const;
+  void remove() const;
 
   QString    m_name;
   QString    m_url;
-  QString    m_sym;
+  QString    m_csvUrl;
+  QString    m_webID;
+  identifyBy m_webIDBy;
   QString    m_price;
   QString    m_date;
   QString    m_dateformat;
@@ -139,13 +139,14 @@ public:
     FinanceQuote
   } quoteSystemE;
 
+  void setDate(const QDate& _from, const QDate& _to);
   /**
-    * This launches a web-based quote update for the given @p _symbol.
+    * This launches a web-based quote update for the given @p _webID.
     * When the quote is received back from the web source, it will be
     * emitted on the 'quote' signal.
     *
-    * @param _symbol the trading symbol of the stock to fetch a price for
-    * @param _id an arbitrary identifier, which will be emitted in the quote
+    * @param _webID the identification of the stock to fetch a price for
+    * @param _kmmID an arbitrary identifier, which will be emitted in the quote
     *                signal when a price is sent back.
     * @param _source the source of the quote (must be a valid value returned
     *                by quoteSources().  Send QString() to use the default
@@ -153,7 +154,7 @@ public:
     * @return bool Whether the quote fetch process was launched successfully
     */
 
-  bool launch(const QString& _symbol, const QString& _id, const QString& _source = QString());
+  bool launch(const QString& _webID, const QString& _kmmID, const QString& _source = QString());
 
   /**
     * This returns a list of the names of the quote sources
@@ -163,23 +164,29 @@ public:
    * @return QStringList of quote source names
     */
   static const QStringList quoteSources(const _quoteSystemE _system = Native);
+  static const QMap<QString, PricesProfile> defaultCSVQuoteSources();
 
-signals:
+Q_SIGNALS:
+  void csvquote(const QString&, const QString&, MyMoneyStatement&);
   void quote(const QString&, const QString&, const QDate&, const double&);
   void failed(const QString&, const QString&);
   void status(const QString&);
   void error(const QString&);
 
-protected slots:
+protected Q_SLOTS:
+  void slotParseCSVQuote(const QString& filename);
   void slotParseQuote(const QString&);
+  void downloadCSV(KJob* job);
+  void downloadResult(KJob* job);
 
 protected:
-  static const QMap<QString, WebPriceQuoteSource> defaultQuoteSources(void);
+  static const QMap<QString, WebPriceQuoteSource> defaultQuoteSources();
 
 private:
-  bool launchNative(const QString& _symbol, const QString& _id, const QString& _source = QString());
-  bool launchFinanceQuote(const QString& _symbol, const QString& _id, const QString& _source = QString());
-  void enter_loop(void);
+  bool launchCSV(const QString& _webID, const QString& _kmmID, const QString& _source = QString());
+  bool launchNative(const QString& _webID, const QString& _kmmID, const QString& _source = QString());
+  bool launchFinanceQuote(const QString& _webID, const QString& _kmmID, const QString& _source = QString());
+  void enter_loop();
 
   static const QStringList quoteSourcesNative();
   static const QStringList quoteSourcesFinanceQuote();
@@ -201,7 +208,7 @@ public:
   explicit MyMoneyDateFormat(const QString& _format): m_format(_format) {}
   const QString convertDate(const QDate& _in) const;
   const QDate convertString(const QString& _in, bool _strict = true, unsigned _centurymidpoint = QDate::currentDate().year()) const;
-  const QString& format(void) const {
+  const QString& format() const {
     return m_format;
   }
 private:
@@ -222,7 +229,7 @@ class QuoteReceiver : public QObject
 public:
   explicit QuoteReceiver(WebPriceQuote* q, QObject *parent = 0);
   ~QuoteReceiver();
-public slots:
+public Q_SLOTS:
   void slotGetQuote(const QString&, const QString&, const QDate&, const double&);
   void slotStatus(const QString&);
   void slotError(const QString&);
