@@ -4,6 +4,7 @@
     begin                : Fri Dec 15 2006
     copyright            : (C) 2006 by Thomas Baumgart
     email                : Thomas Baumgart <ipwizard@users.sourceforge.net>
+                           (C) 2017 by Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
  ***************************************************************************/
 
 /***************************************************************************
@@ -26,39 +27,94 @@
 // ----------------------------------------------------------------------------
 // KDE Includes
 
-#include <klocale.h>
-#include <kpushbutton.h>
+#include <KLocalizedString>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include <kmymoneycategory.h>
-#include <kmymoneyedit.h>
-#include <kmymoneyaccountselector.h>
-#include <kmymoneycompletion.h>
+#include "investtransactioneditor.h"
+#include "mymoneymoney.h"
+#include "kmymoneycategory.h"
+#include "kmymoneyedit.h"
+#include "kmymoneyaccountselector.h"
+#include "kmymoneycompletion.h"
 #include <kmymoneysettings.h>
-#include <mymoneyfile.h>
+#include "mymoneyfile.h"
+#include "mymoneysplit.h"
+#include "mymoneyaccount.h"
+#include "mymoneysecurity.h"
+#include "dialogenums.h"
+#include "mymoneyenums.h"
 
 using namespace Invest;
 using namespace KMyMoneyRegister;
 
+class Invest::ActivityPrivate
+{
+  Q_DISABLE_COPY(ActivityPrivate)
+
+public:
+  ActivityPrivate()
+  {
+  }
+
+  InvestTransactionEditor     *m_parent;
+  QMap<QString, MyMoneyMoney>  m_priceInfo;
+  bool                         m_memoChanged;
+  QString                      m_memoText;
+};
+
+
+Activity::Activity(InvestTransactionEditor* editor) :
+   d_ptr(new ActivityPrivate)
+{
+  Q_D(Activity);
+  d->m_memoChanged = false;
+  d->m_parent = editor;
+}
+
+Activity::~Activity()
+{
+  Q_D(Activity);
+  delete d;
+}
+
+bool& Activity::memoChanged()
+{
+  Q_D(Activity);
+  return d->m_memoChanged;
+}
+
+QString& Activity::memoText()
+{
+  Q_D(Activity);
+  return d->m_memoText;
+}
+
 bool Activity::isComplete(QString& reason) const
 {
+  Q_D(const Activity);
   Q_UNUSED(reason)
 
-  bool rc = false;
-  KMyMoneySecurity* security = dynamic_cast<KMyMoneySecurity*>(haveWidget("security"));
+  auto rc = false;
+  auto security = dynamic_cast<KMyMoneySecurity*>(haveWidget("security"));
   if (!security->currentText().isEmpty()) {
-    rc = (security->selector()->contains(security->currentText()) || (isMultiSelection() && m_memoChanged));
+    rc = (security->selector()->contains(security->currentText()) || (isMultiSelection() && d->m_memoChanged));
   }
   return rc;
 }
 
-bool Activity::haveAssetAccount(void) const
+QWidget* Activity::haveWidget(const QString& name) const
 {
-  KMyMoneyCategory* cat = dynamic_cast<KMyMoneyCategory*>(haveWidget("asset-account"));
+  Q_D(const Activity);
+  return d->m_parent->haveWidget(name);
+}
 
-  bool rc = true;
+bool Activity::haveAssetAccount() const
+{
+  auto cat = dynamic_cast<KMyMoneyCategory*>(haveWidget("asset-account"));
+
+  auto rc = true;
   if (!isMultiSelection())
     rc = !cat->currentText().isEmpty();
 
@@ -70,44 +126,79 @@ bool Activity::haveAssetAccount(void) const
 
 bool Activity::haveCategoryAndAmount(const QString& category, const QString& amount, bool optional) const
 {
-  KMyMoneyCategory* cat = dynamic_cast<KMyMoneyCategory*>(haveWidget(category));
+  Q_D(const Activity);
+  auto cat = dynamic_cast<KMyMoneyCategory*>(haveWidget(category));
 
-  bool rc = true;
-  if (!isMultiSelection() && !optional)
-    rc = !cat->currentText().isEmpty();
+  auto rc = true;
 
-  if (rc && !cat->currentText().isEmpty()) {
+  if (!cat->currentText().isEmpty()) {
     rc = cat->selector()->contains(cat->currentText()) || cat->isSplitTransaction();
-    if (rc && !amount.isEmpty()) {
-      MyMoneyMoney value = dynamic_cast<kMyMoneyEdit*>(haveWidget(amount))->value();
-      if (!isMultiSelection())
+    if (rc && !amount.isEmpty() && !isMultiSelection()) {
+      if (cat->isSplitTransaction()) {
+        QList<MyMoneySplit>::const_iterator split;
+        QList<MyMoneySplit>::const_iterator splitEnd;
+
+        if (category == "fee-account") {
+          split = d->m_parent->feeSplits().cbegin();
+          splitEnd = d->m_parent->feeSplits().cend();
+        } else if (category == "interest-account") {
+          split = d->m_parent->interestSplits().cbegin();
+          splitEnd = d->m_parent->interestSplits().cend();
+        }
+
+        for (; split != splitEnd; ++split) {
+          if ((*split).value().isZero())
+            rc = false;
+        }
+      } else {
+        MyMoneyMoney value = dynamic_cast<KMyMoneyEdit*>(haveWidget(amount))->value();
         rc = !value.isZero();
+      }
     }
+  } else if (!isMultiSelection() && !optional) {
+    rc = false;
   }
   return rc;
 }
 
-bool Activity::haveShares(void) const
+bool Activity::haveFees(bool optional) const
 {
-  kMyMoneyEdit* amount = dynamic_cast<kMyMoneyEdit*>(haveWidget("shares"));
+  return haveCategoryAndAmount("fee-account", "fee-amount", optional);
+}
+
+bool Activity::haveInterest(bool optional) const
+{
+  return haveCategoryAndAmount("interest-account", "interest-amount", optional);
+}
+
+bool Activity::haveShares() const
+{
+  KMyMoneyEdit* amount = dynamic_cast<KMyMoneyEdit*>(haveWidget("shares"));
   if (isMultiSelection() && amount->value().isZero())
     return true;
 
   return !amount->value().isZero();
 }
 
-bool Activity::havePrice(void) const
+bool Activity::havePrice() const
 {
-  kMyMoneyEdit* amount = dynamic_cast<kMyMoneyEdit*>(haveWidget("price"));
+  KMyMoneyEdit* amount = dynamic_cast<KMyMoneyEdit*>(haveWidget("price"));
   if (isMultiSelection() && amount->value().isZero())
     return true;
 
   return !amount->value().isZero();
 }
 
-bool Activity::createCategorySplits(const MyMoneyTransaction& t, KMyMoneyCategory* cat, kMyMoneyEdit* amount, MyMoneyMoney factor, QList<MyMoneySplit>&splits, const QList<MyMoneySplit>& osplits) const
+bool Activity::isMultiSelection() const
 {
-  bool rc = true;
+  Q_D(const Activity);
+  return d->m_parent->isMultiSelection();
+}
+
+bool Activity::createCategorySplits(const MyMoneyTransaction& t, KMyMoneyCategory* cat, KMyMoneyEdit* amount, MyMoneyMoney factor, QList<MyMoneySplit>&splits, const QList<MyMoneySplit>& osplits) const
+{
+  Q_D(const Activity);
+  auto rc = true;
   if (!isMultiSelection() || (isMultiSelection() && !cat->currentText().isEmpty())) {
     if (!cat->isSplitTransaction()) {
       splits.clear();
@@ -118,7 +209,7 @@ bool Activity::createCategorySplits(const MyMoneyTransaction& t, KMyMoneyCategor
         s1.setAccountId(categoryId);
         s1.setValue(amount->value() * factor);
         if (!s1.value().isZero()) {
-          rc = m_parent->setupPrice(t, s1);
+          rc = d->m_parent->setupPrice(t, s1);
         }
         splits.append(s1);
       }
@@ -131,7 +222,7 @@ bool Activity::createCategorySplits(const MyMoneyTransaction& t, KMyMoneyCategor
 
 void Activity::createAssetAccountSplit(MyMoneySplit& split, const MyMoneySplit& stockSplit) const
 {
-  KMyMoneyCategory* cat = dynamic_cast<KMyMoneyCategory*>(haveWidget("asset-account"));
+  auto cat = dynamic_cast<KMyMoneyCategory*>(haveWidget("asset-account"));
   if (!isMultiSelection() || (isMultiSelection() && !cat->currentText().isEmpty())) {
     QString categoryId;
     categoryId = cat->selectedItem();
@@ -142,21 +233,19 @@ void Activity::createAssetAccountSplit(MyMoneySplit& split, const MyMoneySplit&
 
 MyMoneyMoney Activity::sumSplits(const MyMoneySplit& s0, const QList<MyMoneySplit>& feeSplits, const QList<MyMoneySplit>& interestSplits) const
 {
-  MyMoneyMoney total;
-  total = s0.value();
-  QList<MyMoneySplit>::const_iterator it_s;
-  for (it_s = feeSplits.begin(); it_s != feeSplits.end(); ++it_s) {
-    total += (*it_s).value();
-  }
-  for (it_s = interestSplits.begin(); it_s != interestSplits.end(); ++it_s) {
-    total += (*it_s).value();
-  }
+  auto total = s0.value();
+  foreach (const auto feeSplit, feeSplits)
+    total += feeSplit.value();
+
+  foreach (const auto interestSplit, interestSplits)
+    total += interestSplit.value();
+
   return total;
 }
 
 void Activity::setLabelText(const QString& idx, const QString& txt) const
 {
-  QLabel* w = dynamic_cast<QLabel*>(haveWidget(idx));
+  auto w = dynamic_cast<QLabel*>(haveWidget(idx));
   if (w) {
     w->setText(txt);
   } else {
@@ -167,13 +256,14 @@ void Activity::setLabelText(const QString& idx, const QString& txt) const
   }
 }
 
-void Activity::preloadAssetAccount(void)
+void Activity::preloadAssetAccount()
 {
+  Q_D(Activity);
   KMyMoneyCategory* cat;
   cat = dynamic_cast<KMyMoneyCategory*>(haveWidget("asset-account"));
   if (cat->isVisible()) {
     if (cat->currentText().isEmpty()) {
-      MyMoneyAccount acc = MyMoneyFile::instance()->accountByName(i18n("%1 (Brokerage)", m_parent->account().name()));
+      MyMoneyAccount acc = MyMoneyFile::instance()->accountByName(i18n("%1 (Brokerage)", d->m_parent->account().name()));
       if (!acc.id().isEmpty()) {
         bool blocked = cat->signalsBlocked();
         // block signals, so that the focus does not go crazy
@@ -186,32 +276,71 @@ void Activity::preloadAssetAccount(void)
   }
 }
 
-void Buy::showWidgets(void) const
+void Activity::setWidgetVisibility(const QStringList& widgetIds, bool visible) const
 {
-  KMyMoneyCategory* cat;
-  cat = dynamic_cast<KMyMoneyCategory*>(haveWidget("fee-account"));
-  cat->show();
-  cat->splitButton()->show();
-
-  QStringList widgets;
-  widgets << "asset-account" << "shares" << "price" << "total";
-  QStringList::const_iterator it_w;
-  for (it_w = widgets.constBegin(); it_w != widgets.constEnd(); ++it_w) {
-    QWidget* w = haveWidget(*it_w);
-    if (w)
-      w->show();
+  for (QStringList::const_iterator it_w = widgetIds.constBegin(); it_w != widgetIds.constEnd(); ++it_w) {
+    auto w = haveWidget(*it_w);
+    if (w) {
+      if (visible) {
+        w->show();
+      } else {
+        w->hide();
+      }
+    }
   }
+}
 
+eDialogs::PriceMode Activity::priceMode() const
+{
+  Q_D(const Activity);
+  return d->m_parent->priceMode();
+}
+
+QString Activity::priceLabel() const
+{
+  QString label;
+  if (priceMode() == eDialogs::PriceMode::Price) {
+    label = i18n("Price");
+  } else if (priceMode() == eDialogs::PriceMode::PricePerShare) {
+    label = i18n("Price/share");
+  } else if (priceMode() == eDialogs::PriceMode::PricePerTransaction) {
+    label = i18n("Transaction amount");
+  }
+  return label;
+}
+
+Buy::Buy(InvestTransactionEditor* editor) :
+  Activity(editor)
+{
+}
+
+Buy::~Buy()
+{
+}
+
+eMyMoney::Split::InvestmentTransactionType Buy::type() const
+{
+  return eMyMoney::Split::InvestmentTransactionType::BuyShares;
+}
+
+void Buy::showWidgets() const
+{
+  static const QStringList visibleWidgetIds = QStringList() << "asset-account" << "shares" << "price" << "total" << "interest-account" << "fee-account";
+  setWidgetVisibility(visibleWidgetIds, true);
+
+  setLabelText("interest-amount-label", i18n("Interest"));
+  setLabelText("interest-label", i18n("Interest"));
   setLabelText("fee-label", i18n("Fees"));
   setLabelText("asset-label", i18n("Account"));
   setLabelText("shares-label", i18n("Shares"));
-  setLabelText("price-label", i18n("Price/share"));
+  if (dynamic_cast<QLabel*>(haveWidget("price-label")))
+    setLabelText("price-label", priceLabel());
   setLabelText("total-label", i18nc("Total value", "Total"));
 }
 
 bool Buy::isComplete(QString& reason) const
 {
-  bool rc = Activity::isComplete(reason);
+  auto rc = Activity::isComplete(reason);
   rc &= haveAssetAccount();
   rc &= haveFees(true);
   rc &= haveShares();
@@ -222,6 +351,7 @@ bool Buy::isComplete(QString& reason) const
 
 bool Buy::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySplit& assetAccountSplit, QList<MyMoneySplit>& feeSplits, QList<MyMoneySplit>& m_feeSplits, QList<MyMoneySplit>& interestSplits, QList<MyMoneySplit>& m_interestSplits, MyMoneySecurity& security, MyMoneySecurity& currency)
 {
+  Q_D(Activity);
   Q_UNUSED(m_interestSplits);
   Q_UNUSED(security);
   Q_UNUSED(currency);
@@ -230,10 +360,10 @@ bool Buy::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySpli
   if (!isComplete(reason))
     return false;
 
-  kMyMoneyEdit* sharesEdit = dynamic_cast<kMyMoneyEdit*>(haveWidget("shares"));
-  kMyMoneyEdit* priceEdit = dynamic_cast<kMyMoneyEdit*>(haveWidget("price"));
+  auto sharesEdit = dynamic_cast<KMyMoneyEdit*>(haveWidget("shares"));
+  auto priceEdit = dynamic_cast<KMyMoneyEdit*>(haveWidget("price"));
 
-  s0.setAction(MyMoneySplit::BuyShares);
+  s0.setAction(eMyMoney::Split::InvestmentTransactionType::BuyShares);
 
   MyMoneyMoney shares = s0.shares();
   MyMoneyMoney price;
@@ -248,7 +378,7 @@ bool Buy::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySpli
   }
   if (!isMultiSelection() || (isMultiSelection() && !priceEdit->value().isZero())) {
     price = priceEdit->value().abs();
-    if (priceMode() == InvestTransactionEditor::PricePerTransaction) {
+    if (priceMode() == eDialogs::PriceMode::PricePerTransaction) {
       s0.setValue(price.reduce());
       if (!s0.shares().isZero())
         s0.setPrice((price / s0.shares()).reduce());
@@ -258,7 +388,7 @@ bool Buy::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySpli
     }
   }
 
-  if (!createCategorySplits(t, dynamic_cast<KMyMoneyCategory*>(haveWidget("fee-account")), dynamic_cast<kMyMoneyEdit*>(haveWidget("fee-amount")), MyMoneyMoney(1, 1), feeSplits, m_feeSplits))
+  if (!createCategorySplits(t, dynamic_cast<KMyMoneyCategory*>(haveWidget("fee-account")), dynamic_cast<KMyMoneyEdit*>(haveWidget("fee-amount")), MyMoneyMoney::ONE, feeSplits, m_feeSplits))
     return false;
 
   createAssetAccountSplit(assetAccountSplit, s0);
@@ -270,46 +400,48 @@ bool Buy::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySpli
 
   assetAccountSplit.setValue(-total);
 
-  if (!m_parent->setupPrice(t, assetAccountSplit))
+  if (!d->m_parent->setupPrice(t, assetAccountSplit))
     return false;
 
   return true;
 }
 
-void Sell::showWidgets(void) const
+Sell::Sell(InvestTransactionEditor* editor) :
+  Activity(editor)
 {
-  KMyMoneyCategory* cat;
-  cat = dynamic_cast<KMyMoneyCategory*>(haveWidget("interest-account"));
-  cat->show();
-  cat->splitButton()->show();
+}
 
-  cat = dynamic_cast<KMyMoneyCategory*>(haveWidget("fee-account"));
-  cat->show();
-  cat->splitButton()->show();
+Sell::~Sell()
+{
+}
 
-  kMyMoneyEdit* shareEdit = dynamic_cast<kMyMoneyEdit*>(haveWidget("shares"));
-  shareEdit->setPrecision(MyMoneyMoney::denomToPrec(m_parent->security().smallestAccountFraction()));
+eMyMoney::Split::InvestmentTransactionType Sell::type() const
+{
+  return eMyMoney::Split::InvestmentTransactionType::SellShares;
+}
+
+void Sell::showWidgets() const
+{
+  Q_D(const Activity);
+  static const QStringList visibleWidgetIds = QStringList() << "asset-account" << "interest-amount" << "shares" << "price" << "total" << "interest-account" << "fee-account";
+  setWidgetVisibility(visibleWidgetIds, true);
+
+  auto shareEdit = dynamic_cast<KMyMoneyEdit*>(haveWidget("shares"));
+  shareEdit->setPrecision(MyMoneyMoney::denomToPrec(d->m_parent->security().smallestAccountFraction()));
 
-  QStringList widgets;
-  widgets << "asset-account" << "interest-amount" << "shares" << "price" << "total";
-  QStringList::const_iterator it_w;
-  for (it_w = widgets.constBegin(); it_w != widgets.constEnd(); ++it_w) {
-    QWidget* w = haveWidget(*it_w);
-    if (w)
-      w->show();
-  }
   setLabelText("interest-amount-label", i18n("Interest"));
   setLabelText("interest-label", i18n("Interest"));
   setLabelText("fee-label", i18n("Fees"));
   setLabelText("asset-label", i18n("Account"));
   setLabelText("shares-label", i18n("Shares"));
-  setLabelText("price-label", i18n("Price/share"));
+  if (dynamic_cast<QLabel*>(haveWidget("price-label")))
+    setLabelText("price-label", priceLabel());
   setLabelText("total-label", i18nc("Total value", "Total"));
 }
 
 bool Sell::isComplete(QString& reason) const
 {
-  bool rc = Activity::isComplete(reason);
+  auto rc = Activity::isComplete(reason);
   rc &= haveAssetAccount();
   rc &= haveFees(true);
   rc &= haveInterest(true);
@@ -320,6 +452,7 @@ bool Sell::isComplete(QString& reason) const
 
 bool Sell::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySplit& assetAccountSplit, QList<MyMoneySplit>& feeSplits, QList<MyMoneySplit>& m_feeSplits, QList<MyMoneySplit>& interestSplits, QList<MyMoneySplit>& m_interestSplits, MyMoneySecurity& security, MyMoneySecurity& currency)
 {
+  Q_D(Activity);
   Q_UNUSED(m_interestSplits);
   Q_UNUSED(security);
   Q_UNUSED(currency);
@@ -328,10 +461,10 @@ bool Sell::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySpl
   if (!isComplete(reason))
     return false;
 
-  kMyMoneyEdit* sharesEdit = dynamic_cast<kMyMoneyEdit*>(haveWidget("shares"));
-  kMyMoneyEdit* priceEdit = dynamic_cast<kMyMoneyEdit*>(haveWidget("price"));
+  auto sharesEdit = dynamic_cast<KMyMoneyEdit*>(haveWidget("shares"));
+  auto priceEdit = dynamic_cast<KMyMoneyEdit*>(haveWidget("price"));
 
-  s0.setAction(MyMoneySplit::BuyShares);
+  s0.setAction(eMyMoney::Split::InvestmentTransactionType::BuyShares);
 
   MyMoneyMoney shares = s0.shares();
   MyMoneyMoney price;
@@ -346,7 +479,7 @@ bool Sell::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySpl
   }
   if (!isMultiSelection() || (isMultiSelection() && !priceEdit->value().isZero())) {
     price = priceEdit->value().abs();
-    if (priceMode() == InvestTransactionEditor::PricePerTransaction) {
+    if (priceMode() == eDialogs::PriceMode::PricePerTransaction) {
       price = -price;
       s0.setValue(price.reduce());
       if (!s0.shares().isZero())
@@ -357,10 +490,10 @@ bool Sell::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySpl
     }
   }
 
-  if (!createCategorySplits(t, dynamic_cast<KMyMoneyCategory*>(haveWidget("fee-account")), dynamic_cast<kMyMoneyEdit*>(haveWidget("fee-amount")), MyMoneyMoney(1, 1), feeSplits, m_feeSplits))
+  if (!createCategorySplits(t, dynamic_cast<KMyMoneyCategory*>(haveWidget("fee-account")), dynamic_cast<KMyMoneyEdit*>(haveWidget("fee-amount")), MyMoneyMoney::ONE, feeSplits, m_feeSplits))
     return false;
 
-  if (!createCategorySplits(t, dynamic_cast<KMyMoneyCategory*>(haveWidget("interest-account")), dynamic_cast<kMyMoneyEdit*>(haveWidget("interest-amount")), MyMoneyMoney(-1, 1), interestSplits, m_interestSplits))
+  if (!createCategorySplits(t, dynamic_cast<KMyMoneyCategory*>(haveWidget("interest-account")), dynamic_cast<KMyMoneyEdit*>(haveWidget("interest-amount")), MyMoneyMoney::MINUS_ONE, interestSplits, m_interestSplits))
     return false;
 
   createAssetAccountSplit(assetAccountSplit, s0);
@@ -368,40 +501,33 @@ bool Sell::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySpl
   MyMoneyMoney total = sumSplits(s0, feeSplits, interestSplits);
   assetAccountSplit.setValue(-total);
 
-  if (!m_parent->setupPrice(t, assetAccountSplit))
+  if (!d->m_parent->setupPrice(t, assetAccountSplit))
     return false;
 
   return true;
 }
 
-void Div::showWidgets(void) const
+Div::Div(InvestTransactionEditor* editor) :
+  Activity(editor)
 {
-  KMyMoneyCategory* cat;
+}
 
-  cat = dynamic_cast<KMyMoneyCategory*>(haveWidget("interest-account"));
-  cat->show();
-  cat->splitButton()->show();
+Div::~Div()
+{
+}
 
-  cat = dynamic_cast<KMyMoneyCategory*>(haveWidget("fee-account"));
-  cat->show();
-  cat->splitButton()->show();
+eMyMoney::Split::InvestmentTransactionType Div::type() const
+{
+  return eMyMoney::Split::InvestmentTransactionType::Dividend;
+}
 
-  QStringList widgets;
-  widgets << "asset-account" << "interest-amount" << "total";
-  QStringList::const_iterator it_w;
-  for (it_w = widgets.constBegin(); it_w != widgets.constEnd(); ++it_w) {
-    QWidget* w = haveWidget(*it_w);
-    if (w)
-      w->show();
-  }
+void Div::showWidgets() const
+{
+  static const QStringList visibleWidgetIds = QStringList() << "asset-account" << "interest-amount" << "total" << "interest-account" << "fee-account";
+  setWidgetVisibility(visibleWidgetIds, true);
+  static const QStringList hiddenWidgetIds = QStringList() << "shares" << "price";
+  setWidgetVisibility(hiddenWidgetIds, false);
 
-  widgets.clear();
-  widgets << "shares" << "price";
-  for (it_w = widgets.constBegin(); it_w != widgets.constEnd(); ++it_w) {
-    QWidget* w = haveWidget(*it_w);
-    if (w)
-      w->hide();
-  }
   setLabelText("interest-amount-label", i18n("Interest"));
   setLabelText("interest-label", i18n("Interest"));
   setLabelText("fee-label", i18n("Fees"));
@@ -413,7 +539,7 @@ bool Div::isComplete(QString& reason) const
 {
   Q_UNUSED(reason)
 
-  bool rc = Activity::isComplete(reason);
+  auto rc = Activity::isComplete(reason);
   rc &= haveAssetAccount();
   rc &= haveCategoryAndAmount("interest-account", QString(), false);
   rc &= haveInterest(false);
@@ -422,6 +548,7 @@ bool Div::isComplete(QString& reason) const
 
 bool Div::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySplit& assetAccountSplit, QList<MyMoneySplit>& feeSplits, QList<MyMoneySplit>& m_feeSplits, QList<MyMoneySplit>& interestSplits, QList<MyMoneySplit>& m_interestSplits, MyMoneySecurity& security, MyMoneySecurity& currency)
 {
+  Q_D(Activity);
   Q_UNUSED(m_feeSplits);
   Q_UNUSED(security);
   Q_UNUSED(currency);
@@ -430,18 +557,18 @@ bool Div::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySpli
   if (!isComplete(reason))
     return false;
 
-  s0.setAction(MyMoneySplit::Dividend);
+  s0.setAction(eMyMoney::Split::InvestmentTransactionType::Dividend);
 
   // for dividends, we only use the stock split as a marker
   MyMoneyMoney shares;
   s0.setShares(shares);
   s0.setValue(shares);
-  s0.setPrice(MyMoneyMoney(1, 1));
+  s0.setPrice(MyMoneyMoney::ONE);
 
-  if (!createCategorySplits(t, dynamic_cast<KMyMoneyCategory*>(haveWidget("fee-account")), dynamic_cast<kMyMoneyEdit*>(haveWidget("fee-amount")), MyMoneyMoney(1, 1), feeSplits, m_feeSplits))
+  if (!createCategorySplits(t, dynamic_cast<KMyMoneyCategory*>(haveWidget("fee-account")), dynamic_cast<KMyMoneyEdit*>(haveWidget("fee-amount")), MyMoneyMoney::ONE, feeSplits, m_feeSplits))
     return false;
 
-  if (!createCategorySplits(t, dynamic_cast<KMyMoneyCategory*>(haveWidget("interest-account")), dynamic_cast<kMyMoneyEdit*>(haveWidget("interest-amount")), MyMoneyMoney(-1, 1), interestSplits, m_interestSplits))
+  if (!createCategorySplits(t, dynamic_cast<KMyMoneyCategory*>(haveWidget("interest-account")), dynamic_cast<KMyMoneyEdit*>(haveWidget("interest-amount")), MyMoneyMoney::MINUS_ONE, interestSplits, m_interestSplits))
     return false;
 
   createAssetAccountSplit(assetAccountSplit, s0);
@@ -449,48 +576,52 @@ bool Div::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySpli
   MyMoneyMoney total = sumSplits(s0, feeSplits, interestSplits);
   assetAccountSplit.setValue(-total);
 
-  if (!m_parent->setupPrice(t, assetAccountSplit))
+  if (!d->m_parent->setupPrice(t, assetAccountSplit))
     return false;
 
   return true;
 }
 
-void Reinvest::showWidgets(void) const
+Reinvest::Reinvest(InvestTransactionEditor* editor) :
+  Activity(editor)
 {
-  KMyMoneyCategory* cat;
-  cat = dynamic_cast<KMyMoneyCategory*>(haveWidget("interest-account"));
-  cat->show();
-  cat->splitButton()->hide();  //  don't need it as no interest-amount
+}
 
-  cat = dynamic_cast<KMyMoneyCategory*>(haveWidget("fee-account"));
-  cat->show();
-  cat->splitButton()->show();
+Reinvest::~Reinvest()
+{
+}
 
-  kMyMoneyEdit* shareEdit = dynamic_cast<kMyMoneyEdit*>(haveWidget("shares"));
+eMyMoney::Split::InvestmentTransactionType Reinvest::type() const
+{
+  return eMyMoney::Split::InvestmentTransactionType::ReinvestDividend;
+}
+
+void Reinvest::showWidgets() const
+{
+  Q_D(const Activity);
+  static const QStringList visibleWidgetIds = QStringList() << "price" << "interest-account";
+  setWidgetVisibility(visibleWidgetIds, true);
+
+  auto shareEdit = dynamic_cast<KMyMoneyEdit*>(haveWidget("shares"));
   shareEdit->show();
-  shareEdit->setPrecision(MyMoneyMoney::denomToPrec(m_parent->security().smallestAccountFraction()));
+  shareEdit->setPrecision(MyMoneyMoney::denomToPrec(d->m_parent->security().smallestAccountFraction()));
 
-  QWidget* w = haveWidget("price");
-  if (w)
-    w->show();
-
-  kMyMoneyEdit* intAmount = dynamic_cast<kMyMoneyEdit*>(haveWidget("interest-amount"));
+  KMyMoneyEdit* intAmount = dynamic_cast<KMyMoneyEdit*>(haveWidget("interest-amount"));
   intAmount->hide();
   setLabelText("interest-amount-label", QString());
   intAmount->setValue(MyMoneyMoney());
 
-  setLabelText("fee-label", i18n("Fees"));
   setLabelText("interest-label", i18n("Interest"));
   setLabelText("shares-label", i18n("Shares"));
-  setLabelText("price-label", i18n("Price/share"));
+  if (dynamic_cast<QLabel*>(haveWidget("price-label")))
+    setLabelText("price-label", priceLabel());
   setLabelText("total-label", i18nc("Total value", "Total"));
 }
 
 bool Reinvest::isComplete(QString& reason) const
 {
-  bool rc = Activity::isComplete(reason);
+  auto rc = Activity::isComplete(reason);
   rc &= haveCategoryAndAmount("interest-account", QString(), false);
-  rc &= haveFees(true);
   rc &= haveShares();
   rc &= havePrice();
   return rc;
@@ -498,18 +629,20 @@ bool Reinvest::isComplete(QString& reason) const
 
 bool Reinvest::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySplit& assetAccountSplit, QList<MyMoneySplit>& feeSplits, QList<MyMoneySplit>& m_feeSplits, QList<MyMoneySplit>& interestSplits, QList<MyMoneySplit>& m_interestSplits, MyMoneySecurity& security, MyMoneySecurity& currency)
 {
+  Q_D(Activity);
   Q_UNUSED(assetAccountSplit);
   Q_UNUSED(security);
   Q_UNUSED(currency);
+  Q_UNUSED(m_feeSplits)
 
   QString reason;
   if (!isComplete(reason))
     return false;
 
-  kMyMoneyEdit* sharesEdit = dynamic_cast<kMyMoneyEdit*>(haveWidget("shares"));
-  kMyMoneyEdit* priceEdit = dynamic_cast<kMyMoneyEdit*>(haveWidget("price"));
+  auto sharesEdit = dynamic_cast<KMyMoneyEdit*>(haveWidget("shares"));
+  auto priceEdit = dynamic_cast<KMyMoneyEdit*>(haveWidget("price"));
 
-  s0.setAction(MyMoneySplit::ReinvestDividend);
+  s0.setAction(eMyMoney::Split::InvestmentTransactionType::ReinvestDividend);
 
   MyMoneyMoney shares = s0.shares();
   MyMoneyMoney price;
@@ -524,7 +657,7 @@ bool Reinvest::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMone
   }
   if (!isMultiSelection() || (isMultiSelection() && !priceEdit->value().isZero())) {
     price = priceEdit->value().abs();
-    if (priceMode() == InvestTransactionEditor::PricePerTransaction) {
+    if (priceMode() == eDialogs::PriceMode::PricePerTransaction) {
       s0.setValue(price.reduce());
       if (!s0.shares().isZero())
         s0.setPrice((price / s0.shares()).reduce());
@@ -534,16 +667,14 @@ bool Reinvest::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMone
     }
   }
 
-  if (!createCategorySplits(t, dynamic_cast<KMyMoneyCategory*>(haveWidget("fee-account")), dynamic_cast<kMyMoneyEdit*>(haveWidget("fee-amount")), MyMoneyMoney(1, 1), feeSplits, m_feeSplits))
-    return false;
-
-  if (!createCategorySplits(t, dynamic_cast<KMyMoneyCategory*>(haveWidget("interest-account")), dynamic_cast<kMyMoneyEdit*>(haveWidget("interest-amount")), MyMoneyMoney(-1, 1), interestSplits, m_interestSplits))
+  if (!createCategorySplits(t, dynamic_cast<KMyMoneyCategory*>(haveWidget("interest-account")), dynamic_cast<KMyMoneyEdit*>(haveWidget("interest-amount")), MyMoneyMoney::MINUS_ONE, interestSplits, m_interestSplits))
     return false;
 
   if (interestSplits.count() != 1) {
     qDebug("more or less than one interest split in Reinvest::createTransaction. Not created.");
     return false;
   }
+  assetAccountSplit.setAccountId(QString());
 
   MyMoneySplit& s1 = interestSplits[0];
 
@@ -551,24 +682,39 @@ bool Reinvest::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMone
 
   s1.setValue(-total);
 
-  if (!m_parent->setupPrice(t, s1))
+  if (!d->m_parent->setupPrice(t, s1))
     return false;
 
   return true;
 }
 
-void Add::showWidgets(void) const
+Add::Add(InvestTransactionEditor* editor) :
+  Activity(editor)
 {
-  kMyMoneyEdit* shareEdit = dynamic_cast<kMyMoneyEdit*>(haveWidget("shares"));
+}
+
+Add::~Add()
+{
+}
+
+eMyMoney::Split::InvestmentTransactionType Add::type() const
+{
+  return eMyMoney::Split::InvestmentTransactionType::AddShares;
+}
+
+void Add::showWidgets() const
+{
+  Q_D(const Activity);
+  auto shareEdit = dynamic_cast<KMyMoneyEdit*>(haveWidget("shares"));
   shareEdit->show();
-  shareEdit->setPrecision(MyMoneyMoney::denomToPrec(m_parent->security().smallestAccountFraction()));
+  shareEdit->setPrecision(MyMoneyMoney::denomToPrec(d->m_parent->security().smallestAccountFraction()));
 
   setLabelText("shares-label", i18n("Shares"));
 }
 
 bool Add::isComplete(QString& reason) const
 {
-  bool rc = Activity::isComplete(reason);
+  auto rc = Activity::isComplete(reason);
   rc &= haveShares();
   return rc;
 }
@@ -586,12 +732,12 @@ bool Add::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySpli
   if (!isComplete(reason))
     return false;
 
-  kMyMoneyEdit* sharesEdit = dynamic_cast<kMyMoneyEdit*>(haveWidget("shares"));
+  auto sharesEdit = dynamic_cast<KMyMoneyEdit*>(haveWidget("shares"));
 
-  s0.setAction(MyMoneySplit::AddShares);
+  s0.setAction(eMyMoney::Split::InvestmentTransactionType::AddShares);
   s0.setShares(sharesEdit->value().abs());
-  s0.setValue(MyMoneyMoney(0, 1));
-  s0.setPrice(MyMoneyMoney(0, 1));
+  s0.setValue(MyMoneyMoney());
+  s0.setPrice(MyMoneyMoney());
 
   assetAccountSplit.setValue(MyMoneyMoney());//  Clear any leftover value from previous Dividend.
 
@@ -601,17 +747,32 @@ bool Add::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySpli
   return true;
 }
 
-void Remove::showWidgets(void) const
+Remove::Remove(InvestTransactionEditor* editor) :
+  Activity(editor)
 {
-  kMyMoneyEdit* shareEdit = dynamic_cast<kMyMoneyEdit*>(haveWidget("shares"));
+}
+
+Remove::~Remove()
+{
+}
+
+eMyMoney::Split::InvestmentTransactionType Remove::type() const
+{
+  return eMyMoney::Split::InvestmentTransactionType::RemoveShares;
+}
+
+void Remove::showWidgets() const
+{
+  Q_D(const Activity);
+  auto shareEdit = dynamic_cast<KMyMoneyEdit*>(haveWidget("shares"));
   shareEdit->show();
-  shareEdit->setPrecision(MyMoneyMoney::denomToPrec(m_parent->security().smallestAccountFraction()));
+  shareEdit->setPrecision(MyMoneyMoney::denomToPrec(d->m_parent->security().smallestAccountFraction()));
   setLabelText("shares-label", i18n("Shares"));
 }
 
 bool Remove::isComplete(QString& reason) const
 {
-  bool rc = Activity::isComplete(reason);
+  auto rc = Activity::isComplete(reason);
   rc &= haveShares();
   return rc;
 }
@@ -629,12 +790,12 @@ bool Remove::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneyS
   if (!isComplete(reason))
     return false;
 
-  kMyMoneyEdit* sharesEdit = dynamic_cast<kMyMoneyEdit*>(haveWidget("shares"));
+  auto sharesEdit = dynamic_cast<KMyMoneyEdit*>(haveWidget("shares"));
 
-  s0.setAction(MyMoneySplit::AddShares);
+  s0.setAction(eMyMoney::Split::InvestmentTransactionType::AddShares);
   s0.setShares(-(sharesEdit->value().abs()));
-  s0.setValue(MyMoneyMoney(0, 1));
-  s0.setPrice(MyMoneyMoney(0, 1));
+  s0.setValue(MyMoneyMoney());
+  s0.setPrice(MyMoneyMoney());
 
   assetAccountSplit.setValue(MyMoneyMoney());//  Clear any leftover value from previous Dividend.
 
@@ -644,24 +805,38 @@ bool Remove::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneyS
   return true;
 }
 
-void Split::showWidgets(void) const
+Invest::Split::Split(InvestTransactionEditor* editor) :
+  Activity(editor)
+{
+}
+
+Invest::Split::~Split()
+{
+}
+
+eMyMoney::Split::InvestmentTransactionType Invest::Split::type() const
+{
+  return eMyMoney::Split::InvestmentTransactionType::SplitShares;
+}
+
+void Invest::Split::showWidgets() const
 {
   // TODO do we need a special split ratio widget?
   // TODO maybe yes, currently the precision is the one of the fraction and might differ from it
-  kMyMoneyEdit* shareEdit = dynamic_cast<kMyMoneyEdit*>(haveWidget("shares"));
+  auto shareEdit = dynamic_cast<KMyMoneyEdit*>(haveWidget("shares"));
   shareEdit->show();
   shareEdit->setPrecision(-1);
   setLabelText("shares-label", i18n("Ratio 1/"));
 }
 
-bool Split::isComplete(QString& reason) const
+bool Invest::Split::isComplete(QString& reason) const
 {
-  bool rc = Activity::isComplete(reason);
+  auto rc = Activity::isComplete(reason);
   rc &= haveShares();
   return rc;
 }
 
-bool Split::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySplit& assetAccountSplit, QList<MyMoneySplit>& feeSplits, QList<MyMoneySplit>& m_feeSplits, QList<MyMoneySplit>& interestSplits, QList<MyMoneySplit>& m_interestSplits, MyMoneySecurity& security, MyMoneySecurity& currency)
+bool Invest::Split::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySplit& assetAccountSplit, QList<MyMoneySplit>& feeSplits, QList<MyMoneySplit>& m_feeSplits, QList<MyMoneySplit>& interestSplits, QList<MyMoneySplit>& m_interestSplits, MyMoneySecurity& security, MyMoneySecurity& currency)
 {
   Q_UNUSED(t);
   Q_UNUSED(assetAccountSplit);
@@ -670,7 +845,7 @@ bool Split::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySp
   Q_UNUSED(security);
   Q_UNUSED(currency);
 
-  kMyMoneyEdit* sharesEdit = dynamic_cast<kMyMoneyEdit*>(haveWidget("shares"));
+  auto sharesEdit = dynamic_cast<KMyMoneyEdit*>(haveWidget("shares"));
 
   KMyMoneyCategory* cat;
   cat = dynamic_cast<KMyMoneyCategory*>(haveWidget("interest-account"));
@@ -678,10 +853,10 @@ bool Split::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySp
   cat = dynamic_cast<KMyMoneyCategory*>(haveWidget("fee-account"));
   cat->parentWidget()->hide();
 
-  s0.setAction(MyMoneySplit::SplitShares);
+  s0.setAction(eMyMoney::Split::InvestmentTransactionType::SplitShares);
   s0.setShares(sharesEdit->value().abs());
-  s0.setValue(MyMoneyMoney(0, 1));
-  s0.setPrice(MyMoneyMoney(0, 1));
+  s0.setValue(MyMoneyMoney());
+  s0.setPrice(MyMoneyMoney());
 
   feeSplits.clear();
   interestSplits.clear();
@@ -689,35 +864,26 @@ bool Split::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySp
   return true;
 }
 
-void IntInc::showWidgets(void) const
+IntInc::IntInc(InvestTransactionEditor* editor) :
+  Activity(editor)
 {
-  KMyMoneyCategory* cat;
+}
 
-  cat = dynamic_cast<KMyMoneyCategory*>(haveWidget("interest-account"));
-  cat->show();
-  cat->splitButton()->show();
+IntInc::~IntInc()
+{
+}
 
-  cat = dynamic_cast<KMyMoneyCategory*>(haveWidget("fee-account"));
-  cat->show();
-  cat->splitButton()->show();
-
-  QStringList widgets;
-  widgets << "asset-account" << "interest-amount" << "total";
-  QStringList::const_iterator it_w;
-  for (it_w = widgets.constBegin(); it_w != widgets.constEnd(); ++it_w) {
-    QWidget* w = haveWidget(*it_w);
-    if (w)
-      w->show();
-  }
-
-  widgets.clear();
-  widgets << "shares" << "price" << "fee-amount";
-  for (it_w = widgets.constBegin(); it_w != widgets.constEnd(); ++it_w) {
-    QWidget* w = haveWidget(*it_w);
-    if (w)
-      w->hide();
-  }
+eMyMoney::Split::InvestmentTransactionType IntInc::type() const
+{
+  return eMyMoney::Split::InvestmentTransactionType::InterestIncome;
+}
 
+void IntInc::showWidgets() const
+{
+  static const QStringList visibleWidgetIds = QStringList() << "asset-account" << "interest-amount" << "total" << "interest-account" << "fee-account";
+  setWidgetVisibility(visibleWidgetIds, true);
+  static const QStringList hiddenWidgetIds = QStringList() << "shares" << "price" << "fee-amount";
+  setWidgetVisibility(hiddenWidgetIds, false);
 
   setLabelText("interest-amount-label", i18n("Interest"));
   setLabelText("interest-label", i18n("Interest"));
@@ -730,7 +896,7 @@ bool IntInc::isComplete(QString& reason) const
 {
   Q_UNUSED(reason)
 
-  bool rc = Activity::isComplete(reason);
+  auto rc = Activity::isComplete(reason);
   rc &= haveAssetAccount();
   rc &= haveCategoryAndAmount("interest-account", QString(), false);
   rc &= haveInterest(false);
@@ -739,6 +905,7 @@ bool IntInc::isComplete(QString& reason) const
 
 bool IntInc::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySplit& assetAccountSplit, QList<MyMoneySplit>& feeSplits, QList<MyMoneySplit>& m_feeSplits, QList<MyMoneySplit>& interestSplits, QList<MyMoneySplit>& m_interestSplits, MyMoneySecurity& security, MyMoneySecurity& currency)
 {
+  Q_D(Activity);
   Q_UNUSED(security);
   Q_UNUSED(currency);
 
@@ -746,17 +913,17 @@ bool IntInc::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneyS
   if (!isComplete(reason))
     return false;
 
-  s0.setAction(MyMoneySplit::InterestIncome);
+  s0.setAction(eMyMoney::Split::InvestmentTransactionType::InterestIncome);
 
   // for dividends, we only use the stock split as a marker
   MyMoneyMoney shares;
   s0.setShares(shares);
   s0.setValue(shares);
-  s0.setPrice(MyMoneyMoney(1, 1));
+  s0.setPrice(MyMoneyMoney::ONE);
 
-  if (!createCategorySplits(t, dynamic_cast<KMyMoneyCategory*>(haveWidget("fee-account")), dynamic_cast<kMyMoneyEdit*>(haveWidget("fee-amount")), MyMoneyMoney(1, 1), feeSplits, m_feeSplits))
+  if (!createCategorySplits(t, dynamic_cast<KMyMoneyCategory*>(haveWidget("fee-account")), dynamic_cast<KMyMoneyEdit*>(haveWidget("fee-amount")), MyMoneyMoney::ONE, feeSplits, m_feeSplits))
     return false;
-  if (!createCategorySplits(t, dynamic_cast<KMyMoneyCategory*>(haveWidget("interest-account")), dynamic_cast<kMyMoneyEdit*>(haveWidget("interest-amount")), MyMoneyMoney(-1, 1), interestSplits, m_interestSplits))
+  if (!createCategorySplits(t, dynamic_cast<KMyMoneyCategory*>(haveWidget("interest-account")), dynamic_cast<KMyMoneyEdit*>(haveWidget("interest-amount")), MyMoneyMoney::MINUS_ONE, interestSplits, m_interestSplits))
     return false;
 
   createAssetAccountSplit(assetAccountSplit, s0);
@@ -764,10 +931,8 @@ bool IntInc::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneyS
   MyMoneyMoney total = sumSplits(s0, feeSplits, interestSplits);
   assetAccountSplit.setValue(-total);
 
-  if (!m_parent->setupPrice(t, assetAccountSplit))
+  if (!d->m_parent->setupPrice(t, assetAccountSplit))
     return false;
 
   return true;
 }
-
-
