@@ -4,6 +4,7 @@
     begin                : Mon Apr  9 2007
     copyright            : (C) 2007 by Thomas Baumgart
     email                : Thomas Baumgart <ipwizard@users.sourceforge.net>
+                           (C) 2017 by Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
  ***************************************************************************/
 
 /***************************************************************************
@@ -26,50 +27,53 @@
 // ----------------------------------------------------------------------------
 // KDE Includes
 
-#include <klocale.h>
-#include <ktextedit.h>
-#include <kpushbutton.h>
-#include <kmessagebox.h>
+#include <KTextEdit>
+#include <KMessageBox>
+#include <KLocalizedString>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include <mymoneyfile.h>
-#include <kmymoneyutils.h>
-#include <mymoneytransaction.h>
-#include "ui_kconfirmmanualenterdlgdecl.h"
+#include "ui_kconfirmmanualenterdlg.h"
 
-struct KConfirmManualEnterDlg::Private {
-  Ui::KConfirmManualEnterDlgDecl ui;
-};
+#include "mymoneymoney.h"
+#include "mymoneyfile.h"
+#include "mymoneyaccount.h"
+#include "mymoneysecurity.h"
+#include "mymoneypayee.h"
+#include "mymoneysplit.h"
+#include "mymoneyschedule.h"
+#include "mymoneyexception.h"
+#include "kmymoneyutils.h"
+#include "mymoneytransaction.h"
+#include "mymoneyenums.h"
 
 KConfirmManualEnterDlg::KConfirmManualEnterDlg(const MyMoneySchedule& schedule, QWidget* parent) :
-    QDialog(parent), d(new Private)
+    QDialog(parent),
+    ui(new Ui::KConfirmManualEnterDlg)
 {
-  d->ui.setupUi(this);
-  d->ui.buttonGroup1->setId(d->ui.m_discardRadio, 0);
-  d->ui.buttonGroup1->setId(d->ui.m_onceRadio, 1);
-  d->ui.buttonGroup1->setId(d->ui.m_setRadio, 2);
+  ui->setupUi(this);
+  ui->buttonGroup1->setId(ui->m_discardRadio, 0);
+  ui->buttonGroup1->setId(ui->m_onceRadio, 1);
+  ui->buttonGroup1->setId(ui->m_setRadio, 2);
 
-  d->ui.buttonOk->setGuiItem(KStandardGuiItem::ok());
-  d->ui.buttonCancel->setGuiItem(KStandardGuiItem::cancel());
-  d->ui.m_onceRadio->setChecked(true);
+  ui->m_onceRadio->setChecked(true);
 
-  if (schedule.type() == MyMoneySchedule::TYPE_LOANPAYMENT) {
-    d->ui.m_setRadio->setEnabled(false);
-    d->ui.m_discardRadio->setEnabled(false);
+  if (schedule.type() == eMyMoney::Schedule::Type::LoanPayment) {
+    ui->m_setRadio->setEnabled(false);
+    ui->m_discardRadio->setEnabled(false);
   }
 }
 
 KConfirmManualEnterDlg::~KConfirmManualEnterDlg()
 {
-  delete d;
+  delete ui;
 }
 
 void KConfirmManualEnterDlg::loadTransactions(const MyMoneyTransaction& to, const MyMoneyTransaction& tn)
 {
   QString messageDetail("<qt>");
-  MyMoneyFile* file = MyMoneyFile::instance();
+  auto file = MyMoneyFile::instance();
   int noItemsChanged = 0;
 
   try {
@@ -142,6 +146,18 @@ void KConfirmManualEnterDlg::loadTransactions(const MyMoneyTransaction& to, cons
       messageDetail += i18n("<p>Memo changed.<br/>&nbsp;&nbsp;&nbsp;Old: <b>%1</b>, New: <b>%2</b></p>", mo, mn);
     }
 
+    QString no, nn;
+    no = to.splits().front().number();
+    nn = tn.splits().front().number();
+    if (no.isEmpty())
+      no = QString("<i>") + i18nc("No number", "empty") + QString("</i>");
+    if (nn.isEmpty())
+      nn = QString("<i>") + i18nc("No number", "empty") + QString("</i>");
+    if (no != nn) {
+      noItemsChanged++;
+      messageDetail += i18n("<p>Number changed.<br/>&nbsp;&nbsp;&nbsp;Old: <b>%1</b>, New: <b>%2</b></p>", no, nn);
+    }
+
     const MyMoneySecurity& sec = MyMoneyFile::instance()->security(to.commodity());
     MyMoneyMoney ao, an;
     ao = to.splits().front().value();
@@ -151,7 +167,7 @@ void KConfirmManualEnterDlg::loadTransactions(const MyMoneyTransaction& to, cons
       messageDetail += i18n("<p>Amount changed.<br/>&nbsp;&nbsp;&nbsp;Old: <b>%1</b>, New: <b>%2</b></p>", ao.formatMoney(sec.smallestAccountFraction()), an.formatMoney(sec.smallestAccountFraction()));
     }
 
-    MyMoneySplit::reconcileFlagE fo, fn;
+    eMyMoney::Split::State fo, fn;
     fo = to.splits().front().reconcileFlag();
     fn = tn.splits().front().reconcileFlag();
     if (fo != fn) {
@@ -163,15 +179,15 @@ void KConfirmManualEnterDlg::loadTransactions(const MyMoneyTransaction& to, cons
   }
 
   messageDetail += "</qt>";
-  d->ui.m_details->setText(messageDetail);
+  ui->m_details->setText(messageDetail);
   return;
 }
 
-KConfirmManualEnterDlg::Action KConfirmManualEnterDlg::action(void) const
+KConfirmManualEnterDlg::Action KConfirmManualEnterDlg::action() const
 {
-  if (d->ui.m_discardRadio->isChecked())
+  if (ui->m_discardRadio->isChecked())
     return UseOriginal;
-  if (d->ui.m_setRadio->isChecked())
+  if (ui->m_setRadio->isChecked())
     return ModifyAlways;
   return ModifyOnce;
 }
