@@ -4,6 +4,7 @@
     begin                : Sat Apr  7 2007
     copyright            : (C) 2007 by Thomas Baumgart
     email                : Thomas Baumgart <ipwizard@users.sourceforge.net>
+                           (C) 2017 by Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
  ***************************************************************************/
 
 /***************************************************************************
@@ -23,131 +24,164 @@
 #include <QTimer>
 #include <QWidget>
 #include <QLabel>
-#include <QResizeEvent>
+#include <QPushButton>
+#include <QIcon>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
 
-#include <klocale.h>
-#include <kmessagebox.h>
-#include <kpushbutton.h>
-#include <kiconloader.h>
+#include <KMessageBox>
+#include <KHelpClient>
+#include <KLocalizedString>
+#include <KGuiItem>
+#include <KStandardGuiItem>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include "kcurrencycalculator.h"
-#include <register.h>
-#include <transactionform.h>
-#include <transaction.h>
-#include <transactioneditor.h>
-#include <kmymoneyutils.h>
-#include <mymoneyfinancialcalculator.h>
-#include <kmymoneylineedit.h>
-#include <kmymoneycategory.h>
-#include <kmymoneyaccountselector.h>
-#include <kmymoneydateinput.h>
-#include <ktoolinvocation.h>
-#include <kmymoneyglobalsettings.h>
+#include "ui_kenterscheduledlg.h"
 
-#include "kmymoney.h"
+#include "tabbar.h"
+#include "mymoneysplit.h"
+#include "mymoneytransaction.h"
+#include "mymoneyfile.h"
+#include "mymoneyaccount.h"
+#include "mymoneymoney.h"
+#include "mymoneyschedule.h"
+#include "register.h"
+#include "transactionform.h"
+#include "transaction.h"
+#include "selectedtransactions.h"
+#include "transactioneditor.h"
+#include "kmymoneyutils.h"
+#include "kmymoneylineedit.h"
+#include "kmymoneydateinput.h"
+#include "knewaccountdlg.h"
+#include "knewinvestmentwizard.h"
+#include "mymoneyexception.h"
+#include "icons/icons.h"
+#include "mymoneyenums.h"
+#include "dialogenums.h"
+#include "widgetenums.h"
 
-class KEnterScheduleDlg::Private
+using namespace Icons;
+
+class KEnterScheduleDlgPrivate
 {
-public:
-  Private() : m_item(0), m_showWarningOnce(true) {}
-  ~Private() {}
+  Q_DISABLE_COPY(KEnterScheduleDlgPrivate)
 
+public:
+  KEnterScheduleDlgPrivate() :
+    ui(new Ui::KEnterScheduleDlg),
+    m_item(nullptr),
+    m_showWarningOnce(true)
+  {
+  }
+
+  ~KEnterScheduleDlgPrivate()
+  {
+    delete ui;
+  }
+  
+  Ui::KEnterScheduleDlg         *ui;
   MyMoneySchedule                m_schedule;
   KMyMoneyRegister::Transaction* m_item;
   QWidgetList                    m_tabOrderWidgets;
   bool                           m_showWarningOnce;
-  KMyMoneyUtils::EnterScheduleResultCodeE m_extendedReturnCode;
+  eDialogs::ScheduleResultCode m_extendedReturnCode;
 };
 
 KEnterScheduleDlg::KEnterScheduleDlg(QWidget *parent, const MyMoneySchedule& schedule) :
-    KEnterScheduleDlgDecl(parent),
-    d(new Private)
+  QDialog(parent),
+  d_ptr(new KEnterScheduleDlgPrivate)
 {
+  Q_D(KEnterScheduleDlg);
+  d->ui->setupUi(this);
   d->m_schedule = schedule;
-  d->m_extendedReturnCode = KMyMoneyUtils::Enter;
-  buttonOk->setIcon(KIcon(KMyMoneyGlobalSettings::enterScheduleIcon()));
-  buttonSkip->setIcon(KIcon("media-seek-forward"));
-  buttonCancel->setGuiItem(KStandardGuiItem::cancel());
-  buttonHelp->setGuiItem(KStandardGuiItem::help());
-  buttonIgnore->setHidden(true);
-  buttonSkip->setHidden(true);
+  d->m_extendedReturnCode = eDialogs::ScheduleResultCode::Enter;
+  d->ui->buttonOk->setIcon(Icons::get(Icon::KeyEnter));
+  d->ui->buttonSkip->setIcon(Icons::get(Icon::MediaSeekForward));
+  KGuiItem::assign(d->ui->buttonCancel, KStandardGuiItem::cancel());
+  KGuiItem::assign(d->ui->buttonHelp, KStandardGuiItem::help());
+  d->ui->buttonIgnore->setHidden(true);
+  d->ui->buttonSkip->setHidden(true);
 
   // make sure, we have a tabbar with the form
-  KMyMoneyTransactionForm::TabBar* tabbar = m_form->tabBar(m_form->parentWidget());
+  KMyMoneyTransactionForm::TabBar* tabbar = d->ui->m_form->getTabBar(d->ui->m_form->parentWidget());
 
   // we never need to see the register
-  m_register->hide();
+  d->ui->m_register->hide();
 
   // ... setup the form ...
-  m_form->setupForm(d->m_schedule.account());
+  d->ui->m_form->setupForm(d->m_schedule.account());
 
   // ... and the register ...
-  m_register->clear();
+  d->ui->m_register->clear();
 
   // ... now add the transaction to register and form ...
   MyMoneyTransaction t = transaction();
-  d->m_item = KMyMoneyRegister::Register::transactionFactory(m_register, t,
+  d->m_item = KMyMoneyRegister::Register::transactionFactory(d->ui->m_register, t,
               d->m_schedule.transaction().splits().isEmpty() ? MyMoneySplit() : d->m_schedule.transaction().splits().front(), 0);
-  m_register->selectItem(d->m_item);
+  d->ui->m_register->selectItem(d->m_item);
   // show the account row
   d->m_item->setShowRowInForm(0, true);
 
-  m_form->slotSetTransaction(d->m_item);
+  d->ui->m_form->slotSetTransaction(d->m_item);
 
   // no need to see the tabbar
   tabbar->hide();
 
   // setup name and type
-  m_scheduleName->setText(d->m_schedule.name());
-  m_type->setText(KMyMoneyUtils::scheduleTypeToString(d->m_schedule.type()));
+  d->ui->m_scheduleName->setText(d->m_schedule.name());
+  d->ui->m_type->setText(KMyMoneyUtils::scheduleTypeToString(d->m_schedule.type()));
 
-  connect(buttonHelp, SIGNAL(clicked()), this, SLOT(slotShowHelp()));
-  connect(buttonIgnore, SIGNAL(clicked()), this, SLOT(slotIgnore()));
-  connect(buttonSkip, SIGNAL(clicked()), this, SLOT(slotSkip()));
+  connect(d->ui->buttonHelp, &QAbstractButton::clicked, this, &KEnterScheduleDlg::slotShowHelp);
+  connect(d->ui->buttonIgnore, &QAbstractButton::clicked, this, &KEnterScheduleDlg::slotIgnore);
+  connect(d->ui->buttonSkip, &QAbstractButton::clicked, this, &KEnterScheduleDlg::slotSkip);
 }
 
 KEnterScheduleDlg::~KEnterScheduleDlg()
 {
+  Q_D(KEnterScheduleDlg);
   delete d;
 }
 
-KMyMoneyUtils::EnterScheduleResultCodeE KEnterScheduleDlg::resultCode(void) const
+eDialogs::ScheduleResultCode KEnterScheduleDlg::resultCode() const
 {
+  Q_D(const KEnterScheduleDlg);
   if (result() == QDialog::Accepted)
     return d->m_extendedReturnCode;
-  return KMyMoneyUtils::Cancel;
+  return eDialogs::ScheduleResultCode::Cancel;
 }
 
 void KEnterScheduleDlg::showExtendedKeys(bool visible)
 {
-  buttonIgnore->setVisible(visible);
-  buttonSkip->setVisible(visible);
+  Q_D(KEnterScheduleDlg);
+  d->ui->buttonIgnore->setVisible(visible);
+  d->ui->buttonSkip->setVisible(visible);
 }
 
-void KEnterScheduleDlg::slotIgnore(void)
+void KEnterScheduleDlg::slotIgnore()
 {
-  d->m_extendedReturnCode = KMyMoneyUtils::Ignore;
+  Q_D(KEnterScheduleDlg);
+  d->m_extendedReturnCode = eDialogs::ScheduleResultCode::Ignore;
   accept();
 }
 
-void KEnterScheduleDlg::slotSkip(void)
+void KEnterScheduleDlg::slotSkip()
 {
-  d->m_extendedReturnCode = KMyMoneyUtils::Skip;
+  Q_D(KEnterScheduleDlg);
+  d->m_extendedReturnCode = eDialogs::ScheduleResultCode::Skip;
   accept();
 }
 
-MyMoneyTransaction KEnterScheduleDlg::transaction(void)
+MyMoneyTransaction KEnterScheduleDlg::transaction()
 {
-  MyMoneyTransaction t = d->m_schedule.transaction();
+  Q_D(KEnterScheduleDlg);
+  auto t = d->m_schedule.transaction();
 
   try {
-    if (d->m_schedule.type() == MyMoneySchedule::TYPE_LOANPAYMENT) {
+    if (d->m_schedule.type() == eMyMoney::Schedule::Type::LoanPayment) {
       KMyMoneyUtils::calculateAutoLoan(d->m_schedule, t, QMap<QString, MyMoneyMoney>());
     }
   } catch (const MyMoneyException &e) {
@@ -161,25 +195,28 @@ MyMoneyTransaction KEnterScheduleDlg::transaction(void)
 
 QDate KEnterScheduleDlg::date(const QDate& _date) const
 {
-  QDate date(_date);
+  Q_D(const KEnterScheduleDlg);
+  auto date(_date);
   return d->m_schedule.adjustedDate(date, d->m_schedule.weekendOption());
 }
 
 void KEnterScheduleDlg::resizeEvent(QResizeEvent* ev)
 {
-  m_register->resize(KMyMoneyRegister::DetailColumn);
-  m_form->resize(KMyMoneyTransactionForm::ValueColumn1);
-  KEnterScheduleDlgDecl::resizeEvent(ev);
+  Q_UNUSED(ev)
+  Q_D(KEnterScheduleDlg);
+  d->ui->m_register->resize((int)eWidgets::eTransaction::Column::Detail);
+  d->ui->m_form->resize((int)eWidgets::eTransactionForm::Column::Value1);
+  QDialog::resizeEvent(ev);
 }
 
-
-void KEnterScheduleDlg::slotSetupSize(void)
+void KEnterScheduleDlg::slotSetupSize()
 {
   resize(width(), minimumSizeHint().height());
 }
 
-int KEnterScheduleDlg::exec(void)
+int KEnterScheduleDlg::exec()
 {
+  Q_D(KEnterScheduleDlg);
   if (d->m_showWarningOnce) {
     d->m_showWarningOnce = false;
     KMessageBox::information(this, QString("<qt>") + i18n("<p>Please check that all the details in the following dialog are correct and press OK.</p><p>Editable data can be changed and can either be applied to just this occurrence or for all subsequent occurrences for this schedule.  (You will be asked what you intend after pressing OK in the following dialog)</p>") + QString("</qt>"), i18n("Enter scheduled transaction"), "EnterScheduleDlgInfo");
@@ -187,14 +224,16 @@ int KEnterScheduleDlg::exec(void)
 
   // force the initial height to be as small as possible
   QTimer::singleShot(0, this, SLOT(slotSetupSize()));
-
-  return KEnterScheduleDlgDecl::exec();
+  return QDialog::exec();
 }
 
-TransactionEditor* KEnterScheduleDlg::startEdit(void)
+TransactionEditor* KEnterScheduleDlg::startEdit()
 {
-  KMyMoneyRegister::SelectedTransactions list(m_register);
-  TransactionEditor* editor = d->m_item->createEditor(m_form, list, QDate());
+  Q_D(KEnterScheduleDlg);
+  KMyMoneyRegister::SelectedTransactions list(d->ui->m_register);
+  TransactionEditor* editor = d->m_item->createEditor(d->ui->m_form, list, QDate());
+  editor->setScheduleInfo(d->m_schedule.name());
+  editor->setPaymentMethod(d->m_schedule.paymentType());
 
   // check that we use the same transaction commodity in all selected transactions
   // if not, we need to update this in the editor's list. The user can also bail out
@@ -209,34 +248,30 @@ TransactionEditor* KEnterScheduleDlg::startEdit(void)
   }
 
   if (editor) {
-    connect(editor, SIGNAL(transactionDataSufficient(bool)), buttonOk, SLOT(setEnabled(bool)));
-    connect(editor, SIGNAL(escapePressed()), buttonCancel, SLOT(animateClick()));
-    connect(editor, SIGNAL(returnPressed()), buttonOk, SLOT(animateClick()));
+    connect(editor, &TransactionEditor::transactionDataSufficient, d->ui->buttonOk, &QWidget::setEnabled);
+    connect(editor, &TransactionEditor::escapePressed, d->ui->buttonCancel, &QAbstractButton::animateClick);
+    connect(editor, &TransactionEditor::returnPressed, d->ui->buttonOk, &QAbstractButton::animateClick);
 
-    connect(MyMoneyFile::instance(), SIGNAL(dataChanged()), editor, SLOT(slotReloadEditWidgets()));
+    connect(MyMoneyFile::instance(), &MyMoneyFile::dataChanged, editor, &TransactionEditor::slotReloadEditWidgets);
     // connect(editor, SIGNAL(finishEdit(KMyMoneyRegister::SelectedTransactions)), this, SLOT(slotLeaveEditMode(KMyMoneyRegister::SelectedTransactions)));
-    connect(editor, SIGNAL(createPayee(QString,QString&)), kmymoney, SLOT(slotPayeeNew(QString,QString&)));
-    connect(editor, SIGNAL(createTag(QString,QString&)), kmymoney, SLOT(slotTagNew(QString,QString&)));
-    connect(editor, SIGNAL(createCategory(MyMoneyAccount&,MyMoneyAccount)), kmymoney, SLOT(slotCategoryNew(MyMoneyAccount&,MyMoneyAccount)));
-    connect(editor, SIGNAL(createSecurity(MyMoneyAccount&,MyMoneyAccount)), kmymoney, SLOT(slotInvestmentNew(MyMoneyAccount&,MyMoneyAccount)));
-    connect(MyMoneyFile::instance(), SIGNAL(dataChanged()), editor, SLOT(slotReloadEditWidgets()));
+    connect(MyMoneyFile::instance(), &MyMoneyFile::dataChanged, editor, &TransactionEditor::slotReloadEditWidgets);
 
     // create the widgets, place them in the parent and load them with data
     // setup tab order
     d->m_tabOrderWidgets.clear();
-    KMyMoneyRegister::Action action = KMyMoneyRegister::ActionWithdrawal;
+    eWidgets::eRegister::Action action = eWidgets::eRegister::Action::Withdrawal;
     switch (d->m_schedule.type()) {
-      case MyMoneySchedule::TYPE_TRANSFER:
-        action = KMyMoneyRegister::ActionTransfer;
+      case eMyMoney::Schedule::Type::Transfer:
+        action = eWidgets::eRegister::Action::Transfer;
         break;
-      case MyMoneySchedule::TYPE_DEPOSIT:
-        action = KMyMoneyRegister::ActionDeposit;
+      case eMyMoney::Schedule::Type::Deposit:
+        action = eWidgets::eRegister::Action::Deposit;
         break;
-      case MyMoneySchedule::TYPE_LOANPAYMENT:
+      case eMyMoney::Schedule::Type::LoanPayment:
         switch (d->m_schedule.paymentType()) {
-          case MyMoneySchedule::STYPE_DIRECTDEPOSIT:
-          case MyMoneySchedule::STYPE_MANUALDEPOSIT:
-            action = KMyMoneyRegister::ActionDeposit;
+          case eMyMoney::Schedule::PaymentType::DirectDeposit:
+          case eMyMoney::Schedule::PaymentType::ManualDeposit:
+            action = eWidgets::eRegister::Action::Deposit;
             break;
           default:
             break;
@@ -247,19 +282,33 @@ TransactionEditor* KEnterScheduleDlg::startEdit(void)
     }
     editor->setup(d->m_tabOrderWidgets, d->m_schedule.account(), action);
 
-    // if it's not a check, then we need to clear
-    // a possibly assigned check number
-    if (d->m_schedule.paymentType() != MyMoneySchedule::STYPE_WRITECHEQUE) {
-      QWidget* w = editor->haveWidget("number");
+    MyMoneyTransaction t = d->m_schedule.transaction();
+    QString num = t.splits().first().number();
+    QWidget* w = editor->haveWidget("number");
+    if (d->m_schedule.paymentType() == eMyMoney::Schedule::PaymentType::WriteChecque) {
+      auto file = MyMoneyFile::instance();
+      if (file->checkNoUsed(d->m_schedule.account().id(), num)) {
+        //  increment and try again
+        num = KMyMoneyUtils::getAdjacentNumber(num);
+      }
+      num = KMyMoneyUtils::nextCheckNumber(d->m_schedule.account());
+      KMyMoneyUtils::updateLastNumberUsed(d->m_schedule.account(), num);
+      d->m_schedule.account().setValue("lastNumberUsed", num);
+      if (w) {
+        dynamic_cast<KMyMoneyLineEdit*>(w)->loadText(num);
+      }
+    } else {
+      // if it's not a check, then we need to clear
+      // a possibly assigned check number
       if (w)
-        dynamic_cast<kMyMoneyLineEdit*>(w)->loadText(QString());
+        dynamic_cast<KMyMoneyLineEdit*>(w)->loadText(QString());
     }
 
     Q_ASSERT(!d->m_tabOrderWidgets.isEmpty());
 
     // editor->setup() leaves the tabbar as the last widget in the stack, but we
     // need it as first here. So we move it around.
-    QWidget* w = editor->haveWidget("tabbar");
+    w = editor->haveWidget("tabbar");
     if (w) {
       int idx = d->m_tabOrderWidgets.indexOf(w);
       if (idx != -1) {
@@ -269,11 +318,11 @@ TransactionEditor* KEnterScheduleDlg::startEdit(void)
     }
 
     // don't forget our three buttons
-    d->m_tabOrderWidgets.append(buttonOk);
-    d->m_tabOrderWidgets.append(buttonCancel);
-    d->m_tabOrderWidgets.append(buttonHelp);
+    d->m_tabOrderWidgets.append(d->ui->buttonOk);
+    d->m_tabOrderWidgets.append(d->ui->buttonCancel);
+    d->m_tabOrderWidgets.append(d->ui->buttonHelp);
 
-    for (int i = 0; i < d->m_tabOrderWidgets.size(); ++i) {
+    for (auto i = 0; i < d->m_tabOrderWidgets.size(); ++i) {
       QWidget* w = d->m_tabOrderWidgets.at(i);
       if (w) {
         w->installEventFilter(this);
@@ -288,7 +337,7 @@ TransactionEditor* KEnterScheduleDlg::startEdit(void)
     focusWidget->setFocus();
 
     // Make sure, we use the adjusted date
-    kMyMoneyDateInput* dateEdit = dynamic_cast<kMyMoneyDateInput*>(editor->haveWidget("postdate"));
+    KMyMoneyDateInput* dateEdit = dynamic_cast<KMyMoneyDateInput*>(editor->haveWidget("postdate"));
     if (dateEdit) {
       dateEdit->setDate(d->m_schedule.adjustedNextDueDate());
     }
@@ -299,10 +348,10 @@ TransactionEditor* KEnterScheduleDlg::startEdit(void)
 
 bool KEnterScheduleDlg::focusNextPrevChild(bool next)
 {
-  bool rc = false;
-  QWidget *w = 0;
-
-  w = qApp->focusWidget();
+  Q_D(KEnterScheduleDlg);
+  auto rc = false;
+  
+  auto w = qApp->focusWidget();
   int currentWidgetIndex = d->m_tabOrderWidgets.indexOf(w);
   while (w && currentWidgetIndex == -1) {
     // qDebug("'%s' not in list, use parent", w->className());
@@ -330,7 +379,7 @@ bool KEnterScheduleDlg::focusNextPrevChild(bool next)
   return rc;
 }
 
-void KEnterScheduleDlg::slotShowHelp(void)
+void KEnterScheduleDlg::slotShowHelp()
 {
-  KToolInvocation::invokeHelp("details.schedules.entering");
+  KHelpClient::invokeHelp("details.schedules.entering");
 }
