@@ -3,6 +3,7 @@
                              -------------------
     copyright            : (C) 2008 by Thomas Baumgart
     email                : ipwizard@users.sourceforge.net
+                           (C) 2017 by Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
  ***************************************************************************/
 
 /***************************************************************************
@@ -19,32 +20,37 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
+#include <QList>
+
 // ----------------------------------------------------------------------------
 // KDE Includes
 
-#include <kpushbutton.h>
-
 // ----------------------------------------------------------------------------
 // Project Includes
 
+#include "ui_kloadtemplatedlg.h"
+
+#include "mymoneytemplate.h"
 #include "kaccounttemplateselector.h"
-#include <QList>
 
 KLoadTemplateDlg::KLoadTemplateDlg(QWidget* parent) :
-    KLoadTemplateDlgDecl(parent)
+  QDialog(parent),
+  ui(new Ui::KLoadTemplateDlg)
 {
-  buttonOk->setGuiItem(KStandardGuiItem::ok());
-  buttonCancel->setGuiItem(KStandardGuiItem::cancel());
-  buttonHelp->setGuiItem(KStandardGuiItem::help());
-
-  connect(buttonHelp, SIGNAL(clicked()), this, SLOT(slotHelp()));
+  ui->setupUi(this);
+  connect(ui->buttonBox, &QDialogButtonBox::helpRequested, this, &KLoadTemplateDlg::slotHelp);
 }
 
-QList<MyMoneyTemplate> KLoadTemplateDlg::templates(void) const
+KLoadTemplateDlg::~KLoadTemplateDlg()
 {
-  return m_templateSelector->selectedTemplates();
+  delete ui;
 }
 
-void KLoadTemplateDlg::slotHelp(void)
+QList<MyMoneyTemplate> KLoadTemplateDlg::templates() const
+{
+  return ui->m_templateSelector->selectedTemplates();
+}
+
+void KLoadTemplateDlg::slotHelp()
 {
 }
