@@ -10,6 +10,7 @@
                            Thomas Baumgart <ipwizard@users.sourceforge.net>
                            Kevin Tambascio <ktambascio@users.sourceforge.net>
                            Ace Jones <ace.j@hotpop.com>
+                           (C) 2017 by Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
  ***************************************************************************/
 
 /***************************************************************************
@@ -27,8 +28,6 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
-#include <QVector>
-
 // ----------------------------------------------------------------------------
 // KDE Includes
 
@@ -36,57 +35,36 @@
 // Project Includes
 
 #include "kfindtransactiondlg.h"
-#include "mymoneyreport.h"
-#include "mymoneybudget.h"
-#include "kmymoneyreportconfigtabimpl.h"
 
-
-class kMyMoneyReportConfigTab1Decl;
-class kMyMoneyReportConfigTab2Decl;
-class kMyMoneyReportConfigTab3Decl;
-class kMyMoneyReportConfigTabChartDecl;
-class MyMoneyBudget;
+class MyMoneyReport;
 
 /**
   * @author Ace Jones
   */
+class KReportConfigurationFilterDlgPrivate;
 class KReportConfigurationFilterDlg : public KFindTransactionDlg
 {
   Q_OBJECT
+  Q_DISABLE_COPY(KReportConfigurationFilterDlg)
+
 public:
-  explicit KReportConfigurationFilterDlg(MyMoneyReport report, QWidget *parent = 0);
+  explicit KReportConfigurationFilterDlg(MyMoneyReport report, QWidget *parent = nullptr);
   ~KReportConfigurationFilterDlg();
 
-  const MyMoneyReport& getConfig() const {
-    return m_currentState;
-  }
+  MyMoneyReport getConfig() const;
 
-protected:
-  kMyMoneyReportConfigTab1Decl* m_tab1;
-  kMyMoneyReportConfigTab2Decl* m_tab2;
-  kMyMoneyReportConfigTab3Decl* m_tab3;
-  kMyMoneyReportConfigTabChartDecl* m_tabChart;
-
-  MyMoneyReport m_initialState;
-  MyMoneyReport m_currentState;
-
-protected slots:
+protected Q_SLOTS:
   void slotRowTypeChanged(int);
   void slotColumnTypeChanged(int);
   void slotReset();
   void slotSearch();
   void slotShowHelp();
-  /**
-   * This is to enable/disable the check to Include Transfers based on whether Categories are filtered or not.
-   * This is because if Categories are filtered, transfers will not be included anyway
-   */
-  virtual void slotDateChanged();
-
   void slotUpdateCheckTransfers();
   void slotUpdateColumnsCombo();
+  void slotUpdateColumnsCombo(int idx);
+  void slotLogAxisChanged(int state);
 
 private:
-  QVector<MyMoneyBudget> m_budgets;
+    Q_DECLARE_PRIVATE(KReportConfigurationFilterDlg)
 };
-
 #endif
