@@ -9,6 +9,7 @@
                            John C <thetacoturtle@users.sourceforge.net>
                            Thomas Baumgart <ipwizard@users.sourceforge.net>
                            Kevin Tambascio <ktambascio@users.sourceforge.net>
+                           (C) 2017 by Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
  ***************************************************************************/
 
 /***************************************************************************
@@ -26,59 +27,54 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
+#include <QDialog>
+
 // ----------------------------------------------------------------------------
 // KDE Includes
 
-#include <kmenu.h>
-#include <kiconloader.h>
-
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include <mymoneymoney.h>
-#include <mymoneyaccount.h>
-#include <mymoneytransaction.h>
+class MyMoneyMoney;
+class MyMoneySplit;
+class MyMoneyTransaction;
+class MyMoneyAccount;
 
+namespace Ui { class KSplitCorrectionDlg; }
 
-#include "ui_ksplittransactiondlgdecl.h"
-#include "ui_ksplitcorrectiondlg.h"
-
-
-
-class KSplitCorrectionDlgDecl : public KDialog, public Ui::KSplitCorrectionDlgDecl
+class KSplitCorrectionDlg : public QDialog
 {
+  Q_OBJECT
+  Q_DISABLE_COPY(KSplitCorrectionDlg)
+
 public:
-  KSplitCorrectionDlgDecl(QWidget *parent) : KDialog(parent) {
-    setupUi(this);
-  }
+  explicit KSplitCorrectionDlg(QWidget *parent = nullptr);
+  ~KSplitCorrectionDlg();
+
+  Ui::KSplitCorrectionDlg *ui;
 };
 
 /**
   * @author Thomas Baumgart
   */
 
-class KSplitTransactionDlgDecl : public KDialog, public Ui::KSplitTransactionDlgDecl
-{
-public:
-  KSplitTransactionDlgDecl(QWidget *parent) : KDialog(parent) {
-    setupUi(this);
-  }
-};
-class KSplitTransactionDlg : public KSplitTransactionDlgDecl
+class KSplitTransactionDlgPrivate;
+class KSplitTransactionDlg : public QDialog
 {
   Q_OBJECT
+  Q_DISABLE_COPY(KSplitTransactionDlg)
 
 public:
-  KSplitTransactionDlg(const MyMoneyTransaction& t,
-                       const MyMoneySplit& s,
-                       const MyMoneyAccount& acc,
-                       const bool amountValid,
-                       const bool deposit,
-                       const MyMoneyMoney& calculatedValue,
-                       const QMap<QString, MyMoneyMoney>& priceInfo,
-                       QWidget* parent = 0);
+  explicit KSplitTransactionDlg(const MyMoneyTransaction& t,
+                                const MyMoneySplit& s,
+                                const MyMoneyAccount& acc,
+                                const bool amountValid,
+                                const bool deposit,
+                                const MyMoneyMoney& calculatedValue,
+                                const QMap<QString, MyMoneyMoney>& priceInfo,
+                                QWidget* parent = nullptr);
 
-  virtual ~KSplitTransactionDlg();
+  ~KSplitTransactionDlg();
 
   /**
     * Using this method, an external object can retrieve the result
@@ -88,9 +84,7 @@ public:
     *         the construction of this object and modified using the
     *         dialog.
     */
-  const MyMoneyTransaction& transaction(void) const {
-    return m_transaction;
-  };
+  MyMoneyTransaction transaction() const;
 
   /**
     * This method calculates the difference between the split that references
@@ -99,7 +93,7 @@ public:
     *
     * @return difference as MyMoneyMoney object
     */
-  MyMoneyMoney diffAmount(void);
+  MyMoneyMoney diffAmount();
 
   /**
     * This method calculates the sum of the splits shown in the register
@@ -107,30 +101,26 @@ public:
     *
     * @return sum of splits as MyMoneyMoney object
     */
-  MyMoneyMoney splitsValue(void);
+  MyMoneyMoney splitsValue();
 
-private:
-  /**
-    * This method updates the display of the sums below the register
-    */
-  void updateSums(void);
+public Q_SLOTS:
+  int exec() override;
 
-public slots:
-  int exec(void);
-
-protected slots:
-  void accept();
-  void reject();
-  void slotClearAllSplits(void);
-  void slotClearUnusedSplits(void);
+protected Q_SLOTS:
+  void accept() override;
+  void reject() override;
+  void slotClearAllSplits();
+  void slotClearUnusedSplits();
   void slotSetTransaction(const MyMoneyTransaction& t);
   void slotCreateCategory(const QString& txt, QString& id);
-  void slotMergeSplits(void);
+  void slotUpdateButtons();
+  void slotMergeSplits();
+  void slotEditStarted();
 
   /// used internally to setup the initial size of all widgets
-  void initSize(void);
+  void initSize();
 
-signals:
+Q_SIGNALS:
   /**
     * This signal is sent out, when a new category needs to be created
     * Depending on the setting of either a payment or deposit, the parent
@@ -150,43 +140,8 @@ signals:
   void objectCreation(bool state);
 
 private:
-  /**
-    * This member keeps a copy of the current selected transaction
-    */
-  MyMoneyTransaction     m_transaction;
-
-  /**
-    * This member keeps a copy of the currently selected account
-    */
-  MyMoneyAccount         m_account;
-
-  /**
-    * This member keeps a copy of the currently selected split
-    */
-  MyMoneySplit           m_split;
-
-  /**
-    * This member keeps the precision for the values
-    */
-  int                    m_precision;
-
-  /**
-    * flag that shows that the amount specified in the constructor
-    * should be used as fix value (true) or if it can be changed (false)
-    */
-  bool                   m_amountValid;
-
-  /**
-    * This member keeps track if the current transaction is of type
-    * deposit (true) or withdrawal (false).
-    */
-  bool                   m_isDeposit;
-
-  /**
-    * This member keeps the amount that will be assigned to all the
-    * splits that are marked 'will be calculated'.
-    */
-  MyMoneyMoney           m_calculatedValue;
+  KSplitTransactionDlgPrivate * const d_ptr;
+  Q_DECLARE_PRIVATE(KSplitTransactionDlg)
 };
 
 #endif
