@@ -3,6 +3,7 @@
                              --------------------
     copyright            : (C) 2005 by Thomas Baumgart
     email                : ipwizard@users.sourceforge.net
+                           (C) 2017 by Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
  ***************************************************************************/
 
 /***************************************************************************
@@ -22,52 +23,56 @@
 // ----------------------------------------------------------------------------
 // KDE Includes
 
-#include <klineedit.h>
+#include <KLineEdit>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
-KSettingsRegister::KSettingsRegister(QWidget* parent) :
-    KSettingsRegisterDecl(parent)
-{
+#include "ui_ksettingsregister.h"
 
+KSettingsRegister::KSettingsRegister(QWidget* parent) :
+  QWidget(parent),
+  ui(new Ui::KSettingsRegister)
+{
+  ui->setupUi(this);
   // hide the internally used text fields
-  kcfg_sortNormalView->hide();
-  kcfg_sortReconcileView->hide();
-  kcfg_sortSearchView->hide();
+  ui->kcfg_sortNormalView->hide();
+  ui->kcfg_sortReconcileView->hide();
+  ui->kcfg_sortSearchView->hide();
 
   // setup connections, so that the sort optios get loaded once the edit fields are filled
-  connect(kcfg_sortNormalView, SIGNAL(textChanged(QString)), this, SLOT(slotLoadNormal(QString)));
-  connect(kcfg_sortReconcileView, SIGNAL(textChanged(QString)), this, SLOT(slotLoadReconcile(QString)));
-  connect(kcfg_sortSearchView, SIGNAL(textChanged(QString)), this, SLOT(slotLoadSearch(QString)));
+  connect(ui->kcfg_sortNormalView, &QLineEdit::textChanged, this, &KSettingsRegister::slotLoadNormal);
+  connect(ui->kcfg_sortReconcileView, &QLineEdit::textChanged, this, &KSettingsRegister::slotLoadReconcile);
+  connect(ui->kcfg_sortSearchView, &QLineEdit::textChanged, this, &KSettingsRegister::slotLoadSearch);
 
   // setup connections, so that changes by the user are forwarded to the (hidden) edit fields
-  connect(m_sortNormalView, SIGNAL(settingsChanged(QString)), kcfg_sortNormalView, SLOT(setText(QString)));
-  connect(m_sortReconcileView, SIGNAL(settingsChanged(QString)), kcfg_sortReconcileView, SLOT(setText(QString)));
-  connect(m_sortSearchView, SIGNAL(settingsChanged(QString)), kcfg_sortSearchView, SLOT(setText(QString)));
+  connect(ui->m_sortNormalView, &TransactionSortOption::settingsChanged, ui->kcfg_sortNormalView, &KLineEdit::setText);
+  connect(ui->m_sortReconcileView, &TransactionSortOption::settingsChanged, ui->kcfg_sortReconcileView, &KLineEdit::setText);
+  connect(ui->m_sortSearchView, &TransactionSortOption::settingsChanged, ui->kcfg_sortSearchView, &KLineEdit::setText);
 }
 
 KSettingsRegister::~KSettingsRegister()
 {
+  delete ui;
 }
 
 void KSettingsRegister::slotLoadNormal(const QString& text)
 {
   // only need this once
-  disconnect(kcfg_sortNormalView, SIGNAL(textChanged(QString)), this, SLOT(slotLoadNormal(QString)));
-  m_sortNormalView->setSettings(text);
+  disconnect(ui->kcfg_sortNormalView, &QLineEdit::textChanged, this, &KSettingsRegister::slotLoadNormal);
+  ui->m_sortNormalView->setSettings(text);
 }
 
 void KSettingsRegister::slotLoadReconcile(const QString& text)
 {
   // only need this once
-  disconnect(kcfg_sortReconcileView, SIGNAL(textChanged(QString)), this, SLOT(slotLoadReconcile(QString)));
-  m_sortReconcileView->setSettings(text);
+  disconnect(ui->kcfg_sortReconcileView, &QLineEdit::textChanged, this, &KSettingsRegister::slotLoadReconcile);
+  ui->m_sortReconcileView->setSettings(text);
 }
 
 void KSettingsRegister::slotLoadSearch(const QString& text)
 {
   // only need this once
-  disconnect(kcfg_sortSearchView, SIGNAL(textChanged(QString)), this, SLOT(slotLoadSearch(QString)));
-  m_sortSearchView->setSettings(text);
+  disconnect(ui->kcfg_sortSearchView, &QLineEdit::textChanged, this, &KSettingsRegister::slotLoadSearch);
+  ui->m_sortSearchView->setSettings(text);
 }
