@@ -4,6 +4,8 @@
     begin                : Tue Jul 08 2008
     copyright            : (C) 2008 by Thomas Baumgart
     email                : Thomas Baumgart <ipwizard@users.sourceforge.net>
+                         : Christian David <christian-david@web.de>
+                         (C) 2017 by Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
  ***************************************************************************/
 
 /***************************************************************************
@@ -17,19 +19,49 @@
 
 #include "transactionmatcher.h"
 
-#include <QList>
-#include <klocale.h>
+#include <QDate>
 
+#include <KLocalizedString>
+
+#include "mymoneyaccount.h"
+#include "mymoneymoney.h"
+#include "mymoneysecurity.h"
+#include "mymoneysplit.h"
+#include "mymoneytransaction.h"
+#include "mymoneyutils.h"
 #include "mymoneyfile.h"
+#include "mymoneyexception.h"
+#include "mymoneyenums.h"
+
+class TransactionMatcherPrivate
+{
+  Q_DISABLE_COPY(TransactionMatcherPrivate)
+
+public:
+  TransactionMatcherPrivate()
+  {
+  }
+
+  MyMoneyAccount m_account;
+};
 
 TransactionMatcher::TransactionMatcher(const MyMoneyAccount& acc) :
-    m_account(acc)
+  d_ptr(new TransactionMatcherPrivate)
 {
+  Q_D(TransactionMatcher);
+  d->m_account = acc;
+}
+
+TransactionMatcher::~TransactionMatcher()
+{
+  Q_D(TransactionMatcher);
+  delete d;
 }
 
 void TransactionMatcher::match(MyMoneyTransaction tm, MyMoneySplit sm, MyMoneyTransaction ti, MyMoneySplit si, bool allowImportedTransactions)
 {
-  const MyMoneySecurity& sec = MyMoneyFile::instance()->security(m_account.currencyId());
+  Q_D(TransactionMatcher);
+  auto sec = MyMoneyFile::instance()->security(d->m_account.currencyId());
 
   // Now match the transactions.
   //
@@ -62,15 +94,15 @@ void TransactionMatcher::match(MyMoneyTransaction tm, MyMoneySplit sm, MyMoneyTr
   // is to just base the match on the splits referencing the same (currently
   // selected) account.
 
-  // verify, that tm is a manually (non-matched) transaction and ti an imported one
-  if (sm.isMatched() || (!allowImportedTransactions && tm.isImported()))
+  // verify, that tm is a manual (non-matched) transaction
+  // allow matching two manual transactions
+
+  if ((!allowImportedTransactions && tm.isImported()) || sm.isMatched())
     throw MYMONEYEXCEPTION(i18n("First transaction does not match requirement for matching"));
-  if (!ti.isImported())
-    throw MYMONEYEXCEPTION(i18n("Second transaction does not match requirement for matching"));
 
   // verify that the amounts are the same, otherwise we should not be matching!
   if (sm.shares() != si.shares()) {
-    throw MYMONEYEXCEPTION(i18n("Splits for %1 have conflicting values (%2,%3)", m_account.name(), MyMoneyUtils::formatMoney(sm.shares(), m_account, sec), MyMoneyUtils::formatMoney(si.shares(), m_account, sec)));
+    throw MYMONEYEXCEPTION(i18n("Splits for %1 have conflicting values (%2,%3)", d->m_account.name(), MyMoneyUtils::formatMoney(sm.shares(), d->m_account, sec), MyMoneyUtils::formatMoney(si.shares(), d->m_account, sec)));
   }
 
   // ipwizard: I took over the code to keep the bank id found in the endMatchTransaction
@@ -82,18 +114,19 @@ void TransactionMatcher::match(MyMoneyTransaction tm, MyMoneySplit sm, MyMoneyTr
       if (sm.bankID().isEmpty()) {
         sm.setBankID(bankID);
         tm.modifySplit(sm);
-      } else if (sm.bankID() != bankID) {
-        throw MYMONEYEXCEPTION(i18n("Both of these transactions have been imported into %1.  Therefore they cannot be matched.  Matching works with one imported transaction and one non-imported transaction.", m_account.name()));
       }
     } catch (const MyMoneyException &e) {
       QString estr = e.what();
       throw MYMONEYEXCEPTION(i18n("Unable to match all splits (%1)", estr));
     }
   }
+  //
+  //  we now allow matching of two non-imported transactions
+  //
 
   // mark the split as cleared if it does not have a reconciliation information yet
-  if (sm.reconcileFlag() == MyMoneySplit::NotReconciled) {
-    sm.setReconcileFlag(MyMoneySplit::Cleared);
+  if (sm.reconcileFlag() == eMyMoney::Split::State::NotReconciled) {
+    sm.setReconcileFlag(eMyMoney::Split::State::Cleared);
   }
 
   // if we don't have a payee assigned to the manually entered transaction
@@ -125,6 +158,7 @@ void TransactionMatcher::match(MyMoneyTransaction tm, MyMoneySplit sm, MyMoneyTr
   sm.addMatch(ti);
   tm.modifySplit(sm);
 
+  ti.modifySplit(si);///
   MyMoneyFile::instance()->modifyTransaction(tm);
   // Delete the end transaction if it was stored in the engine
   if (!ti.id().isEmpty())
