@@ -1,7 +1,9 @@
+
 /***************************************************************************
                           mymoneyaccount.h
                           -------------------
     copyright            : (C) 2002 by Thomas Baumgart <ipwizard@users.sourceforge.net>
+                           (C) 2017 by Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
 
  ***************************************************************************/
 
@@ -20,26 +22,27 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
-#include <QString>
-#include <QDateTime>
-#include <QStringList>
-#include <QPixmap>
 #include <QMetaType>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include <mymoneymoney.h>
-#include <mymoneyobject.h>
-#include <mymoneykeyvaluecontainer.h>
-#include <mymoneysecurity.h>
-#include <kmm_mymoney_export.h>
-#include <mymoneyunittestable.h>
-#include "mymoneyutils.h"
-class MyMoneyTransaction;
-class MyMoneyInstitution;
+#include "mymoneykeyvaluecontainer.h"
+#include "mymoneyobject.h"
+#include "kmm_mymoney_export.h"
+#include "mymoneyunittestable.h"
+
+class QString;
+class QDate;
+class QPixmap;
+class QDomElement;
+class MyMoneySecurity;
+class MyMoneyMoney;
 class MyMoneySplit;
-class MyMoneyObjectContainer;
+class payeeIdentifier;
+namespace payeeIdentifiers { class ibanBic; }
+namespace eMyMoney { namespace Account { enum class Type; } }
+template <class T> class payeeIdentifierTyped;
 
 /**
   * A representation of an account.
@@ -49,7 +52,7 @@ class MyMoneyObjectContainer;
   *
   * Currently, the following account types are known:
   *
-  * @li  UnknownAccountType
+  * @li  Unknown
   * @li  Checkings
   * @li  Savings
   * @li  Cash
@@ -72,42 +75,18 @@ class MyMoneyObjectContainer;
   *
   * @author Michael Edwardes 2000-2001
   * @author Thomas Baumgart 2002
+  * @author Łukasz Wojniłowicz 2017
   *
 **/
-class KMM_MYMONEY_EXPORT MyMoneyAccount : public MyMoneyObject, public MyMoneyKeyValueContainer
+
+class MyMoneyAccountPrivate;
+class KMM_MYMONEY_EXPORT MyMoneyAccount : public MyMoneyObject, public MyMoneyKeyValueContainer /*, public MyMoneyPayeeIdentifierContainer */
 {
-  friend class MyMoneyObjectContainer;
+  Q_DECLARE_PRIVATE_D(MyMoneyObject::d_ptr, MyMoneyAccount)
+
   KMM_MYMONEY_UNIT_TESTABLE
 
 public:
-
-  /**
-    * Account types currently supported.
-    */
-  typedef enum _accountTypeE {
-    UnknownAccountType = 0, /**< For error handling */
-    Checkings,            /**< Standard checking account */
-    Savings,              /**< Typical savings account */
-    Cash,                 /**< Denotes a shoe-box or pillowcase stuffed
-                               with cash */
-    CreditCard,           /**< Credit card accounts */
-    Loan,                 /**< Loan and mortgage accounts (liability) */
-    CertificateDep,       /**< Certificates of Deposit */
-    Investment,           /**< Investment account */
-    MoneyMarket,          /**< Money Market Account */
-    Asset,                /**< Denotes a generic asset account.*/
-    Liability,            /**< Denotes a generic liability account.*/
-    Currency,             /**< Denotes a currency trading account. */
-    Income,               /**< Denotes an income account */
-    Expense,              /**< Denotes an expense account */
-    AssetLoan,            /**< Denotes a loan (asset of the owner of this object) */
-    Stock,                /**< Denotes an security account as sub-account for an investment */
-    Equity,               /**< Denotes an equity account e.g. opening/closeing balance */
-
-    /* insert new account types above this line */
-    MaxAccountTypes       /**< Denotes the number of different account types */
-  } accountTypeE;
-
   /**
     * This is the constructor for a new empty account
     */
@@ -121,7 +100,8 @@ public:
     * @param id id assigned to the account
     * @param right account definition
     */
-  MyMoneyAccount(const QString& id, const MyMoneyAccount& right);
+  MyMoneyAccount(const QString& id,
+                 const MyMoneyAccount& other);
 
   /**
     * This is the constructor for an account that is described by a
@@ -130,7 +110,12 @@ public:
     * @param el const reference to the QDomElement from which to
     *           create the object
     */
-  MyMoneyAccount(const QDomElement& el);
+  explicit MyMoneyAccount(const QDomElement& el);
+
+  MyMoneyAccount(const MyMoneyAccount & other);
+  MyMoneyAccount(MyMoneyAccount && other);
+  MyMoneyAccount & operator=(MyMoneyAccount other);
+  friend void swap(MyMoneyAccount& first, MyMoneyAccount& second);
 
   /**
     * This is the destructor for any MyMoneyAccount object
@@ -173,7 +158,7 @@ public:
     *
     * @return accountTypeE of major account type
     */
-  MyMoneyAccount::accountTypeE accountGroup() const;
+  eMyMoney::Account::Type accountGroup() const;
 
   /**
     * This method returns the id of the MyMoneyInstitution object this account
@@ -182,88 +167,130 @@ public:
     *         an internal account
     * @see setInstitution
     */
-  const QString& institutionId(void) const {
-    return m_institution;
-  }
+  QString institutionId() const;
+
+  /**
+    * This method is used to set the id of the institution this account
+    * belongs to.
+    *
+    * @param id id of the institution this account belongs to
+    * @see institution
+    */
+  void setInstitutionId(const QString& id);
 
   /**
     * This method returns the name of the account
     * @return name of account
     * @see setName()
     */
-  const QString& name(void) const {
-    return m_name;
-  }
+  QString name() const;
+
+  /**
+    * This method is used to set the name of the account
+    * @param name name of the account
+    * @see name
+    */
+  void setName(const QString& name);
 
   /**
     * This method returns the number of the account at the institution
     * @return number of account at the institution
     * @see setNumber
     */
-  const QString& number(void) const {
-    return m_number;
-  }
+  QString number() const;
+
+  /**
+    * This method is used to set the number of the account at the institution
+    * @param number number of the account
+    * @see number
+    */
+  void setNumber(const QString& number);
 
   /**
     * This method returns the descriptive text of the account.
     * @return description of account
     * @see setDescription
     */
-  const QString& description(void) const {
-    return m_description;
-  }
+  QString description() const;
+
+  /**
+    * This method is used to set the descriptive text of the account
+    *
+    * @param desc text that serves as description
+    * @see setDescription
+    */
+  void setDescription(const QString& desc);
 
   /**
     * This method returns the opening date of this account
     * @return date of opening of this account as const QDate value
     * @see setOpeningDate()
     */
-  const QDate& openingDate(void) const {
-    return m_openingDate;
-  }
+  QDate openingDate() const;
+
+  /**
+    * This method is used to set the opening date information of an
+    * account.
+    *
+    * @param date QDate of opening date
+    * @see openingDate
+    */
+  void setOpeningDate(const QDate& date);
 
   /**
     * This method returns the date of the last reconciliation of this account
     * @return date of last reconciliation as const QDate value
     * @see setLastReconciliationDate
     */
-  const QDate& lastReconciliationDate(void) const {
-    return m_lastReconciliationDate;
-  }
+  QDate lastReconciliationDate() const;
+
+  /**
+    * This method is used to set the date of the last reconciliation
+    * of an account.
+    * @param date QDate of last reconciliation
+    * @see lastReconciliationDate
+    */
+  void setLastReconciliationDate(const QDate& date);
 
   /**
     * This method returns the date the account was last modified
     * @return date of last modification as const QDate value
     * @see setLastModified
     */
-  const QDate& lastModified(void) const {
-    return m_lastModified;
-  }
+  QDate lastModified() const;
+
+  /**
+    * This method is used to modify the date of the last
+    * modification access.
+    * @param date date of last modification
+    * @see lastModified
+    */
+  void setLastModified(const QDate& date);
 
   /**
     * This method is used to return the ID of the parent account
     * @return QString with the ID of the parent of this account
     */
-  const QString& parentAccountId(void) const {
-    return m_parentAccount;
-  };
+  QString parentAccountId() const;
+
+  /**
+    * This method is used to set a new parent account id
+    * @param parent QString reference to new parent account
+    */
+  void setParentAccountId(const QString& parent);
 
   /**
     * This method returns the list of the account id's of
     * subordinate accounts
     * @return QStringList account ids
     */
-  const QStringList& accountList(void) const {
-    return m_accountList;
-  };
+  QStringList accountList() const;
 
   /**
     * This method returns the number of entries in the m_accountList
     * @return number of entries in the accountList
     */
-  int accountCount(void) const {
-    return m_accountList.count();
-  };
+  int accountCount() const;
 
   /**
     * This method is used to add an account id as sub-ordinate account
@@ -282,90 +309,38 @@ public:
     * This method is used to remove all accounts from the list of
     * sub-ordinate accounts.
     */
-  void removeAccountIds(void);
+  void removeAccountIds();
 
   /**
-    * This method is used to modify the date of the last
-    * modification access.
-    * @param date date of last modification
-    * @see lastModified
-    */
-  void setLastModified(const QDate& date);
+   * Return the stored account identifiers
+   *
+   * @internal This method is temporary until MyMoneyAccount is a MyMoneyPayeeIdentifierContainer. But before this
+   * can happen the account numbers and iban kvp data must be moved there.
+   */
+  QList< payeeIdentifier > payeeIdentifiers() const;
 
   /**
-    * This method is used to set the name of the account
-    * @param name name of the account
-    * @see name
-    */
-  void setName(const QString& name);
+   * @see MyMoneyPayeeIdentifierContainer::payeeIdentifiersByType()
+   */
+  template< class type >
+  QList< ::payeeIdentifierTyped<type> > payeeIdentifiersByType() const;
 
   /**
-    * This method is used to set the number of the account at the institution
-    * @param number number of the account
-    * @see number
+    * This method is used to update m_lastModified to the current date
     */
-  void setNumber(const QString& number);
+  void touch();
 
   /**
-    * This method is used to set the descriptive text of the account
-    *
-    * @param desc text that serves as description
-    * @see setDescription
+    * This method returns the type of the account.
     */
-  void setDescription(const QString& desc);
-
-  /**
-    * This method is used to set the id of the institution this account
-    * belongs to.
-    *
-    * @param id id of the institution this account belongs to
-    * @see institution
-    */
-  void setInstitutionId(const QString& id);
-
-  /**
-    * This method is used to set the opening date information of an
-    * account.
-    *
-    * @param date QDate of opening date
-    * @see openingDate
-    */
-  void setOpeningDate(const QDate& date);
-
-  /**
-    * This method is used to set the date of the last reconciliation
-    * of an account.
-    * @param date QDate of last reconciliation
-    * @see lastReconciliationDate
-    */
-  void setLastReconciliationDate(const QDate& date);
+  eMyMoney::Account::Type accountType() const;
 
   /**
     * This method is used to change the account type
     *
     * @param type account type
     */
-  void setAccountType(const accountTypeE type);
-
-  /**
-    * This method is used to set a new parent account id
-    * @param parent QString reference to new parent account
-    */
-  void setParentAccountId(const QString& parent);
-
-  /**
-    * This method is used to update m_lastModified to the current date
-    */
-  void touch(void) {
-    setLastModified(QDate::currentDate());
-  }
-
-  /**
-    * This method returns the type of the account.
-    */
-  accountTypeE accountType(void) const {
-    return m_accountType;
-  }
+  void setAccountType(const eMyMoney::Account::Type type);
 
   /**
     * This method retrieves the id of the currency used with this account.
@@ -373,9 +348,7 @@ public:
     *
     * @return id of currency
     */
-  const QString& currencyId(void) const {
-    return m_currencyId;
-  };
+  QString currencyId() const;
 
   /**
     * This method sets the id of the currency used with this account.
@@ -384,7 +357,7 @@ public:
     */
   void setCurrencyId(const QString& id);
 
-  void writeXML(QDomDocument& document, QDomElement& parent) const;
+  void writeXML(QDomDocument& document, QDomElement& parent) const override;
 
   /**
     * This method checks if a reference to the given object exists. It returns,
@@ -395,15 +368,13 @@ public:
     * @retval true This object references object with id @p id.
     * @retval false This object does not reference the object with id @p id.
     */
-  virtual bool hasReferenceTo(const QString& id) const;
+  bool hasReferenceTo(const QString& id) const override;
 
   /**
     * This member returns the balance of this account based on
     * all transactions stored in the journal.
     */
-  const MyMoneyMoney& balance(void) const {
-    return m_balance;
-  }
+  MyMoneyMoney balance() const;
 
   /**
     * This method adjusts the balance of this account
@@ -425,9 +396,7 @@ public:
     * @param val const reference to MyMoneyMoney object containing the
     *             value to be assigned to the balance
     */
-  void setBalance(const MyMoneyMoney& val) {
-    m_balance = val;
-  }
+  void setBalance(const MyMoneyMoney& val);
 
   /**
     * This method sets the kvp's for online banking with this account
@@ -441,7 +410,7 @@ public:
     *
     * @return The container of kvp's needed when connecting to this account
     */
-  const MyMoneyKeyValueContainer& onlineBankingSettings(void) const;
+  MyMoneyKeyValueContainer onlineBankingSettings() const;
 
   /**
     * This method sets the closed flag for the account. This is just
@@ -459,7 +428,7 @@ public:
     * @retval false account is marked open (the default for new accounts)
     * @retval true account is marked closed
     */
-  bool isClosed(void) const;
+  bool isClosed() const;
 
   /**
     * returns the applicable smallest fraction for this account
@@ -486,7 +455,7 @@ public:
     * @note Don't use this method outside of KMyMoney application context (eg. testcases).
     * Use the above method instead.
     */
-  int fraction(void) const;
+  int fraction() const;
 
   /**
     * This method returns @a true if the account type is
@@ -497,7 +466,7 @@ public:
     *
     * @deprecated use isIncomeExpense() instead
     */
-  KMM_MYMONEY_DEPRECATED bool isCategory(void) const;
+  KMM_MYMONEY_DEPRECATED bool isCategory() const;
 
   /**
     * This method returns @a true if the account type is
@@ -506,7 +475,7 @@ public:
     * @retval true account is of type income or expense
     * @retval false for all other account types
     */
-  bool isIncomeExpense(void) const;
+  bool isIncomeExpense() const;
 
   /**
     * This method returns @a true if the account type is
@@ -515,7 +484,7 @@ public:
     * @retval true account is of type asset or liability
     * @retval false for all other account types
     */
-  bool isAssetLiability(void) const;
+  bool isAssetLiability() const;
 
   /**
     * This method returns @a true if the account type is
@@ -524,7 +493,7 @@ public:
     * @retval true account is of type Loan or AssetLoan
     * @retval false for all other account types
     */
-  bool isLoan(void) const;
+  bool isLoan() const;
 
   /**
     * This method returns @a true if the account type is
@@ -533,7 +502,7 @@ public:
     * @retval true account is of type Stock
     * @retval false for all other account types
     */
-  bool isInvest(void) const;
+  bool isInvest() const;
 
   /**
     * This method returns @a true if the account type is
@@ -542,14 +511,25 @@ public:
     * @retval true account is of type Checkings, Savings or Cash
     * @retval false for all other account types
     */
-  bool isLiquidAsset(void) const;
+  bool isLiquidAsset() const;
+
+  /**
+   * This method returns true if a costcenter assignment is required for this account
+   */
+  bool isCostCenterRequired() const;
+
+  /**
+   * This method allows to control if a cost center assignment is required
+   * for this account. It is if @a required is @c true (the default).
+   */
+  void setCostCenterRequired(bool required = true);
 
   /**
    * This method returns a name that has a brokerage suffix of
    * the current name. It only works on investment accounts and
    * returns the name for all other cases.
    */
-  QString brokerageName(void) const;
+  QString brokerageName() const;
 
   /**
    * @param reconcileFlag if set to @a true a reconcile overlay will be
@@ -557,17 +537,17 @@ public:
    * @param size is a hint for the size of the icon
    * @return a pixmap using DesktopIcon for the account type
    */
-  QPixmap accountPixmap(bool reconcileFlag = false, int size = 0) const;
+  QPixmap accountPixmap(const bool reconcileFlag = false, const int size = 64) const;
 
   /**
    * This method is used to convert the internal representation of
    * an account type into a human readable format
    *
    * @param accountType numerical representation of the account type.
-   *                    For possible values, see MyMoneyAccount::accountTypeE
+   *                    For possible values, see eMyMoney::Account::Type
    * @return QString representing the human readable form
    */
-  static QString accountTypeToString(const MyMoneyAccount::accountTypeE accountType);
+  static QString accountTypeToString(const eMyMoney::Account::Type accountType);
 
   /**
     * keeps a history record of a reconciliation for this account on @a date
@@ -584,172 +564,50 @@ public:
     *
     * @sa addReconciliation()
     */
-  const QMap<QDate, MyMoneyMoney>& reconciliationHistory();
+  QMap<QDate, MyMoneyMoney> reconciliationHistory();
+
+  /**
+    * @return @c true if account has an online mapping, @c false otherwise
+    */
+  bool hasOnlineMapping() const;
 
   QDataStream &operator<<(const MyMoneyAccount &);
   QDataStream &operator>>(MyMoneyAccount &);
-
-private:
-  /**
-    * This member variable identifies the type of account
-    */
-  accountTypeE m_accountType;
-
-  /**
-    * This member variable keeps the ID of the MyMoneyInstitution object
-    * that this object belongs to.
-    */
-  QString m_institution;
-
-  /**
-    * This member variable keeps the name of the account
-    * It is solely for documentation purposes and it's contents is not
-    * used otherwise by the mymoney-engine.
-    */
-  QString m_name;
-
-  /**
-    * This member variable keeps the account number at the institution
-    * It is solely for documentation purposes and it's contents is not
-    * used otherwise by the mymoney-engine.
-    */
-  QString m_number;
-
-  /**
-    * This member variable is a description of the account.
-    * It is solely for documentation purposes and it's contents is not
-    * used otherwise by the mymoney-engine.
-    */
-  QString m_description;
-
-  /**
-    * This member variable keeps the date when the account
-    * was last modified.
-    */
-  QDate m_lastModified;
-
-  /**
-    * This member variable keeps the date when the
-    * account was created as an object in a MyMoneyFile
-    */
-  QDate m_openingDate;
-
-  /**
-    * This member variable keeps the date of the last
-    * reconciliation of this account
-    */
-  QDate m_lastReconciliationDate;
-
-  /**
-    * This member holds the ID's of all sub-ordinate accounts
-    */
-  QStringList m_accountList;
-
-  /**
-    * This member contains the ID of the parent account
-    */
-  QString m_parentAccount;
-
-  /**
-    * This member contains the ID of the currency associated with this account
-    */
-  QString m_currencyId;
-
-  /**
-    * This member holds the balance of all transactions stored in the journal
-    * for this account.
-    */
-  MyMoneyMoney    m_balance;
-
-  /**
-    * This member variable keeps the set of kvp's needed to establish
-    * online banking sessions to this account.
-    */
-  MyMoneyKeyValueContainer m_onlineBankingSettings;
-
-  /**
-    * This member keeps the fraction for the account. It is filled by MyMoneyFile
-    * when set to -1. See also @sa fraction(const MyMoneySecurity&).
-    */
-  int             m_fraction;
-
-  /**
-    * This member keeps the reconciliation history
-    */
-  QMap<QDate, MyMoneyMoney> m_reconciliationHistory;
 };
 
-/**
-  * This class is a convenience class to access data for loan accounts.
-  * It does contain the same member variables as a MyMoneyAccount object,
-  * but serves a set of getter/setter methods to ease the access to
-  * laon relevant data stored in the key value container of the MyMoneyAccount
-  * object.
-  */
-class KMM_MYMONEY_EXPORT MyMoneyAccountLoan : public MyMoneyAccount
+inline void swap(MyMoneyAccount& first, MyMoneyAccount& second) // krazy:exclude=inline
 {
-public:
-  enum interestDueE {
-    paymentDue = 0,
-    paymentReceived
-  };
+  using std::swap;
+  swap(first.MyMoneyObject::d_ptr, second.MyMoneyObject::d_ptr);
+  swap(first.MyMoneyKeyValueContainer::d_ptr, second.MyMoneyKeyValueContainer::d_ptr);
+}
 
-  enum interestChangeUnitE {
-    changeDaily = 0,
-    changeWeekly,
-    changeMonthly,
-    changeYearly
-  };
+inline MyMoneyAccount::MyMoneyAccount(MyMoneyAccount && other) : MyMoneyAccount() // krazy:exclude=inline
+{
+  swap(*this, other);
+}
 
-  MyMoneyAccountLoan() {}
-  MyMoneyAccountLoan(const MyMoneyAccount&);
-  ~MyMoneyAccountLoan() {}
+inline MyMoneyAccount & MyMoneyAccount::operator=(MyMoneyAccount other) // krazy:exclude=inline
+{
+  swap(*this, other);
+  return *this;
+}
 
-  const MyMoneyMoney loanAmount(void) const;
-  void setLoanAmount(const MyMoneyMoney& amount);
-  const MyMoneyMoney interestRate(const QDate& date) const;
-  void setInterestRate(const QDate& date, const MyMoneyMoney& rate);
-  interestDueE interestCalculation(void) const;
-  void setInterestCalculation(const interestDueE onReception);
-  const QDate nextInterestChange(void) const;
-  void setNextInterestChange(const QDate& date);
-  const QString schedule(void) const;
-  void setSchedule(const QString& sched);
-  bool fixedInterestRate(void) const;
-  void setFixedInterestRate(const bool fixed);
-  const MyMoneyMoney finalPayment(void) const;
-  void setFinalPayment(const MyMoneyMoney& finalPayment);
-  unsigned int term(void) const;
-  void setTerm(const unsigned int payments);
-  int interestChangeFrequency(int* unit = 0) const;
-  void setInterestChangeFrequency(const int amount, const int unit);
-  const MyMoneyMoney periodicPayment(void) const;
-  void setPeriodicPayment(const MyMoneyMoney& payment);
-  int interestCompounding(void) const;
-  void setInterestCompounding(int frequency);
-  const QString payee(void) const;
-  void setPayee(const QString& payee);
-  const QString interestAccountId(void) const;
-  void setInterestAccountId(const QString& id);
+template< class type >
+QList< payeeIdentifierTyped<type> > MyMoneyAccount::payeeIdentifiersByType() const
+{
+  QList< payeeIdentifierTyped<type> > typedList;
+  return typedList;
+}
 
-  /**
-    * This method checks if a reference to the given object exists. It returns,
-    * a @p true if the object is referencing the one requested by the
-    * parameter @p id. If it does not, this method returns @p false.
-    *
-    * @param id id of the object to be checked for references
-    * @retval true This object references object with id @p id.
-    * @retval false This object does not reference the object with id @p id.
-    */
-  virtual bool hasReferenceTo(const QString& id) const;
-
-};
+template<>
+QList< payeeIdentifierTyped< ::payeeIdentifiers::ibanBic> > MyMoneyAccount::payeeIdentifiersByType() const;
 
 /**
-  * Make it possible to hold @ref MyMoneyAccount objects inside @ref QVariant objects.
-  */
+ * Make it possible to hold @ref MyMoneyAccount objects,
+ * @ref accountTypeE and @ref amountTypeE inside @ref QVariant objects.
+ */
 Q_DECLARE_METATYPE(MyMoneyAccount)
+Q_DECLARE_METATYPE(eMyMoney::Account::Type)
 
 #endif
-
-
