@@ -14,20 +14,33 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "mymoneyobjectcontainer.h"
+
 // ----------------------------------------------------------------------------
 // QT Includes
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include <mymoneyobjectcontainer.h>
+#include "mymoney/onlinejob.h"
+
+#include "imymoneystorage.h"
+#include "mymoneyinstitution.h"
+#include "mymoneyaccount.h"
+#include "mymoneysecurity.h"
+#include "mymoneyschedule.h"
+#include "mymoneypayee.h"
+#include "mymoneytag.h"
+#include "mymoneycostcenter.h"
 
 struct MyMoneyObjectContainer::Private {
-  Private(MyMoneyObjectContainer *p) : pub(p) {
+  Private(MyMoneyObjectContainer *p) :
+      storage(0),
+      pub(p) {
   }
 
   void assignFraction(MyMoneyAccount* acc) {
-    if (acc != 0 && acc->m_fraction == -1) {
+    if (acc != 0 && acc->fraction() == -1) {
       const MyMoneySecurity& sec = pub->security(acc->currencyId());
       acc->fraction(sec);
     }
@@ -117,6 +130,9 @@ struct MyMoneyObjectContainer::Private {
   QHash<QString, MyMoneySecurity const *> securityCache;
   QHash<QString, MyMoneyInstitution const *> institutionCache;
   QHash<QString, MyMoneySchedule const *> scheduleCache;
+  QHash<QString, onlineJob const *> onlineJobCache;
+  QHash<QString, MyMoneyCostCenter const*> costCenterCache;
+
   IMyMoneyStorage* storage;
   MyMoneyObjectContainer *pub;
 };
@@ -139,6 +155,8 @@ void MyMoneyObjectContainer::clear(IMyMoneyStorage* storage)
   d->clearCache(d->securityCache);
   d->clearCache(d->institutionCache);
   d->clearCache(d->scheduleCache);
+  d->clearCache(d->onlineJobCache);
+  d->clearCache(d->costCenterCache);
 
   if (storage)
     d->storage = storage;
@@ -158,6 +176,10 @@ void MyMoneyObjectContainer::clear(const QString& id)
     return;
   if (d->clearObject(d->scheduleCache, id))
     return;
+  if (d->clearObject(d->onlineJobCache, id))
+    return;
+  if (d->clearObject(d->costCenterCache, id))
+    return;
   qWarning("Ooops, should clear an unknown object with id '%s'", qPrintable(id));
 }
 
@@ -176,6 +198,10 @@ const MyMoneyObject * MyMoneyObjectContainer::object(const QString& id) const
     return obj;
   if ((obj = d->getObject(d->scheduleCache, id)))
     return obj;
+  if ((obj = d->getObject(d->onlineJobCache, id)))
+    return obj;
+  if ((obj = d->getObject(d->costCenterCache, id)))
+    return obj;
   qWarning("Ooops, should get an unknown object with id '%s'", qPrintable(id));
   return 0;
 }
@@ -192,7 +218,7 @@ void MyMoneyObjectContainer::account(QList<MyMoneyAccount>& list)
   }
 }
 
-const MyMoneyAccount& MyMoneyObjectContainer::account(const QString& id)
+MyMoneyAccount MyMoneyObjectContainer::account(const QString& id)
 {
   static MyMoneyAccount nullElement;
   if (id.isEmpty())
@@ -212,7 +238,7 @@ const MyMoneyAccount& MyMoneyObjectContainer::account(const QString& id)
   return nullElement;
 }
 
-const MyMoneyAccount& MyMoneyObjectContainer::accountByName(const QString& name) const
+MyMoneyAccount MyMoneyObjectContainer::accountByName(const QString& name) const
 {
   static MyMoneyAccount nullElement;
   QHash<QString, MyMoneyAccount const *>::const_iterator it;
@@ -240,6 +266,10 @@ void MyMoneyObjectContainer::refresh(const QString& id)
     return;
   if (d->refreshObject(id, d->scheduleCache, &IMyMoneyStorage::schedule))
     return;
+  if (d->refreshObject(id, d->onlineJobCache, &IMyMoneyStorage::getOnlineJob))
+    return;
+  if (d->refreshObject(id, d->costCenterCache, &IMyMoneyStorage::costCenter))
+    return;
 
   // special handling of securities
   QHash<QString, MyMoneySecurity const *>::iterator it = d->securityCache.find(id);
@@ -258,27 +288,27 @@ void MyMoneyObjectContainer::refresh(const QString& id)
   qWarning("Ooops, should refresh an unknown object with id '%s'", qPrintable(id));
 }
 
-const MyMoneyPayee& MyMoneyObjectContainer::payee(const QString& id)
+MyMoneyPayee MyMoneyObjectContainer::payee(const QString& id)
 {
   return d->objectAccessMethodImpl(id, d->payeeCache, &IMyMoneyStorage::payee);
 }
 
-const MyMoneyTag& MyMoneyObjectContainer::tag(const QString& id)
+MyMoneyTag MyMoneyObjectContainer::tag(const QString& id)
 {
   return d->objectAccessMethodImpl(id, d->tagCache, &IMyMoneyStorage::tag);
 }
 
-const MyMoneySecurity& MyMoneyObjectContainer::security(const QString& id)
+MyMoneySecurity MyMoneyObjectContainer::security(const QString& id)
 {
   return d->objectAccessMethodImpl(id, d->securityCache, &IMyMoneyStorage::security);
 }
 
-const MyMoneyInstitution& MyMoneyObjectContainer::institution(const QString& id)
+MyMoneyInstitution MyMoneyObjectContainer::institution(const QString& id)
 {
   return d->objectAccessMethodImpl(id, d->institutionCache, &IMyMoneyStorage::institution);
 }
 
-const MyMoneySchedule& MyMoneyObjectContainer::schedule(const QString& id)
+MyMoneySchedule MyMoneyObjectContainer::schedule(const QString& id)
 {
   return d->objectAccessMethodImpl(id, d->scheduleCache, &IMyMoneyStorage::schedule);
 }
@@ -313,6 +343,11 @@ void MyMoneyObjectContainer::preloadSchedule(const QList<MyMoneySchedule>& list)
   d->preloadListMethodImpl(list, d->scheduleCache);
 }
 
+void MyMoneyObjectContainer::preloadOnlineJob(const QList< onlineJob >& list)
+{
+  d->preloadListMethodImpl(list, d->onlineJobCache);
+}
+
 void MyMoneyObjectContainer::preloadAccount(const MyMoneyAccount& account)
 {
   d->preloadMethodImpl(account, d->accountCache);
@@ -343,6 +378,11 @@ void MyMoneyObjectContainer::preloadSchedule(const MyMoneySchedule& schedule)
   d->preloadMethodImpl(schedule, d->scheduleCache);
 }
 
+void MyMoneyObjectContainer::preloadOnlineJob(const onlineJob& job)
+{
+  d->preloadMethodImpl(job, d->onlineJobCache);
+}
+
 void MyMoneyObjectContainer::payee(QList<MyMoneyPayee>& list)
 {
   d->listMethodImpl(list, d->payeeCache);
