@@ -9,6 +9,7 @@
                            John C <thetacoturtle@users.sourceforge.net>
                            Thomas Baumgart <ipwizard@users.sourceforge.net>
                            Kevin Tambascio <ktambascio@users.sourceforge.net>
+                           (C) 2017 by Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
  ***************************************************************************/
 
 /***************************************************************************
@@ -26,49 +27,48 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
-#include <QString>
-#include <QBitArray>
-#include <QList>
+#include <QtGlobal>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include <mymoneyutils.h>
-#include <mymoneyinstitution.h>
-#include <mymoneyaccount.h>
-#include <mymoneytransaction.h>
-#include <mymoneypayee.h>
-#include <mymoneyscheduled.h>
-#include <mymoneytransactionfilter.h>
-#include <mymoneysecurity.h>
-#include <mymoneyprice.h>
-#include <mymoneyreport.h>
-#include <mymoneybudget.h>
+class QString;
+class QDate;
+class QStringList;
+class QBitArray;
 
-/**
-  * @author Thomas Baumgart
-  *
-  * A simple replacement for QBitArray that does not bark if testBit()
-  * is called with an index out of bounds. It silently returns false
-  * in that case, otherwise calls the base classes implementation.
-  */
-class MyMoneyFileBitArray : public QBitArray
-{
-public:
-  MyMoneyFileBitArray() : QBitArray() {}
-  MyMoneyFileBitArray(int size) : QBitArray(size) {}
-  bool testBit(int index) const;
-  bool operator[](int index) const {
-    return testBit(index);
-  }
-  bool at(int index) const {
-    return testBit(index);
-  }
-};
+class MyMoneyObject;
+class MyMoneyMoney;
+class MyMoneyInstitution;
+class MyMoneyAccount;
+class MyMoneySecurity;
+class MyMoneyPayee;
+class MyMoneyTag;
+class MyMoneyPrice;
+class MyMoneyReport;
+class MyMoneySchedule;
+class MyMoneyBudget;
+class MyMoneySplit;
+class MyMoneyTransaction;
+class MyMoneyTransactionFilter;
+class MyMoneyCostCenter;
+class onlineJob;
 
+template <class Key, class T> class QMap;
+template <typename T> class QList;
+template <class T1, class T2> struct QPair;
+
+typedef QPair<QString, QString> MyMoneySecurityPair;
+typedef QMap<QDate, MyMoneyPrice> MyMoneyPriceEntries;
+typedef QMap<MyMoneySecurityPair, MyMoneyPriceEntries> MyMoneyPriceList;
+
+namespace eMyMoney { namespace Schedule { enum class Type; } }
+namespace eMyMoney { namespace Schedule { enum class Occurrence; } }
+namespace eMyMoney { namespace Schedule { enum class PaymentType; } }
 
 /**
   * @author Thomas Baumgart
+  * @author Łukasz Wojniłowicz
   */
 
 /**
@@ -80,46 +80,22 @@ public:
 class IMyMoneyStorage
 {
 public:
-
-  typedef enum {
-    RefCheckAccount = 0,
-    RefCheckInstitution,
-    RefCheckPayee,
-    RefCheckTransaction,
-    RefCheckReport,
-    RefCheckBudget,
-    RefCheckSchedule,
-    RefCheckSecurity,
-    RefCheckCurrency,
-    RefCheckPrice,
-    // insert new entries above this line
-    MaxRefCheckBits
-  } ReferenceCheckBits;
-
-  // definitions for the ID's of the standard accounts
-#define STD_ACC_LIABILITY "AStd::Liability"
-#define STD_ACC_ASSET     "AStd::Asset"
-#define STD_ACC_EXPENSE   "AStd::Expense"
-#define STD_ACC_INCOME    "AStd::Income"
-#define STD_ACC_EQUITY    "AStd::Equity"
-
-  IMyMoneyStorage();
-  virtual ~IMyMoneyStorage();
+  virtual ~IMyMoneyStorage() = default;
 
   // general get functions
-  virtual const MyMoneyPayee user(void) const = 0;
-  virtual const QDate creationDate(void) const = 0;
-  virtual const QDate lastModificationDate(void) const = 0;
-  virtual unsigned int currentFixVersion(void) const = 0;
-  virtual unsigned int fileFixVersion(void) const = 0;
+  virtual MyMoneyPayee user() const = 0;
+  virtual QDate creationDate() const = 0;
+  virtual QDate lastModificationDate() const = 0;
+  virtual uint currentFixVersion() const = 0;
+  virtual uint fileFixVersion() const = 0;
 
   // general set functions
   virtual void setUser(const MyMoneyPayee& user) = 0;
-  virtual void setFileFixVersion(const unsigned int v) = 0;
+  virtual void setFileFixVersion(const uint v) = 0;
 
   // methods provided by MyMoneyKeyValueContainer
   virtual void setValue(const QString& key, const QString& value) = 0;
-  virtual const QString value(const QString& key) const = 0;
+  virtual QString value(const QString& key) const = 0;
   virtual void deletePair(const QString& key) = 0;
 
   /**
@@ -127,7 +103,7 @@ public:
     * a pointer to the newly created copy. The caller of this method is the
     * new owner of the object and must destroy it.
     */
-  virtual IMyMoneyStorage const * duplicate(void) = 0;
+//  virtual IMyMoneyStorage const * duplicate() = 0;
 
   /**
     * This method is used to create a new account
@@ -167,7 +143,7 @@ public:
     *
     * @return MyMoneyPayee object of payee
     */
-  virtual const MyMoneyPayee payee(const QString& id) const = 0;
+  virtual MyMoneyPayee payee(const QString& id) const = 0;
 
   /**
     * This method is used to retrieve the id to a corresponding
@@ -178,7 +154,7 @@ public:
     *
     * @return MyMoneyPayee object of payee
     */
-  virtual const MyMoneyPayee payeeByName(const QString& payee) const = 0;
+  virtual MyMoneyPayee payeeByName(const QString& payee) const = 0;
 
   /**
     * This method is used to modify an existing payee
@@ -204,7 +180,63 @@ public:
     *
     * @return QList<MyMoneyPayee> containing the payee information
     */
-  virtual const QList<MyMoneyPayee> payeeList(void) const = 0;
+  virtual QList<MyMoneyPayee> payeeList() const = 0;
+
+  /**
+    * This method is used to create a new tag
+    *
+    * An exception will be thrown upon error conditions
+    *
+    * @param tag MyMoneyTag reference to tag information
+    */
+  virtual void addTag(MyMoneyTag& tag) = 0;
+
+  /**
+    * This method is used to retrieve information about a tag
+    * An exception will be thrown upon error conditions.
+    *
+    * @param id QString reference to id of tag
+    *
+    * @return MyMoneyTag object of tag
+    */
+  virtual MyMoneyTag tag(const QString& id) const = 0;
+
+  /**
+    * This method is used to retrieve the id to a corresponding
+    * name of a tag.
+    * An exception will be thrown upon error conditions.
+    *
+    * @param tag QString reference to name of tag
+    *
+    * @return MyMoneyTag object of tag
+    */
+  virtual MyMoneyTag tagByName(const QString& tag) const = 0;
+
+  /**
+    * This method is used to modify an existing tag
+    *
+    * An exception will be thrown upon error conditions
+    *
+    * @param tag MyMoneyTag reference to tag information
+    */
+  virtual void modifyTag(const MyMoneyTag& tag) = 0;
+
+  /**
+    * This method is used to remove an existing tag
+    *
+    * An exception will be thrown upon error conditions
+    *
+    * @param tag MyMoneyTag reference to tag information
+    */
+  virtual void removeTag(const MyMoneyTag& tag) = 0;
+
+  /**
+    * This method returns a list of the tags
+    * inside a MyMoneyStorage object
+    *
+    * @return QList<MyMoneyTag> containing the tag information
+    */
+  virtual QList<MyMoneyTag> tagList() const = 0;
 
   /**
     * Returns the account addressed by it's id.
@@ -215,7 +247,7 @@ public:
     * @return reference to MyMoneyAccount object. An exception is thrown
     *         if the id is unknown
     */
-  virtual const MyMoneyAccount account(const QString& id) const = 0;
+  virtual MyMoneyAccount account(const QString& id) const = 0;
 
   /**
     * This method is used to check whether a given
@@ -263,7 +295,7 @@ public:
     *        referenced in the splits are not updated. This is used for
     *        bulk loading a lot of transactions but not during normal operation
     */
-  virtual void addTransaction(MyMoneyTransaction& transaction, const bool skipAccountUpdate = false) = 0;
+  virtual void addTransaction(MyMoneyTransaction& transaction, bool skipAccountUpdate = false) = 0;
 
   /**
     * This method is used to determince, if the account with the
@@ -287,7 +319,7 @@ public:
     * @param date return balance for specific date
     * @return balance of the account as MyMoneyMoney object
     */
-  virtual const MyMoneyMoney balance(const QString& id, const QDate& date) const = 0;
+  virtual MyMoneyMoney balance(const QString& id, const QDate& date) const = 0;
 
   /**
     * This method is used to return the actual balance of an account
@@ -300,7 +332,7 @@ public:
     * @param date return balance for specific date
     * @return balance of the account as MyMoneyMoney object
     */
-  virtual const MyMoneyMoney totalBalance(const QString& id, const QDate& date) const = 0;
+  virtual MyMoneyMoney totalBalance(const QString& id, const QDate& date) const = 0;
 
   /**
     * Returns the institution of a given ID
@@ -309,7 +341,7 @@ public:
     * @return MyMoneyInstitution object filled with data. If the institution
     *         could not be found, an exception will be thrown
     */
-  virtual const MyMoneyInstitution institution(const QString& id) const = 0;
+  virtual MyMoneyInstitution institution(const QString& id) const = 0;
 
   /**
     * This method returns an indicator if the storage object has been
@@ -317,7 +349,7 @@ public:
     *
     * @return true if changed, false if not
     */
-  virtual bool dirty(void) const = 0;
+  virtual bool dirty() const = 0;
 
   /**
     * This method can be used by an external object to force the
@@ -325,7 +357,7 @@ public:
     * to an external destination failed but the previous storage
     * to a local disk was ok.
     */
-  virtual void setDirty(void) = 0;
+  virtual void setDirty() = 0;
 
   /**
     * This method returns the number of accounts currently known to this storage
@@ -333,7 +365,7 @@ public:
     *
     * @return number of accounts currently known inside a MyMoneyFile object
     */
-  virtual unsigned int accountCount(void) const = 0;
+  virtual uint accountCount() const = 0;
 
   /**
     * This method returns a list of the institutions
@@ -342,7 +374,7 @@ public:
     * @return QList<MyMoneyInstitution> containing the
     *         institution information
     */
-  virtual const QList<MyMoneyInstitution> institutionList(void) const = 0;
+  virtual QList<MyMoneyInstitution> institutionList() const = 0;
 
   /**
     * Modifies an already existing account in the file global account pool.
@@ -352,7 +384,7 @@ public:
     * @param account reference to the new account information
     * @param skipCheck allows to skip the builtin consistency checks
     */
-  virtual void modifyAccount(const MyMoneyAccount& account, const bool skipCheck = false) = 0;
+  virtual void modifyAccount(const MyMoneyAccount& account, bool skipCheck = false) = 0;
 
   /**
     * Modifies an already existing institution in the file global
@@ -405,7 +437,7 @@ public:
     *
     * @return number of transactions in journal/account
     */
-  virtual unsigned int transactionCount(const QString& account = QString()) const = 0;
+  virtual uint transactionCount(const QString& account) const = 0;
 
   /**
     * This method returns a QMap filled with the number of transactions
@@ -416,7 +448,7 @@ public:
     *
     * @return QMap with numbers of transactions per account
     */
-  virtual const QMap<QString, unsigned long> transactionCountMap(void) const = 0;
+  virtual QMap<QString, ulong> transactionCountMap() const = 0;
 
   /**
     * This method is used to pull a list of transactions from the file
@@ -431,7 +463,7 @@ public:
     *
     * @return set of transactions in form of a QList<MyMoneyTransaction>
     */
-  virtual const QList<MyMoneyTransaction> transactionList(MyMoneyTransactionFilter& filter) const = 0;
+  virtual QList<MyMoneyTransaction> transactionList(MyMoneyTransactionFilter& filter) const = 0;
 
   virtual void transactionList(QList<MyMoneyTransaction>& list, MyMoneyTransactionFilter& filter) const = 0;
 
@@ -467,7 +499,7 @@ public:
     * @param id id of transaction as QString.
     * @return reference to the requested transaction
     */
-  virtual const MyMoneyTransaction transaction(const QString& id) const = 0;
+  virtual MyMoneyTransaction transaction(const QString& id) const = 0;
 
   /**
     * This method is used to extract a transaction from the file global
@@ -477,7 +509,7 @@ public:
     * @param idx number of transaction in this account
     * @return reference to MyMoneyTransaction object
     */
-  virtual const MyMoneyTransaction transaction(const QString& account, const int idx) const = 0;
+  virtual MyMoneyTransaction transaction(const QString& account, const int idx) const = 0;
 
   /**
     * This method returns the number of institutions currently known to file
@@ -485,7 +517,7 @@ public:
     *
     * @return number of institutions known to file
     */
-  virtual unsigned int institutionCount(void) const = 0;
+  virtual uint institutionCount() const = 0;
 
   /**
     * This method returns a list of accounts inside the storage object.
@@ -500,31 +532,31 @@ public:
     * This method is used to return the standard liability account
     * @return MyMoneyAccount liability account(group)
     */
-  virtual const MyMoneyAccount liability(void) const = 0;
+  virtual MyMoneyAccount liability() const = 0;
 
   /**
     * This method is used to return the standard asset account
     * @return MyMoneyAccount asset account(group)
     */
-  virtual const MyMoneyAccount asset(void) const = 0;
+  virtual MyMoneyAccount asset() const = 0;
 
   /**
     * This method is used to return the standard expense account
     * @return MyMoneyAccount expense account(group)
     */
-  virtual const MyMoneyAccount expense(void) const = 0;
+  virtual MyMoneyAccount expense() const = 0;
 
   /**
     * This method is used to return the standard income account
     * @return MyMoneyAccount income account(group)
     */
-  virtual const MyMoneyAccount income(void) const = 0;
+  virtual MyMoneyAccount income() const = 0;
 
   /**
     * This method is used to return the standard equity account
     * @return MyMoneyAccount equity account(group)
     */
-  virtual const MyMoneyAccount equity(void) const = 0;
+  virtual MyMoneyAccount equity() const = 0;
 
   /**
     * This method is used to create a new security object.  The ID will be created
@@ -541,7 +573,7 @@ public:
     * This method is used to modify an existing MyMoneySecurity
     * object.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param security reference to the MyMoneySecurity object to be updated
     */
@@ -551,7 +583,7 @@ public:
     * This method is used to remove an existing MyMoneySecurity object
     * from the engine.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param security reference to the MyMoneySecurity object to be removed
     */
@@ -561,12 +593,12 @@ public:
     * This method is used to retrieve a single MyMoneySecurity object.
     * The id of the object must be supplied in the parameter @p id.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param id QString containing the id of the MyMoneySecurity object
     * @return MyMoneySecurity object
     */
-  virtual const MyMoneySecurity security(const QString& id) const = 0;
+  virtual MyMoneySecurity security(const QString& id) const = 0;
 
   /**
     * This method returns a list of the security objects
@@ -574,18 +606,18 @@ public:
     *
     * @return QList<MyMoneySecurity> containing objects
     */
-  virtual const QList<MyMoneySecurity> securityList(void) const = 0;
+  virtual QList<MyMoneySecurity> securityList() const = 0;
 
   virtual void addPrice(const MyMoneyPrice& price) = 0;
   virtual void removePrice(const MyMoneyPrice& price) = 0;
-  virtual const MyMoneyPrice price(const QString& fromId, const QString& toId, const QDate& date, const bool exactDate) const = 0;
+  virtual MyMoneyPrice price(const QString& fromId, const QString& toId, const QDate& date, bool exactDate) const = 0;
 
   /**
     * This method returns a list of all prices.
     *
     * @return MyMoneyPriceList of all MyMoneyPrice objects.
     */
-  virtual const MyMoneyPriceList priceList(void) const = 0;
+  virtual MyMoneyPriceList priceList() const = 0;
 
   /**
     * This method is used to add a scheduled transaction to the engine.
@@ -593,7 +625,7 @@ public:
     * method returns to the caller, the id will be filled with the
     * newly created object id value.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param sched reference to the MyMoneySchedule object
     */
@@ -603,7 +635,7 @@ public:
     * This method is used to modify an existing MyMoneySchedule
     * object. Therefor, the id attribute of the object must be set.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param sched const reference to the MyMoneySchedule object to be updated
     */
@@ -613,7 +645,7 @@ public:
     * This method is used to remove an existing MyMoneySchedule object
     * from the engine. The id attribute of the object must be set.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param sched const reference to the MyMoneySchedule object to be updated
     */
@@ -623,12 +655,12 @@ public:
     * This method is used to retrieve a single MyMoneySchedule object.
     * The id of the object must be supplied in the parameter @p id.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param id QString containing the id of the MyMoneySchedule object
     * @return MyMoneySchedule object
     */
-  virtual const MyMoneySchedule schedule(const QString& id) const = 0;
+  virtual MyMoneySchedule schedule(const QString& id) const = 0;
 
   /**
     * This method is used to extract a list of scheduled transactions
@@ -638,15 +670,15 @@ public:
     *                  accound @p accountId. If accountId is the empty string,
     *                  this filter is off. Default is @p QString().
     * @param type      only schedules of type @p type are searched for.
-    *                  See MyMoneySchedule::typeE for details.
-    *                  Default is MyMoneySchedule::TYPE_ANY
+    *                  See eMyMoney::Schedule::Type for details.
+    *                  Default is eMyMoney::Schedule::Type::Any
     * @param occurrence only schedules of occurrence type @p occurrence are searched for.
-    *                  See MyMoneySchedule::occurrenceE for details.
-    *                  Default is MyMoneySchedule::OCCUR_ANY
+    *                  See eMyMoney::Schedule::Occurrence for details.
+    *                  Default is eMyMoney::Schedule::Occurrence::Any
     * @param paymentType only schedules of payment method @p paymentType
     *                  are searched for.
-    *                  See MyMoneySchedule::paymentTypeE for details.
-    *                  Default is MyMoneySchedule::STYPE_ANY
+    *                  See eMyMoney::Schedule::PaymentType for details.
+    *                  Default is eMyMoney::Schedule::PaymentType::Any
     * @param startDate only schedules with payment dates after @p startDate
     *                  are searched for. Default is all dates (QDate()).
     * @param endDate   only schedules with payment dates ending prior to @p endDate
@@ -654,28 +686,28 @@ public:
     * @param overdue   if true, only those schedules that are overdue are
     *                  searched for. Default is false (all schedules will be returned).
     *
-    * @return const QList<MyMoneySchedule> list of schedule objects.
+    * @return QList<MyMoneySchedule> list of schedule objects.
     */
-  virtual const QList<MyMoneySchedule> scheduleList(const QString& accountId = QString(),
-      const MyMoneySchedule::typeE type = MyMoneySchedule::TYPE_ANY,
-      const MyMoneySchedule::occurrenceE occurrence = MyMoneySchedule::OCCUR_ANY,
-      const MyMoneySchedule::paymentTypeE paymentType = MyMoneySchedule::STYPE_ANY,
-      const QDate& startDate = QDate(),
-      const QDate& endDate = QDate(),
-      const bool overdue = false) const = 0;
+  virtual QList<MyMoneySchedule> scheduleList(const QString& accountId,
+      eMyMoney::Schedule::Type type,
+      eMyMoney::Schedule::Occurrence occurrence,
+      eMyMoney::Schedule::PaymentType paymentType,
+      const QDate& startDate,
+      const QDate& endDate,
+      bool overdue) const = 0;
 
-  virtual const QList<MyMoneySchedule> scheduleListEx(int scheduleTypes,
+  virtual QList<MyMoneySchedule> scheduleListEx(int scheduleTypes,
       int scheduleOcurrences,
       int schedulePaymentTypes,
       QDate startDate,
-      const QStringList& accounts = QStringList()) const = 0;
+      const QStringList& accounts) const = 0;
 
   /**
     * This method is used to add a new currency object to the engine.
     * The ID of the object is the trading symbol, so there is no need for an additional
     * ID since the symbol is guaranteed to be unique.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param currency reference to the MyMoneySecurity object
     */
@@ -685,7 +717,7 @@ public:
     * This method is used to modify an existing MyMoneySecurity
     * object.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param currency reference to the MyMoneyCurrency object
     */
@@ -695,7 +727,7 @@ public:
     * This method is used to remove an existing MyMoneySecurity object
     * from the engine.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param currency reference to the MyMoneySecurity object
     */
@@ -705,32 +737,32 @@ public:
     * This method is used to retrieve a single MyMoneySecurity object.
     * The id of the object must be supplied in the parameter @p id.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param id QString containing the id of the MyMoneySecurity object
     * @return MyMoneyCurrency object
     */
-  virtual const MyMoneySecurity currency(const QString& id) const = 0;
+  virtual MyMoneySecurity currency(const QString& id) const = 0;
 
   /**
     * This method is used to retrieve the list of all currencies
     * known to the engine.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @return QList of all MyMoneySecurity objects representing a currency.
     */
-  virtual const QList<MyMoneySecurity> currencyList(void) const = 0;
+  virtual QList<MyMoneySecurity> currencyList() const = 0;
 
   /**
     * This method is used to retrieve the list of all reports
     * known to the engine.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @return QList of all MyMoneyReport objects.
     */
-  virtual const QList<MyMoneyReport> reportList(void) const = 0;
+  virtual QList<MyMoneyReport> reportList() const = 0;
 
   /**
     * This method is used to add a new report to the engine.
@@ -738,7 +770,7 @@ public:
     * method returns to the caller, the id will be filled with the
     * newly created object id value.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param report reference to the MyMoneyReport object
     */
@@ -748,7 +780,7 @@ public:
     * This method is used to modify an existing MyMoneyReport
     * object. Therefor, the id attribute of the object must be set.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param report const reference to the MyMoneyReport object to be updated
     */
@@ -760,24 +792,24 @@ public:
     *
     * @return number of reports known to file
     */
-  virtual unsigned countReports(void) const = 0;
+  virtual uint countReports() const = 0;
 
   /**
     * This method is used to retrieve a single MyMoneyReport object.
     * The id of the object must be supplied in the parameter @p id.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param id QString containing the id of the MyMoneyReport object
     * @return MyMoneyReport object
     */
-  virtual const MyMoneyReport report(const QString& id) const = 0;
+  virtual MyMoneyReport report(const QString& id) const = 0;
 
   /**
     * This method is used to remove an existing MyMoneyReport object
     * from the engine. The id attribute of the object must be set.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param report const reference to the MyMoneyReport object to be updated
     */
@@ -787,11 +819,11 @@ public:
     * This method is used to retrieve the list of all budgets
     * known to the engine.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @return QList of all MyMoneyBudget objects.
     */
-  virtual const QList<MyMoneyBudget> budgetList(void) const = 0;
+  virtual QList<MyMoneyBudget> budgetList() const = 0;
 
   /**
     * This method is used to add a new budget to the engine.
@@ -799,7 +831,7 @@ public:
     * method returns to the caller, the id will be filled with the
     * newly created object id value.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param budget reference to the MyMoneyBudget object
     */
@@ -814,13 +846,13 @@ public:
     *
     * @return MyMoneyBudget object of budget
     */
-  virtual const MyMoneyBudget budgetByName(const QString& budget) const = 0;
+  virtual MyMoneyBudget budgetByName(const QString& budget) const = 0;
 
   /**
     * This method is used to modify an existing MyMoneyBudget
     * object. Therefor, the id attribute of the object must be set.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param budget const reference to the MyMoneyBudget object to be updated
     */
@@ -832,13 +864,13 @@ public:
     *
     * @return number of budgets known to file
     */
-  virtual unsigned countBudgets(void) const = 0;
+  virtual uint countBudgets() const = 0;
 
   /**
     * This method is used to retrieve a single MyMoneyBudget object.
     * The id of the object must be supplied in the parameter @p id.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param id QString containing the id of the MyMoneyBudget object
     * @return MyMoneyBudget object
@@ -849,38 +881,81 @@ public:
     * This method is used to remove an existing MyMoneyBudget object
     * from the engine. The id attribute of the object must be set.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param budget const reference to the MyMoneyBudget object to be updated
     */
   virtual void removeBudget(const MyMoneyBudget& budget) = 0;
 
 
+  /**
+    * This method is used to add a new onlineJob. The id will be
+    * overwritten.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param job The onlineJob, caller remains owner of it. Id might be updated.
+    */
+  virtual void addOnlineJob(onlineJob& job) = 0;
+
+  /**
+   * @brief Saves a onlineJob
+   * @param job
+   */
+  virtual void modifyOnlineJob(const onlineJob& job) = 0;
+
+  /**
+   * @brief Get onlineJob by id
+   *
+   * @return onlineJob you are not the owner nor allowed to delete it.
+   * @throw MyMoneyException if jobId was not found
+   */
+  virtual onlineJob getOnlineJob(const QString& jobId) const = 0;
+
+  /**
+   * @brief Return all onlineJobs
+   */
+  virtual QList<onlineJob> onlineJobList() const = 0;
+
+  /**
+   * @brief Remove an onlineJobs
+   */
+  virtual void removeOnlineJob(const onlineJob&) = 0;
+
+  /**
+   * @brief Returns a cost center by id
+   */
+  virtual MyMoneyCostCenter costCenter(const QString& id) const = 0;
+
+  /**
+   * @brief Retruns a list of all costcenters
+   */
+  virtual QList<MyMoneyCostCenter> costCenterList() const = 0;
 
   /**
     * This method checks, if the given @p object is referenced
     * by another engine object.
     *
     * @param obj const reference to object to be checked
-    * @param skipCheck MyMoneyFileBitArray with ReferenceCheckBits set for which
+    * @param skipCheck QBitArray with eStorage::Reference bits set for which
     *                  the check should be skipped
     *
     * @retval false @p object is not referenced
     * @retval true @p institution is referenced
     */
-  virtual bool isReferenced(const MyMoneyObject& obj, const MyMoneyFileBitArray& skipCheck = MyMoneyFileBitArray()) const = 0;
+  virtual bool isReferenced(const MyMoneyObject& obj, const QBitArray& skipCheck) const = 0;
 
   /**
     * This method is provided to allow closing of the database before logoff
     */
-  virtual void close(void) = 0;
+  virtual void close() = 0;
 
   /**
     * These methods have to be provided to allow transaction safe data handling.
     */
-  virtual void startTransaction(void) = 0;
-  virtual bool commitTransaction(void) = 0;
-  virtual void rollbackTransaction(void) = 0;
+  virtual void startTransaction() = 0;
+  virtual bool commitTransaction() = 0;
+  virtual void rollbackTransaction() = 0;
 };
 
 #endif
