@@ -21,7 +21,6 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 #include <QDate>
-#include <QtDebug>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -30,19 +29,27 @@
 // Project Includes
 #include "mymoneydbdriver.h"
 #include "mymoneyfile.h"
+#include "mymoneyaccount.h"
+#include "mymoneyexception.h"
+#include "imymoneystorage.h"
+
+#include <alkimia/alkvalue.h>
 
 //***************** THE CURRENT VERSION OF THE DATABASE LAYOUT ****************
-unsigned int MyMoneyDbDef::m_currentVersion = 7;
+unsigned int MyMoneyDbDef::m_currentVersion = 12;
 
 // ************************* Build table descriptions ****************************
 MyMoneyDbDef::MyMoneyDbDef()
 {
   FileInfo();
+  PluginInfo();
   Institutions();
   Payees();
+  PayeesPayeeIdentifier();
   Tags();
   TagSplits(); // a table to bind tags and splits
   Accounts();
+  AccountsPayeeIdentifier();
   Transactions();
   Splits();
   KeyValuePairs();
@@ -54,6 +61,9 @@ MyMoneyDbDef::MyMoneyDbDef()
   Reports();
   Budgets();
   Balances();
+  OnlineJobs();
+  PayeeIdentifier();
+  CostCenter();
 }
 
 /* PRIMARYKEY - these fields combine to form a unique key field on which the db will create an index
@@ -62,15 +72,14 @@ MyMoneyDbDef::MyMoneyDbDef()
    ?ISKEY - where there is no primary key, these fields can be used to uniquely identify a record
   Default is that a field is not a part of a primary key, nullable, and if numeric, signed */
 
-#define PRIMARYKEY true
-#define NOTNULL true
-#define UNSIGNED false
-//#define ISKEY true
+static const bool PRIMARYKEY = true;
+static const bool NOTNULL = true;
+static const bool UNSIGNED = false;
 
-#define appendField(a) fields.append(KSharedPtr<MyMoneyDbColumn>(new a))
-void MyMoneyDbDef::FileInfo(void)
+#define appendField(a) fields.append(QExplicitlySharedDataPointer<MyMoneyDbColumn>(new a))
+void MyMoneyDbDef::FileInfo()
 {
-  QList< KSharedPtr<MyMoneyDbColumn> > fields;
+  QList< QExplicitlySharedDataPointer<MyMoneyDbColumn> > fields;
   appendField(MyMoneyDbColumn("version", "varchar(16)"));
   appendField(MyMoneyDbColumn("created", "date"));
   appendField(MyMoneyDbColumn("lastModified", "date"));
@@ -101,6 +110,8 @@ void MyMoneyDbDef::FileInfo(void)
   appendField(MyMoneyDbColumn("updateInProgress", "char(1)"));
   appendField(MyMoneyDbIntColumn("budgets", MyMoneyDbIntColumn::BIG, UNSIGNED, false, false, 1));
   appendField(MyMoneyDbIntColumn("hiBudgetId", MyMoneyDbIntColumn::BIG, UNSIGNED, false, false, 1));
+  appendField(MyMoneyDbIntColumn("hiOnlineJobId", MyMoneyDbIntColumn::BIG, UNSIGNED, false, false, 8));
+  appendField(MyMoneyDbIntColumn("hiPayeeIdentifierId", MyMoneyDbIntColumn::BIG, UNSIGNED, false, false, 8));
   appendField(MyMoneyDbColumn("logonUser", "varchar(255)", false, false, 1));
   appendField(MyMoneyDbDatetimeColumn("logonAt", false, false, 1));
   appendField(MyMoneyDbIntColumn("fixLevel",
@@ -110,9 +121,9 @@ void MyMoneyDbDef::FileInfo(void)
   m_tables[t.name()] = t;
 }
 
-void MyMoneyDbDef::Institutions(void)
+void MyMoneyDbDef::Institutions()
 {
-  QList<KSharedPtr <MyMoneyDbColumn> > fields;
+  QList<QExplicitlySharedDataPointer <MyMoneyDbColumn> > fields;
   appendField(MyMoneyDbColumn("id", "varchar(32)", PRIMARYKEY, NOTNULL));
   appendField(MyMoneyDbTextColumn("name", MyMoneyDbTextColumn::NORMAL, false, NOTNULL));
   appendField(MyMoneyDbTextColumn("manager"));
@@ -126,9 +137,9 @@ void MyMoneyDbDef::Institutions(void)
   m_tables[t.name()] = t;
 }
 
-void MyMoneyDbDef::Payees(void)
+void MyMoneyDbDef::Payees()
 {
-  QList<KSharedPtr <MyMoneyDbColumn> > fields;
+  QList<QExplicitlySharedDataPointer <MyMoneyDbColumn> > fields;
   appendField(MyMoneyDbColumn("id", "varchar(32)",  PRIMARYKEY, NOTNULL));
   appendField(MyMoneyDbTextColumn("name"));
   appendField(MyMoneyDbTextColumn("reference"));
@@ -148,9 +159,21 @@ void MyMoneyDbDef::Payees(void)
   m_tables[t.name()] = t;
 }
 
-void MyMoneyDbDef::Tags(void)
+void MyMoneyDbDef::PayeesPayeeIdentifier()
 {
-  QList<KSharedPtr <MyMoneyDbColumn> > fields;
+  QList<QExplicitlySharedDataPointer <MyMoneyDbColumn> > fields;
+  appendField(MyMoneyDbColumn("payeeId", "varchar(32)",  PRIMARYKEY, NOTNULL, 8));
+  appendField(MyMoneyDbIntColumn("\"order\"", MyMoneyDbIntColumn::SMALL, UNSIGNED, PRIMARYKEY, NOTNULL, 8, 9));
+  appendField(MyMoneyDbIntColumn("userOrder", MyMoneyDbIntColumn::SMALL, UNSIGNED, PRIMARYKEY, NOTNULL, 10));
+  appendField(MyMoneyDbColumn("identifierId", "varchar(32)", false, NOTNULL, 8));
+  MyMoneyDbTable t("kmmPayeesPayeeIdentifier", fields);
+  t.buildSQLStrings();
+  m_tables[t.name()] = t;
+}
+
+void MyMoneyDbDef::Tags()
+{
+  QList<QExplicitlySharedDataPointer <MyMoneyDbColumn> > fields;
   appendField(MyMoneyDbColumn("id", "varchar(32)",  PRIMARYKEY, NOTNULL));
   appendField(MyMoneyDbTextColumn("name"));
   appendField(MyMoneyDbColumn("closed", "char(1)", false, false, 5));
@@ -161,9 +184,9 @@ void MyMoneyDbDef::Tags(void)
   m_tables[t.name()] = t;
 }
 
-void MyMoneyDbDef::TagSplits(void)
+void MyMoneyDbDef::TagSplits()
 {
-  QList<KSharedPtr <MyMoneyDbColumn> > fields;
+  QList<QExplicitlySharedDataPointer <MyMoneyDbColumn> > fields;
   appendField(MyMoneyDbColumn("transactionId", "varchar(32)", PRIMARYKEY, NOTNULL));
   appendField(MyMoneyDbColumn("tagId", "varchar(32)", PRIMARYKEY, NOTNULL));
   appendField(MyMoneyDbIntColumn("splitId", MyMoneyDbIntColumn::SMALL, UNSIGNED, PRIMARYKEY, NOTNULL));
@@ -172,9 +195,9 @@ void MyMoneyDbDef::TagSplits(void)
   m_tables[t.name()] = t;
 }
 
-void MyMoneyDbDef::Accounts(void)
+void MyMoneyDbDef::Accounts()
 {
-  QList<KSharedPtr <MyMoneyDbColumn> > fields;
+  QList<QExplicitlySharedDataPointer <MyMoneyDbColumn> > fields;
   appendField(MyMoneyDbColumn("id", "varchar(32)",  PRIMARYKEY, NOTNULL));
   appendField(MyMoneyDbColumn("institutionId", "varchar(32)"));
   appendField(MyMoneyDbColumn("parentId", "varchar(32)"));
@@ -196,9 +219,21 @@ void MyMoneyDbDef::Accounts(void)
   m_tables[t.name()] = t;
 }
 
-void MyMoneyDbDef::Transactions(void)
+void MyMoneyDbDef::AccountsPayeeIdentifier()
 {
-  QList<KSharedPtr <MyMoneyDbColumn> > fields;
+  QList<QExplicitlySharedDataPointer <MyMoneyDbColumn> > fields;
+  appendField(MyMoneyDbColumn("accountId", "varchar(32)",  PRIMARYKEY, NOTNULL, 8));
+  appendField(MyMoneyDbIntColumn("\"order\"", MyMoneyDbIntColumn::SMALL, UNSIGNED, PRIMARYKEY, NOTNULL, 8, 9));
+  appendField(MyMoneyDbIntColumn("userOrder", MyMoneyDbIntColumn::SMALL, UNSIGNED, PRIMARYKEY, NOTNULL, 10));
+  appendField(MyMoneyDbColumn("identifierId", "varchar(32)", false, NOTNULL, 8));
+  MyMoneyDbTable t("kmmAccountsPayeeIdentifier", fields);
+  t.buildSQLStrings();
+  m_tables[t.name()] = t;
+}
+
+void MyMoneyDbDef::Transactions()
+{
+  QList<QExplicitlySharedDataPointer <MyMoneyDbColumn> > fields;
   appendField(MyMoneyDbColumn("id", "varchar(32)", PRIMARYKEY, NOTNULL));
   appendField(MyMoneyDbColumn("txType", "char(1)"));
   appendField(MyMoneyDbDatetimeColumn("postDate"));
@@ -211,9 +246,9 @@ void MyMoneyDbDef::Transactions(void)
   m_tables[t.name()] = t;
 }
 
-void MyMoneyDbDef::Splits(void)
+void MyMoneyDbDef::Splits()
 {
-  QList<KSharedPtr <MyMoneyDbColumn> > fields;
+  QList<QExplicitlySharedDataPointer <MyMoneyDbColumn> > fields;
   appendField(MyMoneyDbColumn("transactionId", "varchar(32)",  PRIMARYKEY, NOTNULL));
   appendField(MyMoneyDbColumn("txType", "char(1)"));
   appendField(MyMoneyDbIntColumn("splitId", MyMoneyDbIntColumn::SMALL, UNSIGNED,  PRIMARYKEY, NOTNULL));
@@ -229,6 +264,7 @@ void MyMoneyDbDef::Splits(void)
   appendField(MyMoneyDbTextColumn("priceFormatted", MyMoneyDbTextColumn::MEDIUM, false, false, 2));
   appendField(MyMoneyDbTextColumn("memo"));
   appendField(MyMoneyDbColumn("accountId", "varchar(32)", false, NOTNULL));
+  appendField(MyMoneyDbColumn("costCenterId", "varchar(32)", false, false, 9));
   appendField(MyMoneyDbColumn("checkNumber", "varchar(32)"));
   appendField(MyMoneyDbDatetimeColumn("postDate", false, false, 1));
   appendField(MyMoneyDbTextColumn("bankId", MyMoneyDbTextColumn::MEDIUM, false, false, 5));
@@ -240,9 +276,9 @@ void MyMoneyDbDef::Splits(void)
   m_tables[t.name()] = t;
 }
 
-void MyMoneyDbDef::KeyValuePairs(void)
+void MyMoneyDbDef::KeyValuePairs()
 {
-  QList<KSharedPtr <MyMoneyDbColumn> > fields;
+  QList<QExplicitlySharedDataPointer <MyMoneyDbColumn> > fields;
   appendField(MyMoneyDbColumn("kvpType", "varchar(16)", false, NOTNULL));
   appendField(MyMoneyDbColumn("kvpId", "varchar(32)"));
   appendField(MyMoneyDbColumn("kvpKey", "varchar(255)", false, NOTNULL));
@@ -255,23 +291,24 @@ void MyMoneyDbDef::KeyValuePairs(void)
   m_tables[t.name()] = t;
 }
 
-void MyMoneyDbDef::Schedules(void)
+void MyMoneyDbDef::Schedules()
 {
-  QList<KSharedPtr <MyMoneyDbColumn> > fields;
+  QList<QExplicitlySharedDataPointer <MyMoneyDbColumn> > fields;
   appendField(MyMoneyDbColumn("id", "varchar(32)", PRIMARYKEY, NOTNULL));
   appendField(MyMoneyDbTextColumn("name", MyMoneyDbTextColumn::NORMAL, false, NOTNULL));
   appendField(MyMoneyDbIntColumn("type", MyMoneyDbIntColumn::TINY, UNSIGNED, false, NOTNULL));
   appendField(MyMoneyDbTextColumn("typeString"));
   appendField(MyMoneyDbIntColumn("occurence", MyMoneyDbIntColumn::SMALL, UNSIGNED, false, // krazy:exclude=spelling
                                  NOTNULL));
-  appendField(MyMoneyDbIntColumn("occurenceMultiplier", MyMoneyDbIntColumn::SMALL, UNSIGNED,
+  appendField(MyMoneyDbIntColumn("occurenceMultiplier", MyMoneyDbIntColumn::SMALL, UNSIGNED, // krazy:exclude=spelling
                                  false, NOTNULL, 3));
-  appendField(MyMoneyDbTextColumn("occurenceString"));
+  appendField(MyMoneyDbTextColumn("occurenceString")); // krazy:exclude=spelling
   appendField(MyMoneyDbIntColumn("paymentType", MyMoneyDbIntColumn::TINY, UNSIGNED));
   appendField(MyMoneyDbTextColumn("paymentTypeString", MyMoneyDbTextColumn::LONG));
   appendField(MyMoneyDbColumn("startDate", "date", false, NOTNULL));
   appendField(MyMoneyDbColumn("endDate", "date"));
   appendField(MyMoneyDbColumn("fixed", "char(1)", false, NOTNULL));
+  appendField(MyMoneyDbColumn("lastDayInMonth", "char(1)", false, NOTNULL, 11, std::numeric_limits<int>::max(), QLatin1String("N")));
   appendField(MyMoneyDbColumn("autoEnter", "char(1)", false, NOTNULL));
   appendField(MyMoneyDbColumn("lastPayment", "date"));
   appendField(MyMoneyDbColumn("nextPaymentDue", "date"));
@@ -283,9 +320,9 @@ void MyMoneyDbDef::Schedules(void)
   m_tables[t.name()] = t;
 }
 
-void MyMoneyDbDef::SchedulePaymentHistory(void)
+void MyMoneyDbDef::SchedulePaymentHistory()
 {
-  QList<KSharedPtr <MyMoneyDbColumn> > fields;
+  QList<QExplicitlySharedDataPointer <MyMoneyDbColumn> > fields;
   appendField(MyMoneyDbColumn("schedId", "varchar(32)", PRIMARYKEY, NOTNULL));
   appendField(MyMoneyDbColumn("payDate", "date", PRIMARYKEY,  NOTNULL));
   MyMoneyDbTable t("kmmSchedulePaymentHistory", fields);
@@ -293,25 +330,27 @@ void MyMoneyDbDef::SchedulePaymentHistory(void)
   m_tables[t.name()] = t;
 }
 
-void MyMoneyDbDef::Securities(void)
+void MyMoneyDbDef::Securities()
 {
-  QList<KSharedPtr <MyMoneyDbColumn> > fields;
+  QList<QExplicitlySharedDataPointer <MyMoneyDbColumn> > fields;
   appendField(MyMoneyDbColumn("id", "varchar(32)", PRIMARYKEY, NOTNULL));
   appendField(MyMoneyDbColumn("name", "text", false, NOTNULL));
   appendField(MyMoneyDbTextColumn("symbol"));
   appendField(MyMoneyDbIntColumn("type", MyMoneyDbIntColumn::SMALL, UNSIGNED, false, NOTNULL));
   appendField(MyMoneyDbTextColumn("typeString"));
   appendField(MyMoneyDbColumn("smallestAccountFraction", "varchar(24)"));
+  appendField(MyMoneyDbIntColumn("pricePrecision", MyMoneyDbIntColumn::SMALL, UNSIGNED, false, NOTNULL));
   appendField(MyMoneyDbTextColumn("tradingMarket"));
   appendField(MyMoneyDbColumn("tradingCurrency", "char(3)"));
+  appendField(MyMoneyDbIntColumn("roundingMethod", MyMoneyDbIntColumn::SMALL, UNSIGNED, false, NOTNULL, 11, std::numeric_limits<int>::max(), QString("%1").arg(AlkValue::RoundRound)));
   MyMoneyDbTable t("kmmSecurities", fields);
   t.buildSQLStrings();
   m_tables[t.name()] = t;
 }
 
-void MyMoneyDbDef::Prices(void)
+void MyMoneyDbDef::Prices()
 {
-  QList<KSharedPtr <MyMoneyDbColumn> > fields;
+  QList<QExplicitlySharedDataPointer <MyMoneyDbColumn> > fields;
   appendField(MyMoneyDbColumn("fromId", "varchar(32)", PRIMARYKEY, NOTNULL));
   appendField(MyMoneyDbColumn("toId", "varchar(32)",  PRIMARYKEY, NOTNULL));
   appendField(MyMoneyDbColumn("priceDate", "date", PRIMARYKEY, NOTNULL));
@@ -323,9 +362,9 @@ void MyMoneyDbDef::Prices(void)
   m_tables[t.name()] = t;
 }
 
-void MyMoneyDbDef::Currencies(void)
+void MyMoneyDbDef::Currencies()
 {
-  QList<KSharedPtr <MyMoneyDbColumn> > fields;
+  QList<QExplicitlySharedDataPointer <MyMoneyDbColumn> > fields;
   appendField(MyMoneyDbColumn("ISOcode", "char(3)", PRIMARYKEY, NOTNULL));
   appendField(MyMoneyDbTextColumn("name", MyMoneyDbTextColumn::NORMAL, false, NOTNULL));
   appendField(MyMoneyDbIntColumn("type", MyMoneyDbIntColumn::SMALL, UNSIGNED));
@@ -334,17 +373,18 @@ void MyMoneyDbDef::Currencies(void)
   appendField(MyMoneyDbIntColumn("symbol2", MyMoneyDbIntColumn::SMALL, UNSIGNED));
   appendField(MyMoneyDbIntColumn("symbol3", MyMoneyDbIntColumn::SMALL, UNSIGNED));
   appendField(MyMoneyDbColumn("symbolString", "varchar(255)"));
-  appendField(MyMoneyDbColumn("partsPerUnit", "varchar(24)"));
   appendField(MyMoneyDbColumn("smallestCashFraction", "varchar(24)"));
   appendField(MyMoneyDbColumn("smallestAccountFraction", "varchar(24)"));
+  // the default for price precision was taken from MyMoneySecurity
+  appendField(MyMoneyDbIntColumn("pricePrecision", MyMoneyDbIntColumn::SMALL, UNSIGNED, false, NOTNULL, 11, std::numeric_limits<int>::max(), QLatin1String("4")));
   MyMoneyDbTable t("kmmCurrencies", fields);
   t.buildSQLStrings();
   m_tables[t.name()] = t;
 }
 
-void MyMoneyDbDef::Reports(void)
+void MyMoneyDbDef::Reports()
 {
-  QList<KSharedPtr <MyMoneyDbColumn> > fields;
+  QList<QExplicitlySharedDataPointer <MyMoneyDbColumn> > fields;
   appendField(MyMoneyDbColumn("name", "varchar(255)", false, NOTNULL));
   appendField(MyMoneyDbTextColumn("XML", MyMoneyDbTextColumn::LONG));
   appendField(MyMoneyDbColumn("id", "varchar(32)", PRIMARYKEY, NOTNULL, 6));
@@ -353,9 +393,51 @@ void MyMoneyDbDef::Reports(void)
   m_tables[t.name()] = t;
 }
 
-void MyMoneyDbDef::Budgets(void)
+void MyMoneyDbDef::OnlineJobs()
 {
-  QList<KSharedPtr <MyMoneyDbColumn> > fields;
+  QList<QExplicitlySharedDataPointer <MyMoneyDbColumn> > fields;
+
+  appendField(MyMoneyDbColumn("id", "varchar(32)", PRIMARYKEY, NOTNULL, 8));
+  appendField(MyMoneyDbColumn("type", "varchar(255)", false, NOTNULL, 8));
+  appendField(MyMoneyDbDatetimeColumn("jobSend", false, false, 8));
+  appendField(MyMoneyDbDatetimeColumn("bankAnswerDate", false, false, 8));
+  appendField(MyMoneyDbColumn("state", "varchar(15)", false, NOTNULL, 8));
+  appendField(MyMoneyDbColumn("locked", "char(1)", false, NOTNULL, 8));
+
+  MyMoneyDbTable t("kmmOnlineJobs", fields);
+  t.buildSQLStrings();
+  m_tables[t.name()] = t;
+}
+
+void MyMoneyDbDef::PayeeIdentifier()
+{
+  QList<QExplicitlySharedDataPointer <MyMoneyDbColumn> > fields;
+
+  appendField(MyMoneyDbColumn("id", "varchar(32)", PRIMARYKEY, NOTNULL, 8));
+  appendField(MyMoneyDbColumn("type", "varchar(255)", false, false, 8));
+
+  MyMoneyDbTable t("kmmPayeeIdentifier", fields);
+  t.buildSQLStrings();
+  m_tables[t.name()] = t;
+}
+
+void MyMoneyDbDef::PluginInfo()
+{
+  QList<QExplicitlySharedDataPointer <MyMoneyDbColumn> > fields;
+
+  appendField(MyMoneyDbColumn("iid", "varchar(255)", PRIMARYKEY, NOTNULL, 8));
+  appendField(MyMoneyDbIntColumn("versionMajor", MyMoneyDbIntColumn::TINY, false, false, NOTNULL, 8));
+  appendField(MyMoneyDbIntColumn("versionMinor", MyMoneyDbIntColumn::TINY, false, false, false, 8));
+  appendField(MyMoneyDbTextColumn("uninstallQuery", MyMoneyDbTextColumn::LONG, false, false, 8));
+
+  MyMoneyDbTable t("kmmPluginInfo", fields);
+  t.buildSQLStrings();
+  m_tables[t.name()] = t;
+}
+
+void MyMoneyDbDef::Budgets()
+{
+  QList<QExplicitlySharedDataPointer <MyMoneyDbColumn> > fields;
   appendField(MyMoneyDbColumn("id", "varchar(32)", PRIMARYKEY, NOTNULL));
   appendField(MyMoneyDbColumn("name", "text", false, NOTNULL));
   appendField(MyMoneyDbColumn("start", "date", false, NOTNULL));
@@ -365,7 +447,17 @@ void MyMoneyDbDef::Budgets(void)
   m_tables[t.name()] = t;
 }
 
-void MyMoneyDbDef::Balances(void)
+void MyMoneyDbDef::CostCenter()
+{
+  QList<QExplicitlySharedDataPointer<MyMoneyDbColumn> > fields;
+  appendField(MyMoneyDbColumn("id", "varchar(32)", PRIMARYKEY, NOTNULL));
+  appendField(MyMoneyDbColumn("name", "text", false, NOTNULL));
+  MyMoneyDbTable t("kmmCostCenter", fields);
+  t.buildSQLStrings();
+  m_tables[t.name()] = t;
+}
+
+void MyMoneyDbDef::Balances()
 {
   MyMoneyDbView v("kmmBalances", "CREATE VIEW kmmBalances AS "
                   "SELECT kmmAccounts.id AS id, kmmAccounts.currencyId, "
@@ -379,8 +471,9 @@ void MyMoneyDbDef::Balances(void)
   m_views[v.name()] = v;
 }
 
+
 // function to write create SQL to a stream
-const QString MyMoneyDbDef::generateSQL(const KSharedPtr<MyMoneyDbDriver>& driver) const
+const QString MyMoneyDbDef::generateSQL(const QExplicitlySharedDataPointer<MyMoneyDbDriver>& driver) const
 {
   QString retval;
 
@@ -422,7 +515,10 @@ const QString MyMoneyDbDef::generateSQL(const KSharedPtr<MyMoneyDbDriver>& drive
                 + QLatin1Char('\'');
     if ((*fit)->name() == "updateInProgress")
       replace = enclose("N");
-    qs.replace(toReplace, replace);
+
+    qs.replace(QRegExp(toReplace + "(?=[,\\s\\)])"), replace);
+    // only replace parameters followed by comma, whitespace, closing parenthesis - otherwise
+    // conflicts may occur if one parameter starts with the name of another one.
   }
   qs += "\n\n";
   retval += qs;
@@ -449,7 +545,7 @@ const QString MyMoneyDbDef::generateSQL(const KSharedPtr<MyMoneyDbDriver>& drive
       toReplace.prepend(':');
       QString replace = "NULL";
       if ((*act)->name() == "accountType")
-        replace = QString::number(pac->accountType());
+        replace = QString::number((int)pac->accountType());
       if ((*act)->name() == "accountTypeString")
         replace = enclose(pac->name());
       if ((*act)->name() == "isStockAccount")
@@ -473,7 +569,7 @@ void MyMoneyDbTable::addIndex(const QString& name, const QStringList& columns, b
   m_indices.push_back(MyMoneyDbIndex(m_name, name, columns, unique));
 }
 
-void MyMoneyDbTable::buildSQLStrings(void)
+void MyMoneyDbTable::buildSQLStrings()
 {
   // build fixed SQL strings for this table
   // build the insert string with placeholders for each field
@@ -529,19 +625,20 @@ const QString MyMoneyDbTable::columnList(const int version) const
   QString qs;
   ft = m_fields.begin();
   while (ft != m_fields.end()) {
-    if ((*ft)->initVersion() <= version)
+    if ((*ft)->initVersion() <= version && (*ft)->lastVersion() >= version) {
       qs += QString("%1, ").arg((*ft)->name());
+    }
     ++ft;
   }
   return (qs.left(qs.length() - 2));
 }
 
-const QString MyMoneyDbTable::generateCreateSQL(const KSharedPtr<MyMoneyDbDriver>& driver, int version) const
+const QString MyMoneyDbTable::generateCreateSQL(const QExplicitlySharedDataPointer<MyMoneyDbDriver>& driver, int version) const
 {
   QString qs = QString("CREATE TABLE %1 (").arg(name());
   QString pkey;
   for (field_iterator it = m_fields.begin(); it != m_fields.end(); ++it) {
-    if ((*it)->initVersion() <= version) {
+    if ((*it)->initVersion() <= version && (*it)->lastVersion() >= version) {
       qs += (*it)->generateDDL(driver) + ", ";
       if ((*it)->isPrimaryKey())
         pkey += (*it)->name() + ", ";
@@ -564,7 +661,7 @@ const QString MyMoneyDbTable::generateCreateSQL(const KSharedPtr<MyMoneyDbDriver
   return qs;
 }
 
-const QString MyMoneyDbTable::dropPrimaryKeyString(const KSharedPtr<MyMoneyDbDriver>& driver) const
+const QString MyMoneyDbTable::dropPrimaryKeyString(const QExplicitlySharedDataPointer<MyMoneyDbDriver>& driver) const
 {
   return driver->dropPrimaryKeyString(m_name);
 }
@@ -573,7 +670,7 @@ bool MyMoneyDbTable::hasPrimaryKey(int version) const
 {
   field_iterator ft = m_fields.constBegin();
   while (ft != m_fields.constEnd()) {
-    if ((*ft)->initVersion() <= version) {
+    if ((*ft)->initVersion() <= version && (*ft)->lastVersion() >= version) {
       if ((*ft)->isPrimaryKey())
         return (true);
     }
@@ -582,7 +679,7 @@ bool MyMoneyDbTable::hasPrimaryKey(int version) const
   return (false);
 }
 
-const QString MyMoneyDbTable::modifyColumnString(const KSharedPtr<MyMoneyDbDriver>& driver, const QString& columnName, const MyMoneyDbColumn& newDef) const
+const QString MyMoneyDbTable::modifyColumnString(const QExplicitlySharedDataPointer<MyMoneyDbDriver>& driver, const QString& columnName, const MyMoneyDbColumn& newDef) const
 {
   return driver->modifyColumnString(m_name, columnName, newDef);
 }
@@ -597,7 +694,7 @@ int MyMoneyDbTable::fieldNumber(const QString& name) const
 }
 
 //*****************************************************************************
-const QString MyMoneyDbIndex::generateDDL(const KSharedPtr<MyMoneyDbDriver>& driver) const
+const QString MyMoneyDbIndex::generateDDL(const QExplicitlySharedDataPointer<MyMoneyDbDriver>& driver) const
 {
   Q_UNUSED(driver);
 
@@ -647,26 +744,31 @@ MyMoneyDbTextColumn* MyMoneyDbTextColumn::clone() const
   return (new MyMoneyDbTextColumn(*this));
 }
 
-const QString MyMoneyDbColumn::generateDDL(const KSharedPtr<MyMoneyDbDriver>& driver) const
+const QString MyMoneyDbColumn::generateDDL(const QExplicitlySharedDataPointer<MyMoneyDbDriver>& driver) const
 {
   Q_UNUSED(driver);
 
   QString qs = name() + ' ' + type();
   if (isNotNull()) qs += " NOT NULL";
+  if (!defaultValue().isEmpty())
+      qs += QString(" DEFAULT \"%1\"").arg(defaultValue());
   return qs;
 }
 
-const QString MyMoneyDbIntColumn::generateDDL(const KSharedPtr<MyMoneyDbDriver>& driver) const
+const QString MyMoneyDbIntColumn::generateDDL(const QExplicitlySharedDataPointer<MyMoneyDbDriver>& driver) const
 {
-  return driver->intString(*this);
+  QString qs = driver->intString(*this);
+  if (!defaultValue().isEmpty())
+      qs += QString(" DEFAULT %1").arg(defaultValue());
+  return qs;
 }
 
-const QString MyMoneyDbTextColumn::generateDDL(const KSharedPtr<MyMoneyDbDriver>& driver) const
+const QString MyMoneyDbTextColumn::generateDDL(const QExplicitlySharedDataPointer<MyMoneyDbDriver>& driver) const
 {
   return driver->textString(*this);
 }
 
-const QString MyMoneyDbDatetimeColumn::generateDDL(const KSharedPtr<MyMoneyDbDriver>& driver) const
+const QString MyMoneyDbDatetimeColumn::generateDDL(const QExplicitlySharedDataPointer<MyMoneyDbDriver>& driver) const
 {
   return driver->timestampString(*this);
 }
