@@ -31,6 +31,20 @@
 // Project Includes
 
 #include "mymoneystoragexml.h"
+#include "mymoneymoney.h"
+
+//class QDomElement;
+
+//class MyMoneyAccount;
+//class MyMoneyInstitution;
+//class MyMoneySecurity;
+//class MyMoneyBudget;
+//class MyMoneySchedule;
+//class MyMoneyPayee;
+//class MyMoneyTag;
+//class MyMoneyTransaction;
+class MyMoneyKeyValueContainer;
+//class onlineJob;
 
 /**
   * @author Kevin Tambascio (ktambascio@users.sourceforge.net)
@@ -86,6 +100,9 @@ protected:
 
   void writeSecurity(QDomElement& securityElement, const MyMoneySecurity& security);
 
+  /** Cannot remove prive data from plugins, yet. It is simply doing nothing. */
+  void writeOnlineJob(QDomElement& onlineJobs, const onlineJob& job);
+
   QDomElement findChildElement(const QString& name, const QDomElement& root);
 
 
