@@ -1,5 +1,5 @@
 /*
- * This file is part of KMyMoney, A Personal Finance Manager for KDE
+ * This file is part of KMyMoney, A Personal Finance Manager by KDE
  * Copyright (C) 2014 Christian Dávid <christian-david@web.de>
  *
  * This program is free software; you can redistribute it and/or
@@ -18,10 +18,8 @@
 
 #include "kibanlineedit.h"
 
-#include <QTimer>
-
 #include "ibanvalidator.h"
-
+#include "kmymoneyvalidationfeedback.h"
 
 KIbanLineEdit::KIbanLineEdit(QWidget* parent)
     : KLineEdit(parent)
