@@ -4,6 +4,7 @@
     begin                : Wed Jan 5 2005
     copyright            : (C) 2005 Thomas Baumgart
     email                : ipwizard@users.sourceforge.net
+                           (C) 2017 by Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
  ***************************************************************************/
 
 /***************************************************************************
@@ -27,7 +28,6 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
-class KMyMoneyApp;
 class MyMoneyAccount;
 class MyMoneyKeyValueContainer;
 
@@ -45,29 +45,27 @@ class KMMStatementInterface : public StatementInterface
   Q_OBJECT
 
 public:
-  KMMStatementInterface(KMyMoneyApp* app, QObject* parent, const char* name = 0);
+  explicit KMMStatementInterface(QObject* parent, const char* name = 0);
   ~KMMStatementInterface() {}
 
   /**
     * This method imports a MyMoneyStatement into the engine
     */
-  bool import(const MyMoneyStatement& s);
+  bool import(const MyMoneyStatement& s, bool silent = false);
 
   /**
    * This method returns the account for a given @a key - @a value pair.
    * If the account is not found in the list of accounts, MyMoneyAccount()
-   * is returned.
+   * is returned. The @a key - @a value pair can be in the account's kvp
+   * container or the account's online settings kvp container.
    */
-  const MyMoneyAccount& account(const QString& key, const QString& value) const;
+  MyMoneyAccount account(const QString& key, const QString& value) const;
 
   /**
    * This method stores the online parameters in @a kvps used by the plugin
    * with the account @a acc.
    */
   void setAccountOnlineParameters(const MyMoneyAccount&acc, const MyMoneyKeyValueContainer& kvps) const;
-
-private:
-  KMyMoneyApp*    m_app;
 };
 
 } // namespace
