@@ -20,60 +20,83 @@
 
 #include "reconciliationreport.h"
 
+//! @todo remove
+#include <QDebug>
+#include <QPointer>
+#include <QUrl>
+#include <QDate>
+
 // KDE includes
 #include <KPluginFactory>
-#include <KAction>
-#include <KGlobal>
-#include <KGlobalSettings>
-#include <KLocale>
-#include <KStandardDirs>
-#include <KPluginInfo>
 #include <KColorScheme>
+#include <KLocalizedString>
 
 // KMyMoney includes
 #include "mymoneyfile.h"
-#include "pluginloader.h"
+#include "mymoneyaccount.h"
+#include "mymoneypayee.h"
+#include "mymoneysecurity.h"
+#include "mymoneysplit.h"
+#include "mymoneytransaction.h"
+#include "mymoneytransactionfilter.h"
+#include "mymoneyutils.h"
+#include "viewinterface.h"
+#include "mymoneyenums.h"
+#include "kmymoneyglobalsettings.h"
 
 #include "kreconciliationreportdlg.h"
 
-K_PLUGIN_FACTORY(ReconciliationReportFactory, registerPlugin<KMMReconciliationReportPlugin>();)
-K_EXPORT_PLUGIN(ReconciliationReportFactory("kmm_reconciliationreport", "kmymoney"))
-
-KMMReconciliationReportPlugin::KMMReconciliationReportPlugin(QObject *parent, const QVariantList&)
-    : KMyMoneyPlugin::Plugin(parent, "Reconciliation report"/*must be the same as X-KDE-PluginInfo-Name*/)
+ReconciliationReport::ReconciliationReport(QObject *parent, const QVariantList &args) :
+  KMyMoneyPlugin::Plugin(parent, "reconciliationreport"/*must be the same as X-KDE-PluginInfo-Name*/)
 {
-  // For ease announce that we have been loaded.
-  qDebug("KMyMoney reconciliation report plugin loaded");
-
-  // connect to the pluginloader's relevant signals
-  connect(KMyMoneyPlugin::PluginLoader::instance(), SIGNAL(plug(KPluginInfo*)), this, SLOT(slotPlug(KPluginInfo*)));
-  connect(KMyMoneyPlugin::PluginLoader::instance(), SIGNAL(unplug(KPluginInfo*)), this, SLOT(slotUnplug(KPluginInfo*)));
-
+  Q_UNUSED(args);
+  qDebug("Plugins: reconciliation report loaded");
 }
 
-void KMMReconciliationReportPlugin::slotGenerateReconciliationReport(const MyMoneyAccount& account, const QDate& date, const MyMoneyMoney& startingBalance, const MyMoneyMoney& endingBalance, const QList<QPair<MyMoneyTransaction, MyMoneySplit> >& transactionList)
+ReconciliationReport::~ReconciliationReport()
+{
+  qDebug("Plugins: reconciliation report unloaded");
+}
+
+void ReconciliationReport::injectExternalSettings(KMyMoneySettings* p)
+{
+  KMyMoneyGlobalSettings::injectExternalSettings(p);
+}
+
+void ReconciliationReport::plug()
+{
+  connect(viewInterface(), &KMyMoneyPlugin::ViewInterface::accountReconciled, this, &ReconciliationReport::slotGenerateReconciliationReport);
+//  qDebug() << "Connect was done" << viewInterface();
+}
+
+void ReconciliationReport::unplug()
+{
+  disconnect(viewInterface(), &KMyMoneyPlugin::ViewInterface::accountReconciled, this, &ReconciliationReport::slotGenerateReconciliationReport);
+}
+
+void ReconciliationReport::slotGenerateReconciliationReport(const MyMoneyAccount& account, const QDate& date, const MyMoneyMoney& startingBalance, const MyMoneyMoney& endingBalance, const QList<QPair<MyMoneyTransaction, MyMoneySplit> >& transactionList)
 {
   MyMoneyFile* file = MyMoneyFile::instance();
   MyMoneySecurity currency = file->currency(account.currencyId());
 
   QString filename;
   if (!MyMoneyFile::instance()->value("reportstylesheet").isEmpty())
-    filename = KGlobal::dirs()->findResource("apps/kmymoney", QString("html/%1").arg(MyMoneyFile::instance()->value("reportstylesheet")));
+    filename = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QString("html/%1").arg(MyMoneyFile::instance()->value("reportstylesheet")));
   if (filename.isEmpty())
-    filename = KGlobal::dirs()->findResource("apps/kmymoney", "html/kmymoney.css");
+    filename = QStandardPaths::locate(QStandardPaths::GenericDataLocation, "html/kmymoney.css");
   QString header = QString("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\">\n") +
-                   QString("<html><head><link rel=\"stylesheet\" type=\"text/css\" href=\"%1\">").arg(filename);
+                   QString("<html><head><link rel=\"stylesheet\" type=\"text/css\" href=\"%1\">").arg(QUrl::fromLocalFile(filename).url());
 
   header += "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />";
 
   QColor tcolor = KColorScheme(QPalette::Active).foreground(KColorScheme::NormalText).color();
   QString css;
-  css += "<style type=\"text/css\">\n<!--\n";
-  css += QString(".row-even, .item0 { background-color: %1; color: %2 }\n")
-         .arg((KColorScheme(QPalette::Normal).background(KColorScheme::AlternateBackground).color()).name()).arg(tcolor.name());
-  css += QString(".row-odd, .item1  { background-color: %1; color: %2 }\n")
-         .arg((KColorScheme(QPalette::Normal).background(KColorScheme::NormalBackground).color()).name()).arg(tcolor.name());
-  css += "-->\n</style>\n";
+  css += "<style type=\"text/css\">\n<!--\n"
+      +  QString(".row-even, .item0 { background-color: %1; color: %2 }\n")
+         .arg((KColorScheme(QPalette::Normal).background(KColorScheme::AlternateBackground).color()).name(), tcolor.name())
+      +  QString(".row-odd, .item1  { background-color: %1; color: %2 }\n")
+         .arg((KColorScheme(QPalette::Normal).background(KColorScheme::NormalBackground).color()).name(), tcolor.name())
+      +  "-->\n</style>\n";
   header += css;
 
   header += "</head><body>\n";
@@ -90,7 +113,7 @@ void KMMReconciliationReportPlugin::slotGenerateReconciliationReport(const MyMon
   QList<QPair<MyMoneyTransaction, MyMoneySplit> >::const_iterator it;
   for (it = transactionList.begin(); it != transactionList.end(); ++it) {
     // if this split is a stock split, we can't just add the amount of shares
-    if ((*it).second.reconcileFlag() == MyMoneySplit::NotReconciled) {
+    if ((*it).second.reconcileFlag() == eMyMoney::Split::State::NotReconciled) {
       if ((*it).second.shares().isNegative()) {
         outstandingPayments++;
         outstandingPaymentAmount += (*it).second.shares();
@@ -113,7 +136,7 @@ void KMMReconciliationReportPlugin::slotGenerateReconciliationReport(const MyMon
   QString reportName = i18n("Reconciliation report of account %1", account.name());
   QString report = QString("<h2 class=\"report\">%1</h2>\n").arg(reportName);
   report += QString("<div class=\"subtitle\">");
-  report += QString("%1").arg(KGlobal::locale()->formatDate(date, KLocale::ShortDate));
+  report += QString("%1").arg(QLocale().toString(date, QLocale::ShortFormat));
   report += QString("</div>\n");
   report += QString("<div class=\"gap\">&nbsp;</div>\n");
   report += QString("<div class=\"subtitle\">");
@@ -173,7 +196,7 @@ void KMMReconciliationReportPlugin::slotGenerateReconciliationReport(const MyMon
   report += "</td></tr>";
   // row 8
   report += "<tr class=\"row-even\"><td class=\"left\">";
-  report += i18n("Register balance as of %1", KGlobal::locale()->formatDate(date, KLocale::ShortDate));
+  report += i18n("Register balance as of %1", QLocale().toString(date, QLocale::ShortFormat));
   report += "</td><td>";
   report += MyMoneyUtils::formatMoney(MyMoneyFile::instance()->balance(account.id(), date), currency);
   report += "</td></tr>";
@@ -181,8 +204,8 @@ void KMMReconciliationReportPlugin::slotGenerateReconciliationReport(const MyMon
   // retrieve list of all transactions after the reconciliation date that are not reconciled or cleared
   QList<QPair<MyMoneyTransaction, MyMoneySplit> > afterTransactionList;
   MyMoneyTransactionFilter filter(account.id());
-  filter.addState(MyMoneyTransactionFilter::cleared);
-  filter.addState(MyMoneyTransactionFilter::notReconciled);
+  filter.addState((int)eMyMoney::TransactionFilter::State::Cleared);
+  filter.addState((int)eMyMoney::TransactionFilter::State::NotReconciled);
   filter.setDateFilter(date.addDays(1), QDate());
   filter.setConsiderCategory(false);
   filter.setReportAllSplits(true);
@@ -193,7 +216,7 @@ void KMMReconciliationReportPlugin::slotGenerateReconciliationReport(const MyMon
   int afterPayments = 0;
   for (it = afterTransactionList.constBegin(); it != afterTransactionList.constEnd(); ++it) {
     // if this split is a stock split, we can't just add the amount of shares
-    if ((*it).second.reconcileFlag() == MyMoneySplit::NotReconciled) {
+    if ((*it).second.reconcileFlag() == eMyMoney::Split::State::NotReconciled) {
       if ((*it).second.shares().isNegative()) {
         afterPayments++;
         afterPaymentAmount += (*it).second.shares();
@@ -206,13 +229,13 @@ void KMMReconciliationReportPlugin::slotGenerateReconciliationReport(const MyMon
 
   // row 9
   report += "<tr class=\"row-odd\"><td class=\"left\">";
-  report += i18np("%1 payment after %2", "%1 payments after %2", afterPayments, KGlobal::locale()->formatDate(date, KLocale::ShortDate));
+  report += i18np("%1 payment after %2", "%1 payments after %2", afterPayments, QLocale().toString(date, QLocale::ShortFormat));
   report += "</td><td>";
   report += MyMoneyUtils::formatMoney(afterPaymentAmount, currency);
   report += "</td></tr>";
   // row 10
   report += "<tr class=\"row-even\"><td class=\"left\">";
-  report += i18np("%1 deposit after %2", "%1 deposits after %2", afterDeposits, KGlobal::locale()->formatDate(date, KLocale::ShortDate));
+  report += i18np("%1 deposit after %2", "%1 deposits after %2", afterDeposits, QLocale().toString(date, QLocale::ShortFormat));
   report += "</td><td>";
   report += MyMoneyUtils::formatMoney(afterDepositAmount, currency);
   report += "</td></tr>";
@@ -240,31 +263,30 @@ void KMMReconciliationReportPlugin::slotGenerateReconciliationReport(const MyMon
   QString detailsReport = QString("<h2 class=\"report\">%1</h2>\n").arg(i18n("Outstanding payments"));
   detailsReport += detailsTableHeader;
 
-  int index = 0;
-  for (it = transactionList.begin(); it != transactionList.end(); ++it) {
-    if ((*it).second.reconcileFlag() == MyMoneySplit::NotReconciled && (*it).second.shares().isNegative()) {
-      QList<MyMoneySplit>::const_iterator it_s;
+  auto index = 0;
+  foreach (const auto transaction, transactionList) {
+    if (transaction.second.reconcileFlag() == eMyMoney::Split::State::NotReconciled && transaction.second.shares().isNegative()) {
       QString category;
-      for (it_s = (*it).first.splits().begin(); it_s != (*it).first.splits().end(); ++it_s) {
-        if ((*it_s).accountId() != account.id()) {
+      foreach (const auto split, transaction.first.splits()) {
+        if (split.accountId() != account.id()) {
           if (!category.isEmpty())
             category += ", "; // this is a split transaction
-          category = file->account((*it_s).accountId()).name();
+          category = file->account(split.accountId()).name();
         }
       }
 
       detailsReport += QString("<tr class=\"%1\"><td>").arg((index++ % 2 == 1) ? "row-odd" : "row-even");
-      detailsReport += QString("%1").arg(KGlobal::locale()->formatDate((*it).first.entryDate(), KLocale::ShortDate));
+      detailsReport += QString("%1").arg(QLocale().toString(transaction.first.entryDate(), QLocale::ShortFormat));
       detailsReport += "</td><td>";
-      detailsReport += QString("%1").arg((*it).second.number());
+      detailsReport += QString("%1").arg(transaction.second.number());
       detailsReport += "</td><td>";
-      detailsReport += QString("%1").arg(file->payee((*it).second.payeeId()).name());
+      detailsReport += QString("%1").arg(file->payee(transaction.second.payeeId()).name());
       detailsReport += "</td><td>";
-      detailsReport += QString("%1").arg((*it).first.memo());
+      detailsReport += QString("%1").arg(transaction.first.memo());
       detailsReport += "</td><td>";
       detailsReport += QString("%1").arg(category);
       detailsReport += "</td><td>";
-      detailsReport += QString("%1").arg(MyMoneyUtils::formatMoney((*it).second.shares(), file->currency(account.currencyId())));
+      detailsReport += QString("%1").arg(MyMoneyUtils::formatMoney(transaction.second.shares(), file->currency(account.currencyId())));
       detailsReport += "</td></tr>";
     }
   }
@@ -278,54 +300,43 @@ void KMMReconciliationReportPlugin::slotGenerateReconciliationReport(const MyMon
 
   index = 0;
   for (it = transactionList.begin(); it != transactionList.end(); ++it) {
-    if ((*it).second.reconcileFlag() == MyMoneySplit::NotReconciled && !(*it).second.shares().isNegative()) {
-      QList<MyMoneySplit>::const_iterator it_s;
+    if ((*it).second.reconcileFlag() == eMyMoney::Split::State::NotReconciled && !(*it).second.shares().isNegative()) {
       QString category;
-      for (it_s = (*it).first.splits().begin(); it_s != (*it).first.splits().end(); ++it_s) {
-        if ((*it_s).accountId() != account.id()) {
+      foreach (const auto split, (*it).first.splits()) {
+        if (split.accountId() != account.id()) {
           if (!category.isEmpty())
             category += ", "; // this is a split transaction
-          category = file->account((*it_s).accountId()).name();
+          category = file->account(split.accountId()).name();
         }
       }
 
-      detailsReport += QString("<tr class=\"%1\"><td>").arg((index++ % 2 == 1) ? "row-odd" : "row-even");
-      detailsReport += QString("%1").arg(KGlobal::locale()->formatDate((*it).first.entryDate(), KLocale::ShortDate));
-      detailsReport += "</td><td>";
-      detailsReport += QString("%1").arg((*it).second.number());
-      detailsReport += "</td><td>";
-      detailsReport += QString("%1").arg(file->payee((*it).second.payeeId()).name());
-      detailsReport += "</td><td>";
-      detailsReport += QString("%1").arg((*it).first.memo());
-      detailsReport += "</td><td>";
-      detailsReport += QString("%1").arg(category);
-      detailsReport += "</td><td>";
-      detailsReport += QString("%1").arg(MyMoneyUtils::formatMoney((*it).second.shares(), file->currency(account.currencyId())));
-      detailsReport += "</td></tr>";
+      detailsReport += QString("<tr class=\"%1\"><td>").arg((index++ % 2 == 1) ? "row-odd" : "row-even")
+                    + QString("%1").arg(QLocale().toString((*it).first.entryDate(), QLocale::ShortFormat))
+                    + "</td><td>"
+                    + QString("%1").arg((*it).second.number())
+                    + "</td><td>"
+                    + QString("%1").arg(file->payee((*it).second.payeeId()).name())
+                    + "</td><td>"
+                    + QString("%1").arg((*it).first.memo())
+                    + "</td><td>"
+                    + QString("%1").arg(category)
+                    + "</td><td>"
+                    + QString("%1").arg(MyMoneyUtils::formatMoney((*it).second.shares(), file->currency(account.currencyId())))
+                    + "</td></tr>";
     }
   }
 
-  detailsReport += "<tr class=\"sectionfooter\">";
-  detailsReport += QString("<td class=\"left1\" colspan=\"5\">%1</td><td>%2</td></tr>").arg(i18np("One outstanding deposit of", "Total of %1 outstanding deposits amounting to", outstandingDeposits)).arg(MyMoneyUtils::formatMoney(outstandingDepositAmount, currency));
+  detailsReport += "<tr class=\"sectionfooter\">"
+                + QString("<td class=\"left1\" colspan=\"5\">%1</td><td>%2</td></tr>").arg(i18np("One outstanding deposit of", "Total of %1 outstanding deposits amounting to", outstandingDeposits)).arg(MyMoneyUtils::formatMoney(outstandingDepositAmount, currency))
 
   // end of the table
-  detailsReport += "</table>\n";
+                + "</table>\n";
 
   QPointer<KReportDlg> dlg = new KReportDlg(0, header + report + footer, header + detailsReport + footer);
   dlg->exec();
   delete dlg;
 }
 
-void KMMReconciliationReportPlugin::slotPlug(KPluginInfo* info)
-{
-  if (info->pluginName() == objectName()) {
-    connect(viewInterface(), SIGNAL(accountReconciled(MyMoneyAccount,QDate,MyMoneyMoney,MyMoneyMoney,QList<QPair<MyMoneyTransaction,MyMoneySplit> >)), this, SLOT(slotGenerateReconciliationReport(MyMoneyAccount,QDate,MyMoneyMoney,MyMoneyMoney,QList<QPair<MyMoneyTransaction,MyMoneySplit> >)));
-  }
-}
+K_PLUGIN_FACTORY_WITH_JSON(ReconciliationReportFactory, "reconciliationreport.json", registerPlugin<ReconciliationReport>();)
 
-void KMMReconciliationReportPlugin::slotUnplug(KPluginInfo* info)
-{
-  if (info->pluginName() == objectName()) {
-    disconnect(viewInterface(), SIGNAL(accountReconciled(MyMoneyAccount,QDate,MyMoneyMoney,MyMoneyMoney,QList<QPair<MyMoneyTransaction,MyMoneySplit> >)), this, SLOT(slotGenerateReconciliationReport(MyMoneyAccount,QDate,MyMoneyMoney,MyMoneyMoney,QList<QPair<MyMoneyTransaction,MyMoneySplit> >)));
-  }
-}
+#include "reconciliationreport.moc"
