@@ -2,6 +2,7 @@
  * This file is part of KMyMoney, A Personal Finance Manager by KDE
  * Copyright (C) 2014-2015 Romain Bignon <romain@symlink.me>
  * Copyright (C) 2014-2015 Florent Fourcot <weboob@flo.fourcot.fr>
+ * (C) 2017 by Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,27 +18,30 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef WEBOOB_HPP
-#define WEBOOB_HPP
+#ifndef WEBOOB_H
+#define WEBOOB_H
 
-#include <memory>
+// ----------------------------------------------------------------------------
+// QT Includes
 
-#ifdef HAVE_CONFIG_H
-#include <config-kmymoney.h>
-#endif
+// ----------------------------------------------------------------------------
+// KDE Includes
+
+// ----------------------------------------------------------------------------
+// Project Includes
 
 #include "kmymoneyplugin.h"
-#include "mymoneyaccount.h"
-#include "mymoneykeyvaluecontainer.h"
-#include "weboobext.h"
 
+class MyMoneyAccount;
+class MyMoneyKeyValueContainer;
+
+class WeboobPrivate;
 class Weboob : public KMyMoneyPlugin::Plugin, public KMyMoneyPlugin::OnlinePlugin
 {
   Q_OBJECT
   Q_INTERFACES(KMyMoneyPlugin::OnlinePlugin)
 
 public:
-  WeboobExt weboob;
   explicit Weboob(QObject *parent, const QVariantList &args);
   ~Weboob() override;
 
@@ -55,16 +59,13 @@ public:
   bool updateAccount(const MyMoneyAccount& acc, bool moreAccounts = false) override;
 
   void injectExternalSettings(KMyMoneySettings* p) override;
-
-protected Q_SLOTS:
-
-  void gotAccount();
-
+  
 private:
+  Q_DECLARE_PRIVATE(Weboob)
+  WeboobPrivate * const d_ptr;
 
-  struct Private;
-  /// \internal d-pointer instance.
-  const std::unique_ptr<Private> d;
+private Q_SLOTS:
+  void gotAccount();
 };
 
-#endif /* WEBOOB_PLUGIN_HPP */
+#endif
