@@ -19,16 +19,18 @@
 #ifndef KBALANCEAXIS_H
 #define KBALANCEAXIS_H
 
-#include <KDChartCartesianAxis>
+#include <qobjectdefs.h>
 
-using namespace KDChart;
+#include <KChartCartesianAxis>
 
-class KBalanceAxis : public KDChart::CartesianAxis
+namespace KChart { class AbstractCartesianDiagram; }
+
+class KBalanceAxis : public KChart::CartesianAxis
 {
   Q_OBJECT
 public:
   KBalanceAxis();
-  explicit KBalanceAxis(KDChart::AbstractCartesianDiagram* parent);
+  explicit KBalanceAxis(KChart::AbstractCartesianDiagram* parent);
   const QString customizedLabel(const QString& label) const;
 };
 
