@@ -34,9 +34,13 @@ const unsigned PivotOuterGroup::m_kDefaultSortOrder = 100;
 
 PivotCell::PivotCell(const MyMoneyMoney& value) :
     MyMoneyMoney(value),
-    m_stockSplit(MyMoneyMoney::ONE)
+    m_stockSplit(MyMoneyMoney::ONE),
+    m_cellUsed(!value.isZero())
+{
+}
+
+PivotCell::~PivotCell()
 {
-  m_cellUsed |= !value.isZero();
 }
 
 PivotCell PivotCell::operator += (const PivotCell& right)
