@@ -156,14 +156,18 @@ protected:
       return true;
 
     while (child) {
+      // if we are a child of the given parent, we have a match
       if (child == parent)
         return true;
+      // if we are at the application level, we don't have a match
+      if (child->inherits("KMyMoneyApp"))
+        return false;
       // If one of the ancestors is a KPassivePopup or a KDialog or a popup widget then
       // it's as if it is a child of our own because these widgets could
       // appear during transaction entry (message boxes, completer widgets)
       if (dynamic_cast<KPassivePopup*>(child) ||
           ((child->windowFlags() & Qt::Popup) && /*child != kmymoney*/
-           child->parentWidget())) // has no parent, then it must be top-level window like KMyMoneyApp
+           !child->parentWidget())) // has no parent, then it must be top-level window
         return true;
       child = child->parentWidget();
     }
@@ -383,14 +387,14 @@ public:
   void loadAccounts()
   {
     const auto file = MyMoneyFile::instance();
-    auto const model = Models::instance()->accountsModel();
 
     // check if the current account still exists and make it the
     // current account
-    if (!m_currentAccount.id().isEmpty()) {
+    if (!m_lastSelectedAccountID.isEmpty()) {
       try {
-        m_currentAccount = file->account(m_currentAccount.id());
+        m_currentAccount = file->account(m_lastSelectedAccountID);
       } catch (const MyMoneyException &) {
+        m_lastSelectedAccountID.clear();
         m_currentAccount = MyMoneyAccount();
         return;
       }
@@ -405,7 +409,7 @@ public:
 
     if (m_currentAccount.id().isEmpty()) {
       // find the first favorite account
-      QModelIndexList list = model->match(model->index(0, 0),
+      QModelIndexList list = m_filterProxyModel->match(m_filterProxyModel->index(0, 0),
                              (int)eAccountsModel::Role::Favorite,
                              QVariant(true),
                              1,
@@ -419,7 +423,7 @@ public:
 
       if (m_currentAccount.id().isEmpty()) {
         // there are no favorite accounts find any account
-        QModelIndexList list = model->match(model->index(0, 0),
+        QModelIndexList list = m_filterProxyModel->match(m_filterProxyModel->index(0, 0),
                                Qt::DisplayRole,
                                QVariant(QString("*")),
                                -1,
@@ -430,7 +434,7 @@ public:
           QVariant accountId = (*it).data((int)eAccountsModel::Role::ID);
           if (accountId.isValid()) {
             MyMoneyAccount a = file->account(accountId.toString());
-            if (!a.isInvest()) {
+            if (!a.isInvest() && !a.isClosed()) {
               m_currentAccount = a;
               break;
             }
@@ -939,6 +943,7 @@ public:
           }
       } else if (!list.isEmpty()) {
         sch = MyMoneyFile::instance()->schedule(list.first().scheduleId());
+        m_selectedTransactions.append(list.first());
         ret = false;
       }
 
@@ -1578,6 +1583,7 @@ public:
     * This member holds the currently selected account
     */
   MyMoneyAccount m_currentAccount;
+  QString m_lastSelectedAccountID;
 
   MyMoneyAccount m_reconciliationAccount;
 
