@@ -176,7 +176,7 @@ KMyMoneyView::KMyMoneyView(KMyMoneyApp *kmymoney)
   // Page 3
   m_scheduledView = new KScheduledView;
 //this is to solve the way long strings are handled differently among versions of KPageWidget
-  viewFrames[View::Schedules] = m_model->addPage(m_scheduledView, i18n("Scheduled transactions"));
+  viewFrames[View::Schedules] = m_model->addPage(m_scheduledView, i18nc("use \u2028 as line break", "Scheduled\u2028transactions"));
   viewFrames[View::Schedules]->setIcon(Icons::get(Icon::ViewSchedules));
   connect(m_scheduledView, &KMyMoneyViewBase::aboutToShow, this, &KMyMoneyView::connectView);
   connect(m_scheduledView, &KMyMoneyViewBase::aboutToShow, this, &KMyMoneyView::resetViewSelection);
@@ -248,6 +248,8 @@ KMyMoneyView::KMyMoneyView(KMyMoneyApp *kmymoney)
   connect(m_ledgerView, &KGlobalLedgerView::switchViewRequested, this, &KMyMoneyView::slotSwitchView);
   connect(m_homeView, &KHomeView::ledgerSelected, m_ledgerView, &KGlobalLedgerView::slotLedgerSelected);
 
+  connect(kmymoney, &KMyMoneyApp::transactionSelected, m_ledgerView, &KGlobalLedgerView::slotLedgerSelected);
+
 #ifdef ENABLE_UNFINISHEDFEATURES
   SimpleLedgerView* view = new SimpleLedgerView(kmymoney, this);
   KPageWidgetItem* frame = m_model->addPage(view, i18n("New ledger"));
@@ -394,8 +396,10 @@ void KMyMoneyView::slotShowForecastPage()
 
 void KMyMoneyView::slotShowOutboxPage()
 {
-  showPage(viewFrames[View::OnlineJobOutbox]);
-  m_onlineJobOutboxView->setDefaultFocus();
+  if (m_onlineJobOutboxView) {
+    showPage(viewFrames[View::OnlineJobOutbox]);
+    m_onlineJobOutboxView->setDefaultFocus();
+  }
 }
 
 void KMyMoneyView::showTitleBar(bool show)
@@ -475,8 +479,15 @@ void KMyMoneyView::slotAccountTreeViewChanged(const eAccountsModel::Column colum
 
 void KMyMoneyView::setOnlinePlugins(QMap<QString, KMyMoneyPlugin::OnlinePlugin*>& plugins)
 {
+  qDebug() << "Online plugins found" << plugins.count();
   m_accountsView->setOnlinePlugins(plugins);
-  m_onlineJobOutboxView->setOnlinePlugins(plugins);
+  if (m_onlineJobOutboxView) {
+    m_onlineJobOutboxView->setOnlinePlugins(plugins);
+  }
+  if (plugins.isEmpty()) {
+    m_model->removePage(viewFrames[View::OnlineJobOutbox]);
+    m_onlineJobOutboxView = nullptr;
+  }
 }
 
 eDialogs::ScheduleResultCode KMyMoneyView::enterSchedule(MyMoneySchedule& schedule, bool autoEnter, bool extendedKeys)
@@ -577,8 +588,10 @@ void KMyMoneyView::enableViewsIfFileOpen()
     viewFrames[View::Reports]->setEnabled(m_fileOpen);
   if (viewFrames[View::Forecast]->isEnabled() != m_fileOpen)
     viewFrames[View::Forecast]->setEnabled(m_fileOpen);
-  if (viewFrames[View::OnlineJobOutbox]->isEnabled() != m_fileOpen)
-    viewFrames[View::OnlineJobOutbox]->setEnabled(m_fileOpen);
+  if (m_onlineJobOutboxView) {
+    if (viewFrames[View::OnlineJobOutbox]->isEnabled() != m_fileOpen)
+      viewFrames[View::OnlineJobOutbox]->setEnabled(m_fileOpen);
+  }
   emit viewStateChanged(m_fileOpen);
 }
 
@@ -1482,7 +1495,8 @@ void KMyMoneyView::selectBaseCurrency()
       try {
         if (!(*it).currencyId().isEmpty() || (*it).currencyId().length() != 0)
           cid = MyMoneyFile::instance()->currency((*it).currencyId()).id();
-      } catch (const MyMoneyException &) {
+      } catch (const MyMoneyException& e) {
+        qDebug() << QLatin1String("Account") << (*it).id() << (*it).name() << e.what();
       }
 
       if (cid.isEmpty()) {
@@ -2180,6 +2194,7 @@ void KMyMoneyView::connectView(const View view)
       connect(m_ledgerView, &KGlobalLedgerView::openPayeeRequested,     this, &KMyMoneyView::slotPayeeSelected);
       connect(m_ledgerView, &KGlobalLedgerView::objectSelected,         this, &KMyMoneyView::slotObjectSelected);
       connect(m_ledgerView, &KGlobalLedgerView::transactionsSelected,   this, &KMyMoneyView::slotTransactionsSelected);
+      connect(m_ledgerView, &KGlobalLedgerView::contextMenuRequested,   this, &KMyMoneyView::slotContextMenuRequested);
       connect(m_ledgerView, &KGlobalLedgerView::transactionsContextMenuRequested,   this, &KMyMoneyView::slotTransactionsMenuRequested);
       connect(m_ledgerView, &KGlobalLedgerView::statusProgress,   this, &KMyMoneyView::statusProgress);
       connect(m_ledgerView, &KGlobalLedgerView::statusMsg,   this, &KMyMoneyView::statusMsg);
@@ -2256,7 +2271,9 @@ void KMyMoneyView::slotObjectSelected(const MyMoneyObject& obj)
     m_accountsView->updateActions(obj);
     m_ledgerView->updateActions(obj);
     m_reportsView->updateActions(obj);
-    m_onlineJobOutboxView->updateActions(obj);
+    if (m_onlineJobOutboxView) {
+      m_onlineJobOutboxView->updateActions(obj);
+    }
 
     // for plugin only
     const auto& acc = static_cast<const MyMoneyAccount&>(obj);
