@@ -1,8 +1,9 @@
 /***************************************************************************
                           kmymoneyview.h
                              -------------------
-    copyright            : (C) 2000-2001 by Michael Edwardes
-    email                : <mte@users.sourceforge.net>
+    copyright            : (C) 2000-2001 by Michael Edwardes <mte@users.sourceforge.net>
+                               2004 by Thomas Baumgart <ipwizard@users.sourceforge.net>
+                               2017 by Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
  ***************************************************************************/
 
 /***************************************************************************
@@ -14,47 +15,44 @@
  *                                                                         *
  ***************************************************************************/
 
-
-
 #ifndef KMYMONEYVIEW_H
 #define KMYMONEYVIEW_H
 
-#ifdef HAVE_CONFIG_H
 #include <config-kmymoney.h>
-#endif
 
 // ----------------------------------------------------------------------------
 // QT Includes
 
-#include <QWidget>
-#include <QList>
-#include <QVBoxLayout>
-
+#include <QFileDevice>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
 
-#include <ksavefile.h>
-#include <kmenu.h>
 #include <KPageWidget>
 
-#include <kurl.h>
-
 // ----------------------------------------------------------------------------
 // Project Includes
+#include "selectedtransactions.h"
 
-#include <mymoneyaccount.h>
-#include <mymoneyinstitution.h>
-#include <mymoneytransaction.h>
-#include <mymoneyscheduled.h>
-#include <mymoneysecurity.h>
-#include <selectedtransaction.h>
+#ifdef KF5Activities_FOUND
+namespace KActivities
+{
+class ResourceInstance;
+}
+#endif
 
+namespace eAccountsModel { enum class Column; }
+namespace eMenu { enum class Action; }
+namespace KMyMoneyPlugin { class OnlinePlugin; }
+namespace eDialogs { enum class ScheduleResultCode; }
+
+class KMyMoneyApp;
 class KHomeView;
 class KAccountsView;
 class KCategoriesView;
 class KInstitutionsView;
 class KPayeesView;
+class KTagsView;
 class KBudgetView;
 class KScheduledView;
 class KGlobalLedgerView;
@@ -62,11 +60,18 @@ class IMyMoneyStorageFormat;
 class MyMoneyTransaction;
 class KInvestmentView;
 class KReportsView;
-class KMyMoneyViewBase;
+class MyMoneySchedule;
+class MyMoneySecurity;
 class MyMoneyReport;
 class TransactionEditor;
 class KForecastView;
+class KOnlineJobOutbox;
 class KMyMoneyTitleLabel;
+class MyMoneyAccount;
+class MyMoneyMoney;
+class MyMoneyObject;
+class QLabel;
+
 /**
   * This class represents the view of the MyMoneyFile which contains
   * Banks/Accounts/Transactions, Recurring transactions (or Bills & Deposits)
@@ -77,23 +82,11 @@ class KMyMoneyTitleLabel;
   *
   * @short Handles the view of the MyMoneyFile.
   */
+enum class View;
 class KMyMoneyView : public KPageWidget
 {
   Q_OBJECT
 public:
-  enum viewID {
-    HomeView = 0,
-    AccountsView,
-    InstitutionsView,
-    SchedulesView,
-    CategoriesView,
-    PayeesView,
-    LedgersView,
-    InvestmentsView,
-    ReportsView,
-    BudgetView,
-    ForecastView
-  };
   // file actions for plugin
   enum fileActions {
     preOpen, postOpen, preSave, postSave, preClose, postClose
@@ -112,7 +105,7 @@ private:
     CategoryNew
   };
 
-  enum storageTypeE { // not used but keep for future implementation
+  enum storageTypeE {
     Memory = 0,
     Database
   } _storageType;
@@ -124,33 +117,21 @@ private:
   KInstitutionsView *m_institutionsView;
   KCategoriesView *m_categoriesView;
   KPayeesView *m_payeesView;
+  KTagsView *m_tagsView;
   KBudgetView *m_budgetView;
   KScheduledView *m_scheduledView;
   KGlobalLedgerView *m_ledgerView;
   KInvestmentView *m_investmentView;
   KReportsView* m_reportsView;
   KForecastView* m_forecastView;
+  KOnlineJobOutbox* m_onlineJobOutboxView;
 
-  KPageWidgetItem* m_homeViewFrame;
-  KPageWidgetItem* m_accountsViewFrame;
-  KPageWidgetItem* m_institutionsViewFrame;
-  KPageWidgetItem* m_categoriesViewFrame;
-  KPageWidgetItem* m_payeesViewFrame;
-  KPageWidgetItem* m_budgetViewFrame;
-  KPageWidgetItem* m_scheduleViewFrame;
-  KPageWidgetItem* m_ledgerViewFrame;
-  KPageWidgetItem* m_investmentViewFrame;
-  KPageWidgetItem* m_reportsViewFrame;
-  KPageWidgetItem* m_forecastViewFrame;
+  QHash<View,KPageWidgetItem*> viewFrames;
 
   KMyMoneyTitleLabel* m_header;
-
   bool m_inConstructor;
-
   bool m_fileOpen;
-
-  int  m_fmode;
-
+  QFileDevice::Permissions m_fmode;
   int m_lastViewSelected;
 
   // Keep a note of the file type
@@ -165,30 +146,18 @@ private:
   } fileTypeE;
   fileTypeE m_fileType;
 
+#ifdef KF5Activities_FOUND
+private:
+  KActivities::ResourceInstance * m_activityResourceInstance;
+#endif
+
 private:
   void ungetString(QIODevice *qfile, char * buf, int len);
 
-  /**
-    * This method creates the currency @p curr if it does not exist and
-    * @p create is @p true. If the currency already exists, it checks
-    * if the name is equal. If it is not, the name of the object in the
-    * engine is updated to the name passed with @p curr.
-    *
-    * @param curr MyMoneySecurity to be checked
-    * @param create If true and currency does not exist it will be created
-                    If false currency will not be created even if it does not exist
-    */
-  void loadDefaultCurrency(const MyMoneySecurity& curr, const bool create);
-
-  /**
-    *
-    */
-  void loadAncientCurrency(const QString& id, const QString& name, const QString& sym, const QDate& date, const MyMoneyMoney& rate, const QString& newId, const int partsPerUnit = 100, const int smallestCashFraction = 100, const int smallestAccountFraction = 0);
-
   /**
     * if no base currency is defined, start the dialog and force it to be set
     */
-  void selectBaseCurrency(void);
+  void selectBaseCurrency();
 
   /**
     * This method attaches an empty storage object to the MyMoneyFile
@@ -201,7 +170,7 @@ private:
     * This method removes an attached storage from the MyMoneyFile
     * object.
     */
-  void removeStorage(void);
+  void removeStorage();
 
   void viewAccountList(const QString& selectAccount); // Show the accounts view
 
@@ -209,10 +178,10 @@ private:
 
   /**
     */
-  void fixFile_0(void);
-  void fixFile_1(void);
-  void fixFile_2(void);
-  void fixFile_3(void);
+  void fixFile_0();
+  void fixFile_1();
+  void fixFile_2();
+  void fixFile_3();
 
   /**
     */
@@ -220,7 +189,7 @@ private:
 
   /**
     */
-  void fixTransactions_0(void);
+  void fixTransactions_0();
   void fixSchedule_0(MyMoneySchedule sched);
   void fixDuplicateAccounts_0(MyMoneyTransaction& t);
 
@@ -233,7 +202,7 @@ public:
     * The constructor for KMyMoneyView. Just creates all the tabs for the
     * different aspects of the MyMoneyFile.
     */
-  explicit KMyMoneyView(QWidget *parent = 0);
+  explicit KMyMoneyView(KMyMoneyApp *kmymoney);
 
   /**
     * Destructor
@@ -245,12 +214,12 @@ public:
     *
     * @return Whether the file is open and initialised
     */
-  bool fileOpen(void);
+  bool fileOpen();
 
   /**
     * Closes the open MyMoneyFile and frees all the allocated memory, I hope !
     */
-  void closeFile(void);
+  void closeFile();
 
 
   /**
@@ -263,7 +232,7 @@ public:
     *
     * @return Whether the read was successful.
     */
-  bool readFile(const KUrl& url);
+  bool readFile(const QUrl &url, IMyMoneyStorageFormat *pExtReader = nullptr);
 
   /**
     * Saves the data into permanent storage using the XML format.
@@ -277,7 +246,7 @@ public:
     * @retval false save operation failed
     * @retval true save operation was successful
     */
-  bool saveFile(const KUrl& url, const QString& keyList = QString());
+  bool saveFile(const QUrl &url, const QString& keyList = QString());
   /**
    * Saves the data into permanent storage on a new or empty SQL database.
    *
@@ -286,7 +255,7 @@ public:
    * @retval false save operation failed
    * @retval true save operation was successful
    */
-  //const bool saveDatabase(const KUrl& url); This no longer relevant
+  //const bool saveDatabase(const QUrl &url); This no longer relevant
   /**
    * Saves the data into permanent storage on a new or empty SQL database.
    *
@@ -295,7 +264,7 @@ public:
    * @retval false save operation failed
    * @retval true save operation was successful
    */
-  bool saveAsDatabase(const KUrl& url);
+  bool saveAsDatabase(const QUrl &url);
 
   /**
     * Call this to find out if the currently open file is native KMM
@@ -323,123 +292,61 @@ public:
     * @retval true if any data has been modified but not saved
     * @retval false otherwise
     */
-  bool dirty(void);
+  bool dirty();
 
   /**
     * Close the currently opened file and create an empty new file.
     *
     * @see MyMoneyFile
     */
-  void newFile(void);
+  void newFile();
 
   /**
-    * Moves the view up from transaction to Bank/Account view.
+    * This method enables the state of all views (except home view) according
+    * to an open file.
     */
-  void viewUp(void);
-
-  /**
-    * This method allows to set the enable state of all views (except home view)
-    * The argument @p state controls the availability.
-    *
-    * @param state Controls whether views are disabled @p (0), enabled @p (1) or
-    *              enabled/disabled according to an open file @p (-1). The latter
-    *              is the default.
-    */
-  void enableViews(int state = -1);
-
-  KMyMoneyViewBase* addBasePage(const QString& title, const QString& icon = QString());
+  void enableViewsIfFileOpen();
 
   void addWidget(QWidget* w);
 
   void showPage(KPageWidgetItem* pageItem);
 
-  /**
-    * check if the current view allows to create a transaction
-    *
-    * @param list list of selected transactions
-    * @param tooltip reference to string receiving the tooltip text
-    *        which explains why the modify function is not available (in case
-    *        of returning @c false)
-    *
-    * @retval true Yes, view allows to create a transaction (tooltip is not changed)
-    * @retval false No, view cannot to create a transaction (tooltip is updated with message)
-    */
-  bool canCreateTransactions(const KMyMoneyRegister::SelectedTransactions& list, QString& tooltip) const;
-
-  /**
-    * check if the current view allows to modify (edit/delete) the selected transactions
-    *
-    * @param list list of selected transactions
-    * @param tooltip reference to string receiving the tooltip text
-    *        which explains why the modify function is not available (in case
-    *        of returning @c false)
-    *
-    * @retval true Yes, view allows to edit/delete transactions (tooltip is not changed)
-    * @retval false No, view cannot edit/delete transactions (tooltip is updated with message)
-    */
-  bool canModifyTransactions(const KMyMoneyRegister::SelectedTransactions& list, QString& tooltip) const;
-
-  bool canDuplicateTransactions(const KMyMoneyRegister::SelectedTransactions& list, QString& tooltip) const;
-
-  /**
-  * check if the current view allows to edit the selected transactions
-  *
-  * @param list list of selected transactions
-  * @param tooltip reference to string receiving the tooltip text
-  *        which explains why the edit function is not available (in case
-  *        of returning @c false)
-  *
-  * @retval true Yes, view allows to enter/edit transactions
-  * @retval false No, view cannot enter/edit transactions
-  */
-  bool canEditTransactions(const KMyMoneyRegister::SelectedTransactions& list, QString& tooltip) const;
-
   /**
     * check if the current view allows to print something
     *
     * @retval true Yes, view allows to print
     * @retval false No, view cannot print
     */
-  bool canPrint(void);
+  bool canPrint();
 
-  TransactionEditor* startEdit(const KMyMoneyRegister::SelectedTransactions&);
-
-  bool createNewTransaction(void);
-
-  /**
-    * Used to start reconciliation of account @a account. It switches the
-    * ledger view into reconciliation mode and updates the view.
-    *
-    * @param account account which should be reconciled
-    * @param reconciliationDate the statement date
-    * @param endingBalance the ending balance entered for this account
-    *
-    * @retval true Reconciliation started
-    * @retval false Account cannot be reconciled
-    */
-  bool startReconciliation(const MyMoneyAccount& account, const QDate& reconciliationDate, const MyMoneyMoney& endingBalance);
-
-  /**
-    * Used to finish reconciliation of account @a account. It switches the
-    * ledger view to normal mode and updates the view.
-    *
-    * @param account account which should be reconciled
-    */
   void finishReconciliation(const MyMoneyAccount& account);
 
   /**
-    * This method preloads all known currencies into the engine.
+    * This method updates names of currencies from file to localized names
     */
-  void loadDefaultCurrencies(void);
+  void updateCurrencyNames();
 
-  void loadAncientCurrencies(void);
+  /**
+    * This method loads all known currencies and saves them to the storage
+    */
+  void loadAllCurrencies();
 
   void showTitleBar(bool show);
 
   /**
     * This method changes the view type according to the settings.
     */
-  void updateViewType(void);
+  void updateViewType();
+
+  void slotAccountTreeViewChanged(const eAccountsModel::Column column, const bool show);
+
+  void setOnlinePlugins(QMap<QString, KMyMoneyPlugin::OnlinePlugin*>& plugins);
+
+  // TODO: remove that function
+  /**
+   * ugly proxy function
+   */
+  eDialogs::ScheduleResultCode enterSchedule(MyMoneySchedule& schedule, bool autoEnter, bool extendedKeys);
 
 protected:
   /**
@@ -448,7 +355,7 @@ protected:
   virtual bool showPageHeader() const;
 
 
-public slots:
+public Q_SLOTS:
   /**
     * This slot writes information about the page passed as argument @a current
     * in the kmymoney.rc file so that it can be selected automatically when
@@ -469,16 +376,6 @@ public slots:
     **/
   void slotRefreshViews();
 
-  /**
-    * Called, whenever the ledger view should pop up and a specific
-    * transaction in an account should be shown. If @p transaction
-    * is empty, the last transaction should be selected
-    *
-    * @param acc The ID of the account to be shown
-    * @param transaction The ID of the transaction to be selected
-    */
-  void slotLedgerSelected(const QString& acc, const QString& transaction = QString());
-
   /**
     * Called, whenever the payees view should pop up and a specific
     * transaction in an account should be shown.
@@ -490,42 +387,20 @@ public slots:
   void slotPayeeSelected(const QString& payeeId, const QString& accountId, const QString& transactionId);
 
   /**
-    * Called, whenever the schedule view should pop up and a specific
-    * schedule should be shown.
+    * Called, whenever the tags view should pop up and a specific
+    * transaction in an account should be shown.
     *
-    * @param schedule The ID of the schedule to be shown
+    * @param tagId The ID of the tag to be shown
+    * @param accountId The ID of the account to be shown
+    * @param transactionId The ID of the transaction to be selected
     */
-  void slotScheduleSelected(const QString& schedule);
-
-  /**
-    * Called, whenever the report view should pop up and a specific
-    * report should be shown.
-    *
-    * @param reportid The ID of the report to be shown
-    */
-  void slotShowReport(const QString& reportid);
-
-  /**
-    * Same as the above, but the caller passes in an actual report
-    * definition to be shown.
-    *
-    * @param report The report to be shown
-    */
-  void slotShowReport(const MyMoneyReport& report);
+  void slotTagSelected(const QString& tagId, const QString& accountId, const QString& transactionId);
 
   /**
     * This slot prints the current view.
     */
-  void slotPrintView(void);
+  void slotPrintView();
 
-  /**
-    * This slot switches the view to present the home page
-    */
-  void slotShowHomePage(void) {
-    setCurrentPage(m_homeViewFrame);
-  }
-
-protected slots:
   /**
     * Called when the user changes the detail
     * setting of the transaction register
@@ -534,8 +409,54 @@ protected slots:
     */
   void slotShowTransactionDetail(bool detailed);
 
+
+
   /**
-   * eventually replace this with KMyMoneyApp::slotCurrencySetBase(void).
+   * Informs respective views about selected object, so they can
+   * update action states and current object.
+   * @param obj Account, Category, Investment, Stock, Institution
+   */
+  void slotObjectSelected(const MyMoneyObject& obj);
+
+  void slotTransactionsSelected(const KMyMoneyRegister::SelectedTransactions& list);
+
+private Q_SLOTS:
+  /**
+    * This slots switches the view to the specific page
+    */
+  void slotShowHomePage();
+  void slotShowInstitutionsPage();
+  void slotShowAccountsPage();
+  void slotShowSchedulesPage();
+  void slotShowCategoriesPage();
+  void slotShowTagsPage();
+  void slotShowPayeesPage();
+  void slotShowLedgersPage();
+  void slotShowInvestmentsPage();
+  void slotShowReportsPage();
+  void slotShowBudgetPage();
+  void slotShowForecastPage();
+  void slotShowOutboxPage();
+
+  /**
+   * Opens object in ledgers or edits in case of institution
+   * @param obj Account, Category, Investment, Stock, Institution
+   */
+  void slotOpenObjectRequested(const MyMoneyObject& obj);
+
+  /**
+   * Opens context menu based on objects's type
+   * @param obj Account, Category, Investment, Stock, Institution
+   */
+  void slotContextMenuRequested(const MyMoneyObject& obj);
+
+  void slotTransactionsMenuRequested(const KMyMoneyRegister::SelectedTransactions& list);
+
+  void slotSwitchView(View view);
+
+protected Q_SLOTS:
+  /**
+   * eventually replace this with KMyMoneyApp::slotCurrencySetBase().
    * it contains the same code
    *
    * @deprecated
@@ -547,12 +468,12 @@ private:
    * This method is called from readFile to open a database file which
    * is to be processed in 'proper' database mode, i.e. in-place updates
    *
-   * @param dbaseURL pseudo-KUrl representation of database
+   * @param dbaseURL pseudo-QUrl representation of database
    *
    * @retval true Database opened successfully
    * @retval false Could not open or read database
    */
-  bool openDatabase(const KUrl& dbaseURL);
+  bool openDatabase(const QUrl &dbaseURL);
   /**
    * This method is used after a file or database has been
    * read into storage, and performs various initialization tasks
@@ -583,7 +504,10 @@ private:
     */
   void accountNew(const bool createCategory);
 
-signals:
+  void resetViewSelection(const View);
+  void connectView(const View);
+
+Q_SIGNALS:
   /**
     * This signal is emitted whenever a view is selected.
     * The parameter @p view is identified as one of KMyMoneyView::viewID.
@@ -605,57 +529,42 @@ signals:
   void kmmFilePlugin(unsigned int action);
 
   /**
-    * Signal is emitted when reconciliation starts or ends. In case of end,
-    * @a account is MyMoneyAccount()
-    *
-    * @param account account for which reconciliation starts or MyMoneyAccount()
-    *                if reconciliation ends.
-    * @param reconciliationDate the statement date
-    * @param endingBalance collected ending balance when reconciliation starts
-    *                0 otherwise
-    */
-  void reconciliationStarts(const MyMoneyAccount& account, const QDate& reconciliationDate, const MyMoneyMoney& endingBalance);
-
-};
-
-/**
-  * This class is an abstract base class that all specific views
-  * should be based on.
-  */
-class KMyMoneyViewBase : public QWidget
-{
-  Q_OBJECT
-public:
-  KMyMoneyViewBase(QWidget* parent, const QString& name, const QString& title);
-  virtual ~KMyMoneyViewBase();
-
-  void setTitle(const QString& title);
-  QVBoxLayout* layout(void) const;
-  void addWidget(QWidget* w);
+   * This signal is emitted after a data source has been closed
+   */
+  void fileClosed();
 
   /**
-    * This method is used to edit the currently selected transactions
-    * The default implementation returns @p false which signals to the caller, that
-    * the view was not capable to edit the transactions.
-    *
-    * @retval false view was not capable to edit transactions
-    * @retval true view was capable to edit the transactions and did so
-    */
-  bool editTransactions(const QList<MyMoneyTransaction>& transactions) const {
-    Q_UNUSED(transactions)  return false;
-  }
+   * This signal is emitted after a data source has been opened
+   */
+  void fileOpened();
 
-signals:
   /**
-    * This signal is emitted whenever the view is about to be shown.
+   * @brief proxy signal
+   */
+  void statusMsg(const QString& txt);
+
+  /**
+   * @brief proxy signal
+   */
+  void statusProgress(int cnt, int base);
+
+  void accountReconciled(const MyMoneyAccount& account, const QDate& date, const MyMoneyMoney& startingBalance, const MyMoneyMoney& endingBalance, const QList<QPair<MyMoneyTransaction, MyMoneySplit> >& transactionList);
+
+  /**
+    * This signal is emitted when a transaction/list of transactions has been selected by
+    * the GUI. If no transaction is selected or the selection is removed,
+    * @p transactions is identical to an empty QList. This signal is used
+    * by plugins to get information about changes.
     */
-  void aboutToShow();
-
-private:
-  /// \internal d-pointer class.
-  class Private;
-  /// \internal d-pointer instance.
-  Private* const d;
+  void transactionsSelected(const KMyMoneyRegister::SelectedTransactions& transactions);
 
+  /**
+    * This signal is emitted when a new account has been selected by
+    * the GUI. If no account is selected or the selection is removed,
+    * @a account is identical to MyMoneyAccount(). This signal is used
+    * by plugins to get information about changes.
+    */
+  void accountSelected(const MyMoneyAccount& account);
 };
+
 #endif
