@@ -4,6 +4,7 @@
    begin                : Fri Jan 27 2006
    copyright            : (C) 2006 Tony Bloomfield
    email                : Tony Bloomfield <tonybloom@users.sourceforge.net>
+                          (C) 2017 by Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
 ***************************************************************************/
 
 /***************************************************************************
@@ -26,20 +27,24 @@
 #include <QWidget>
 #include <QSpinBox>
 #include <QApplication>
+#include <QListWidget>
+#include <QList>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
 
-#include <klistwidget.h>
-#include <kcombobox.h>
-#include <klineedit.h>
-#include <kurlrequester.h>
-#include <kmymoneyedit.h>
-#include "kmymoneymvccombo.h"
+#include <KComboBox>
+#include <KLineEdit>
+#include <KUrlRequester>
+#include "kmymoneyedit.h"
+
 // ----------------------------------------------------------------------------
 // Project Includes
 
 #include "kmymoneyglobalsettings.h"
+#include "onlinetasks/interfaces/ui/ionlinejobedit.h"
+#include "kmymoneytextedit.h"
+#include "kmymoneypayeecombo.h"
 
 /**************************************************************************
  *                                                                        *
@@ -53,29 +58,58 @@
  * With further widgets added by Allan Anderson for missing fields.       *
  **************************************************************************/
 
-void kMandatoryFieldGroup::add(QWidget *widget)
+class KMandatoryFieldGroupPrivate
 {
-  if (!m_widgets.contains(widget)) {
+  Q_DISABLE_COPY(KMandatoryFieldGroupPrivate)
+
+public:
+  KMandatoryFieldGroupPrivate() :
+    m_okButton(0),
+    m_enabled(true)
+  {
+  }
+
+  QList<QWidget *>      m_widgets;
+  QPushButton*          m_okButton;
+  bool                  m_enabled;
+};
+
+KMandatoryFieldGroup::KMandatoryFieldGroup(QObject *parent) :
+    QObject(parent),
+    d_ptr(new KMandatoryFieldGroupPrivate)
+{
+}
+
+KMandatoryFieldGroup::~KMandatoryFieldGroup()
+{
+  Q_D(KMandatoryFieldGroup);
+  delete d;
+}
+
+void KMandatoryFieldGroup::add(QWidget *widget)
+{
+  Q_D(KMandatoryFieldGroup);
+  if (!d->m_widgets.contains(widget)) {
     if (qobject_cast<QCheckBox*>(widget))
       connect(qobject_cast<QCheckBox*>(widget),
-              SIGNAL(clicked()),
-              this, SLOT(changed()));
+              &QCheckBox::clicked,
+              this, &KMandatoryFieldGroup::changed);
 
     else if (qobject_cast<KComboBox*>(widget)) {
       KComboBox* combo = qobject_cast<KComboBox*>(widget);
       KLineEdit* lineedit = qobject_cast<KLineEdit*>(combo->lineEdit());
       if (lineedit) {
-        connect(lineedit, SIGNAL(textChanged(QString)), this, SLOT(changed()));
+        connect(lineedit, &QLineEdit::textChanged, this, &KMandatoryFieldGroup::changed);
       } else {
-        connect(combo, SIGNAL(highlighted(int)), this, SLOT(changed()));
+        connect(combo, static_cast<void (QComboBox::*)(int)>(&QComboBox::highlighted), this, &KMandatoryFieldGroup::changed);
       }
     }
 
-    else if (qobject_cast<kMyMoneyEdit*>(widget)) {
-      kMyMoneyEdit* amount = qobject_cast<kMyMoneyEdit*>(widget);
+    else if (qobject_cast<KMyMoneyEdit*>(widget)) {
+      KMyMoneyEdit* amount = qobject_cast<KMyMoneyEdit*>(widget);
       KLineEdit* lineedit = qobject_cast<KLineEdit*>(amount->lineedit());
       if (lineedit) {
-        connect(lineedit, SIGNAL(textChanged(QString)), this, SLOT(changed()));
+        connect(lineedit, &QLineEdit::textChanged, this, &KMandatoryFieldGroup::changed);
       } else {
         connect(amount, SIGNAL(highlighted(int)), this, SLOT(changed()));
       }
@@ -83,58 +117,85 @@ void kMandatoryFieldGroup::add(QWidget *widget)
 
     else if (qobject_cast<QLineEdit*>(widget)) {
       connect(qobject_cast<QLineEdit*>(widget),
-              SIGNAL(textChanged(QString)),
-              this, SLOT(changed()));
+              &QLineEdit::textChanged,
+              this, &KMandatoryFieldGroup::changed);
     }
 
     else if (qobject_cast<QSpinBox*>(widget))
       connect(qobject_cast<QSpinBox*>(widget),
-              SIGNAL(valueChanged(QString)),
-              this, SLOT(changed()));
+              static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),
+              this, &KMandatoryFieldGroup::changed);
 
-    else if (qobject_cast<KListWidget*>(widget))
-      connect(qobject_cast<KListWidget*>(widget),
-              SIGNAL(itemSelectionChanged()),
-              this, SLOT(changed()));
+    else if (qobject_cast<QListWidget*>(widget))
+      connect(qobject_cast<QListWidget*>(widget),
+              &QListWidget::itemSelectionChanged,
+              this, &KMandatoryFieldGroup::changed);
 
     else if (qobject_cast<KUrlRequester*>(widget))
       connect(qobject_cast<KUrlRequester*>(widget),
-              SIGNAL(textChanged(QString)),
+              &KUrlRequester::textChanged,
+              this, &KMandatoryFieldGroup::changed);
+
+    else if (qobject_cast<KMyMoneyTextEdit*>(widget))
+      connect(qobject_cast<KMyMoneyTextEdit*>(widget),
+              &KMyMoneyTextEdit::textChanged,
+              this, &KMandatoryFieldGroup::changed);
+
+    else if (qobject_cast<IonlineJobEdit*>(widget)) {
+      connect(qobject_cast<IonlineJobEdit*>(widget),
+              SIGNAL(validityChanged(bool)),
               this, SLOT(changed()));
 
+      // Do not set palette for IonlineJobEdits as they contain subwidgets
+      d->m_widgets.append(widget);
+      changed();
+      return;
+    }
+
     else {
       qWarning("MandatoryFieldGroup: unsupported class %s", (widget->metaObject()->className()));
       return;
     }
 
     QPalette palette = widget->palette();
-    palette.setColor(QPalette::Base, KMyMoneyGlobalSettings::requiredFieldColor());
+    palette.setColor(QPalette::Base, KMyMoneyGlobalSettings::schemeColor(SchemeColor::FieldRequired));
     widget->setPalette(palette);
-    m_widgets.append(widget);
+    d->m_widgets.append(widget);
     changed();
   }
 }
 
-void kMandatoryFieldGroup::remove(QWidget *widget)
+void KMandatoryFieldGroup::removeAll()
 {
+  Q_D(KMandatoryFieldGroup);
+  while(!d->m_widgets.isEmpty()) {
+    remove(d->m_widgets.at(0));
+  }
+}
+
+void KMandatoryFieldGroup::remove(QWidget *widget)
+{
+  Q_D(KMandatoryFieldGroup);
   widget->setPalette(QApplication::palette());
-  m_widgets.removeOne(widget);
+  d->m_widgets.removeOne(widget);
   changed();
 }
 
-void kMandatoryFieldGroup::setOkButton(QPushButton *button)
+void KMandatoryFieldGroup::setOkButton(QPushButton *button)
 {
-  if (m_okButton && m_okButton != button)
-    m_okButton->setEnabled(true);
-  m_okButton = button;
+  Q_D(KMandatoryFieldGroup);
+  if (d->m_okButton && d->m_okButton != button)
+    d->m_okButton->setEnabled(true);
+  d->m_okButton = button;
   changed();
 }
 
-void kMandatoryFieldGroup::changed(void)
+void KMandatoryFieldGroup::changed()
 {
+  Q_D(KMandatoryFieldGroup);
   bool enable = true;
   QList<QWidget *>::ConstIterator i;
-  for (i = m_widgets.constBegin(); i != m_widgets.constEnd(); ++i) {
+  for (i = d->m_widgets.constBegin(); i != d->m_widgets.constEnd(); ++i) {
     QWidget *widget = *i;
     // disabled widgets don't count
     if (!(widget->isEnabled())) {
@@ -168,8 +229,8 @@ void kMandatoryFieldGroup::changed(void)
       } else
         continue;
     }
-    if ((qobject_cast<KListWidget*>(widget))) {
-      if ((qobject_cast<KListWidget*>(widget))->selectedItems().count() == 0) {
+    if ((qobject_cast<QListWidget*>(widget))) {
+      if ((qobject_cast<QListWidget*>(widget))->selectedItems().count() == 0) {
         enable = false;
         break;
       } else
@@ -182,32 +243,55 @@ void kMandatoryFieldGroup::changed(void)
       } else
         continue;
     }
-    if ((qobject_cast<kMyMoneyEdit*>(widget))) {
-      if ((qobject_cast<kMyMoneyEdit*>(widget))->text() == "0/1") {
+    if ((qobject_cast<KMyMoneyEdit*>(widget))) {
+      if ((qobject_cast<KMyMoneyEdit*>(widget))->text() == "0/1") {
         enable = false;
         break;
       } else
         continue;
     }
+    if (qobject_cast<KMyMoneyTextEdit*>(widget)) {
+      if (!(qobject_cast<KMyMoneyTextEdit*>(widget))->isValid()) {
+        enable = false;
+        break;
+      } else {
+        continue;
+      }
+    }
+    if (qobject_cast<IonlineJobEdit*>(widget)) {
+      if (!(qobject_cast<IonlineJobEdit*>(widget))->isValid()) {
+        enable = false;
+        break;
+      } else {
+        continue;
+      }
+    }
   }
 
-  if (m_okButton)
-    m_okButton->setEnabled(enable);
-  m_enabled = enable;
+  if (d->m_okButton)
+    d->m_okButton->setEnabled(enable);
+  d->m_enabled = enable;
 
   emit stateChanged();
   emit stateChanged(enable);
 }
 
-void kMandatoryFieldGroup::clear(void)
+bool KMandatoryFieldGroup::isEnabled() const
 {
+  Q_D(const KMandatoryFieldGroup);
+  return d->m_enabled;
+}
+
+void KMandatoryFieldGroup::clear()
+{
+  Q_D(KMandatoryFieldGroup);
   QList<QWidget *>::Iterator i;
-  for (i = m_widgets.begin(); i != m_widgets.end(); ++i)
+  for (i = d->m_widgets.begin(); i != d->m_widgets.end(); ++i)
     (*i)->setPalette(QApplication::palette());
-  m_widgets.clear();
-  if (m_okButton) {
-    m_okButton->setEnabled(true);
-    m_okButton = 0;
-    m_enabled = true;
+  d->m_widgets.clear();
+  if (d->m_okButton) {
+    d->m_okButton->setEnabled(true);
+    d->m_okButton = 0;
+    d->m_enabled = true;
   }
 }
