@@ -1,5 +1,6 @@
 /***************************************************************************
  *   Copyright 2010  Cristian Onet onet.cristian@gmail.com                 *
+ *   Copyright 2017  Łukasz Wojniłowicz lukasz.wojnilowicz@gmail.com       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or         *
  *   modify it under the terms of the GNU General Public License as        *
@@ -32,47 +33,42 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include "accountsmodel.h"
+class MyMoneyObject;
+class AccountsViewProxyModel;
+
+namespace eAccountsModel { enum class Column; }
+enum class View;
 
 /**
   * This view was created to handle the actions that could be performed with the accounts.
   */
+class KMyMoneyAccountTreeViewPrivate;
 class KMyMoneyAccountTreeView : public QTreeView
 {
   Q_OBJECT
+  Q_DISABLE_COPY(KMyMoneyAccountTreeView)
 
 public:
-  KMyMoneyAccountTreeView(QWidget *parent = 0);
+  explicit KMyMoneyAccountTreeView(QWidget* parent = nullptr);
   ~KMyMoneyAccountTreeView();
 
-public slots:
-  /**
-    * Overridden for internal reasons. @sa collapsedAll().
-    */
-  void collapseAll(void);
-
-  /**
-    * Overridden for internal reasons. @sa expandedAll().
-    */
-  void expandAll(void);
-
-  void setConfigGroupName(const QString& group);
+  AccountsViewProxyModel *init(View view);
 
 protected:
-  void mouseDoubleClickEvent(QMouseEvent *event);
-  void keyPressEvent(QKeyEvent *event);
+  void mouseDoubleClickEvent(QMouseEvent *event) override;
+  void keyPressEvent(QKeyEvent *event) override;
 
-protected slots:
-  void customContextMenuRequested(const QPoint &pos);
-  void selectionChanged(const QItemSelection &selected, const QItemSelection &deselected);
+protected Q_SLOTS:
+  void customContextMenuRequested(const QPoint);
+  void selectionChanged(const QItemSelection &selected, const QItemSelection &deselected) override;
 
-signals:
+Q_SIGNALS:
   /**
     * This signal serves as proxy for KMyMoneyAccountTree::selectObject()
     *
     * @param obj const reference to object
     */
-  void selectObject(const MyMoneyObject& obj);
+  void objectSelected(const MyMoneyObject& obj);
 
   /**
     * This signal serves as proxy for
@@ -80,7 +76,7 @@ signals:
     *
     * @param obj const reference to object
     */
-  void openContextMenu(const MyMoneyObject& obj);
+  void contextMenuRequested(const MyMoneyObject& obj);
 
   /**
     * This signal is emitted whenever the user requests to open an object
@@ -88,52 +84,13 @@ signals:
     * @param obj reference to actual MyMoneyObject (is either
     *            MyMoneyAccount or MyMoneyInstitution depending on selected item)
     */
-  void openObject(const MyMoneyObject& obj);
+  void openObjectRequested(const MyMoneyObject& obj);
 
-  /**
-    * collapsedAll() and expandedAll() are emitted when the resp.
-    * collapseAll() and expandAll() slots were invoked after the
-    * base functionality of QTreeView::collapseAll()/QTreeView::expandAll()
-    * has been finished.
-    */
-  void collapsedAll(void);
-  void expandedAll(void);
+  void columnToggled(const eAccountsModel::Column column, const bool show);
 
 private:
-  void openIndex(const QModelIndex &index);
-
-private:
-  QString m_groupName;
-};
-
-/**
-  * This model is specialized to organize the data for the accounts tree view
-  * based on the data of the @ref AccountsModel.
-  */
-class AccountsViewFilterProxyModel : public AccountsFilterProxyModel
-{
-  Q_OBJECT
-
-public:
-  AccountsViewFilterProxyModel(QObject *parent = 0);
-  ~AccountsViewFilterProxyModel();
-
-  virtual QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
-
-public slots:
-  void collapseAll(void);
-  void expandAll(void);
-
-protected:
-  bool filterAcceptsRow(int source_row, const QModelIndex &source_parent) const;
-
-protected slots:
-  void collapsed(const QModelIndex &index);
-  void expanded(const QModelIndex &index);
-
-private:
-  class Private;
-  Private* const d;
+  KMyMoneyAccountTreeViewPrivate * const d_ptr;
+  Q_DECLARE_PRIVATE(KMyMoneyAccountTreeView)
 };
 
 #endif // KMYMONEYACCOUNTTREEVIEW_H
