@@ -4,6 +4,7 @@
     begin                : Sat May 11 2008
     copyright            : (C) 2008 by Thomas Baumgart
     email                : Thomas Baumgart <ipwizard@users.sourceforge.net>
+                           (C) 2017 by Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
  ***************************************************************************/
 
 /***************************************************************************
@@ -16,43 +17,57 @@
  ***************************************************************************/
 
 #include "stdtransactionmatched.h"
+#include "stdtransaction_p.h"
 
 // ----------------------------------------------------------------------------
 // QT Includes
 
 #include <QList>
-#include <QHeaderView>
+#include <QPainter>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
 
-#include <klocale.h>
-#include <kdebug.h>
+#include <KLocalizedString>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include <kmymoneyglobalsettings.h>
-#include <register.h>
+#include "kmymoneyglobalsettings.h"
+#include "mymoneyaccount.h"
+#include "mymoneymoney.h"
+#include "mymoneysplit.h"
+#include "mymoneytransaction.h"
+#include "mymoneyexception.h"
+#include "widgetenums.h"
 
 using namespace KMyMoneyRegister;
 using namespace KMyMoneyTransactionForm;
 
 StdTransactionMatched::StdTransactionMatched(Register *parent, const MyMoneyTransaction& transaction, const MyMoneySplit& split, int uniqueId) :
-    StdTransaction(parent, transaction, split, uniqueId)
+  StdTransaction(parent, transaction, split, uniqueId)
 {
   // setup initial size
   setNumRowsRegister(numRowsRegister(KMyMoneyGlobalSettings::showRegisterDetailed()));
 }
 
-bool StdTransactionMatched::paintRegisterCellSetup(QPainter *painter, QStyleOptionViewItemV4 &option, const QModelIndex &index)
+StdTransactionMatched::~StdTransactionMatched()
 {
-  bool rc = Transaction::paintRegisterCellSetup(painter, option, index);
+}
+
+const char* StdTransactionMatched::className()
+{
+  return "StdTransactionMatched";
+}
+
+bool StdTransactionMatched::paintRegisterCellSetup(QPainter *painter, QStyleOptionViewItem &option, const QModelIndex &index)
+{
+  auto rc = Transaction::paintRegisterCellSetup(painter, option, index);
 
   // if not selected paint in matched background color
   if (!isSelected()) {
-    option.palette.setColor(QPalette::Base, KMyMoneyGlobalSettings::matchedTransactionColor());
-    option.palette.setColor(QPalette::AlternateBase, KMyMoneyGlobalSettings::matchedTransactionColor());
+    option.palette.setColor(QPalette::Base, KMyMoneyGlobalSettings::schemeColor(SchemeColor::TransactionMatched));
+    option.palette.setColor(QPalette::AlternateBase, KMyMoneyGlobalSettings::schemeColor(SchemeColor::TransactionMatched));
   }
   //TODO: the first line needs to be painted across all columns
   return rc;
@@ -60,16 +75,17 @@ bool StdTransactionMatched::paintRegisterCellSetup(QPainter *painter, QStyleOpti
 
 void StdTransactionMatched::registerCellText(QString& txt, Qt::Alignment& align, int row, int col, QPainter* painter)
 {
+  Q_D(StdTransaction);
   // run through the standard
   StdTransaction::registerCellText(txt, align, row, col, painter);
 
   // we only cover the additional rows
-  if (row >= m_rowsRegister - m_additionalRows) {
+  if (row >= RegisterItem::numRowsRegister() - m_additionalRows) {
     // make row relative to the last three rows
-    row += m_additionalRows - m_rowsRegister;
+    row += m_additionalRows - RegisterItem::numRowsRegister();
 
     // remove anything that had been added by the standard method
-    txt = "";
+    txt = QString();
 
     // and we draw this information in italics
     if (painter) {
@@ -78,19 +94,17 @@ void StdTransactionMatched::registerCellText(QString& txt, Qt::Alignment& align,
       painter->setFont(font);
     }
 
-    MyMoneyTransaction matchedTransaction = m_split.matchedTransaction();
+    MyMoneyTransaction matchedTransaction = d->m_split.matchedTransaction();
     MyMoneySplit matchedSplit;
     try {
-      matchedSplit = matchedTransaction.splitById(m_split.value("kmm-match-split"));
+      matchedSplit = matchedTransaction.splitById(d->m_split.value("kmm-match-split"));
     } catch (const MyMoneyException &) {
     }
 
-    QList<MyMoneySplit>::const_iterator it_s;
-    const QList<MyMoneySplit>& list = matchedTransaction.splits();
     MyMoneyMoney importedValue;
-    for (it_s = list.begin(); it_s != list.end(); ++it_s) {
-      if ((*it_s).accountId() == m_account.id()) {
-        importedValue += (*it_s).shares();
+    foreach (const auto split, matchedTransaction.splits()) {
+      if (split.accountId() == d->m_account.id()) {
+        importedValue += split.shares();
       }
     }
 
@@ -98,19 +112,19 @@ void StdTransactionMatched::registerCellText(QString& txt, Qt::Alignment& align,
     QString memo;
     switch (row) {
       case 0:
-        if (painter && col == DetailColumn)
-          txt = QString(" ") + i18n("KMyMoney has matched a downloaded transaction with a manually entered one (result above)");
+        if (painter && col == (int)eWidgets::eTransaction::Column::Detail)
+          txt = QString(" ") + i18n("KMyMoney has matched the two selected transactions (result above)");
         // return true for the first visible column only
         break;
 
       case 1:
         switch (col) {
-          case DateColumn:
+          case (int)eWidgets::eTransaction::Column::Date:
             align |= Qt::AlignLeft;
             txt = i18n("Bank entry:");
             break;
 
-          case DetailColumn:
+          case (int)eWidgets::eTransaction::Column::Detail:
             align |= Qt::AlignLeft;
             memo = matchedTransaction.memo();
             memo.replace("\n\n", "\n");
@@ -118,17 +132,17 @@ void StdTransactionMatched::registerCellText(QString& txt, Qt::Alignment& align,
             txt = QString("%1 %2").arg(matchedTransaction.postDate().toString(Qt::ISODate)).arg(memo);
             break;
 
-          case PaymentColumn:
+          case (int)eWidgets::eTransaction::Column::Payment:
             align |= Qt::AlignRight;
             if (importedValue.isNegative()) {
-              txt = (-importedValue).formatMoney(m_account.fraction());
+              txt = (-importedValue).formatMoney(d->m_account.fraction());
             }
             break;
 
-          case DepositColumn:
+          case (int)eWidgets::eTransaction::Column::Deposit:
             align |= Qt::AlignRight;
             if (!importedValue.isNegative()) {
-              txt = importedValue.formatMoney(m_account.fraction());
+              txt = importedValue.formatMoney(d->m_account.fraction());
             }
             break;
         }
@@ -136,18 +150,18 @@ void StdTransactionMatched::registerCellText(QString& txt, Qt::Alignment& align,
 
       case 2:
         switch (col) {
-          case DateColumn:
+          case (int)eWidgets::eTransaction::Column::Date:
             align |= Qt::AlignLeft;
             txt = i18n("Your entry:");
             break;
 
-          case DetailColumn:
+          case (int)eWidgets::eTransaction::Column::Detail:
             align |= Qt::AlignLeft;
-            postDate = m_transaction.postDate();
-            if (!m_split.value("kmm-orig-postdate").isEmpty()) {
-              postDate = QDate::fromString(m_split.value("kmm-orig-postdate"), Qt::ISODate);
+            postDate = d->m_transaction.postDate();
+            if (!d->m_split.value("kmm-orig-postdate").isEmpty()) {
+              postDate = QDate::fromString(d->m_split.value("kmm-orig-postdate"), Qt::ISODate);
             }
-            memo = m_split.memo();
+            memo = d->m_split.memo();
             if (!matchedSplit.memo().isEmpty() && memo != matchedSplit.memo()) {
               int pos = memo.lastIndexOf(matchedSplit.memo());
               if (pos != -1) {
@@ -160,17 +174,17 @@ void StdTransactionMatched::registerCellText(QString& txt, Qt::Alignment& align,
             txt = QString("%1 %2").arg(postDate.toString(Qt::ISODate)).arg(memo);
             break;
 
-          case PaymentColumn:
+          case (int)eWidgets::eTransaction::Column::Payment:
             align |= Qt::AlignRight;
-            if (m_split.value().isNegative()) {
-              txt = (-m_split.value(m_transaction.commodity(), m_splitCurrencyId)).formatMoney(m_account.fraction());
+            if (d->m_split.value().isNegative()) {
+              txt = (-d->m_split.value(d->m_transaction.commodity(), d->m_splitCurrencyId)).formatMoney(d->m_account.fraction());
             }
             break;
 
-          case DepositColumn:
+          case (int)eWidgets::eTransaction::Column::Deposit:
             align |= Qt::AlignRight;
-            if (!m_split.value().isNegative()) {
-              txt = m_split.value(m_transaction.commodity(), m_splitCurrencyId).formatMoney(m_account.fraction());
+            if (!d->m_split.value().isNegative()) {
+              txt = d->m_split.value(d->m_transaction.commodity(), d->m_splitCurrencyId).formatMoney(d->m_account.fraction());
             }
             break;
 
@@ -179,3 +193,13 @@ void StdTransactionMatched::registerCellText(QString& txt, Qt::Alignment& align,
     }
   }
 }
+
+int StdTransactionMatched::numRowsRegister(bool expanded) const
+{
+  return StdTransaction::numRowsRegister(expanded) + m_additionalRows;
+}
+
+int StdTransactionMatched::numRowsRegister() const
+{
+  return StdTransaction::numRowsRegister();
+}
