@@ -4,6 +4,7 @@
     begin                : Sat May 31 2008
     copyright            : (C) 2008 by Thomas Baumgart
     email                : Thomas Baumgart <ipwizard@users.sourceforge.net>
+                           (C) 2017 by Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
  ***************************************************************************/
 
 /***************************************************************************
@@ -20,7 +21,6 @@
 
 // ----------------------------------------------------------------------------
 // QT Includes
-#include <QBrush>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -28,46 +28,39 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include <transaction.h>
+#include "stdtransaction.h"
 
-namespace KMyMoneyTransactionForm
-{
-class TransactionForm;
-} // namespace
+class MyMoneySplit;
+class MyMoneyTransaction;
 
 namespace KMyMoneyRegister
 {
 
-class StdTransactionMatched : public StdTransaction
-{
-  static const int m_additionalRows = 3;
+  class Register;
+  class StdTransactionMatched : public StdTransaction
+  {
+    static const int m_additionalRows = 3;
 
-public:
-  StdTransactionMatched(Register* parent, const MyMoneyTransaction& transaction, const MyMoneySplit& split, int uniqueId);
-  virtual ~StdTransactionMatched() {}
+  public:
+    explicit StdTransactionMatched(Register* getParent, const MyMoneyTransaction& transaction, const MyMoneySplit& split, int uniqueId);
+    ~StdTransactionMatched() override;
 
-  virtual const char* className() {
-    return "StdTransactionMatched";
-  }
+    const char* className() override;
 
-  virtual bool paintRegisterCellSetup(QPainter *painter, QStyleOptionViewItemV4 &option, const QModelIndex &index);
+    bool paintRegisterCellSetup(QPainter *painter, QStyleOptionViewItem &option, const QModelIndex &index) override;
 
-  void registerCellText(QString& txt, Qt::Alignment& align, int row, int col, QPainter* painter = 0);
+    void registerCellText(QString& txt, Qt::Alignment& align, int row, int col, QPainter* painter = 0) override;
 
-  /**
+    /**
     * Provided for internal reasons. No API change. See RegisterItem::numRowsRegister(bool)
     */
-  int numRowsRegister(bool expanded) const {
-    return StdTransaction::numRowsRegister(expanded) + m_additionalRows;
-  }
+    int numRowsRegister(bool expanded) const override;
 
-  /**
+    /**
     * Provided for internal reasons. No API change. See RegisterItem::numRowsRegister()
     */
-  int numRowsRegister() const {
-    return StdTransaction::numRowsRegister();
-  }
-};
+    int numRowsRegister() const override;
+  };
 
 } // namespace
 
