@@ -26,8 +26,6 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
-#include <QBitArray>
-#include <QWidget>
 #include <QWizard>
 
 // ----------------------------------------------------------------------------
@@ -36,10 +34,12 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include "ui_knewloanwizarddecl.h"
-#include "mymoneyschedule.h"
-#include "kmymoneyaccountselector.h"
-#include "kmymoneydateinput.h"
+class QString;
+
+class MyMoneyAccount;
+class MyMoneySchedule;
+class MyMoneyAccountLoan;
+class MyMoneyTransaction;
 
 /**
   * @author Thomas Baumgart
@@ -54,15 +54,8 @@
   * created loan.
   *
   */
-class KNewLoanWizardDecl : public QWizard, public Ui::KNewLoanWizardDecl
-{
-public:
-  KNewLoanWizardDecl(QWidget *parent) : QWizard(parent) {
-    setupUi(this);
-  }
-};
-
-class KNewLoanWizard : public KNewLoanWizardDecl
+class KNewLoanWizardPrivate;
+class KNewLoanWizard : public QWizard
 {
   Q_OBJECT
 
@@ -82,7 +75,7 @@ public:
          Page_AssetAccount, Page_Summary
        };
 
-  KNewLoanWizard(QWidget *parent = 0);
+  explicit KNewLoanWizard(QWidget *parent = nullptr);
   ~KNewLoanWizard();
 
   /**
@@ -96,7 +89,7 @@ public:
     *
     * @return MyMoneySchedule object for payments
     */
-  MyMoneySchedule schedule(void) const;
+  MyMoneySchedule schedule() const;
 
   /**
     * This method returns the id of the account to/from which
@@ -106,7 +99,7 @@ public:
     *
     * @return id of account or empty QString
     */
-  QString initialPaymentAccount(void) const;
+  QString initialPaymentAccount() const;
 
   /**
     * This method returns the date of the payout transaction.
@@ -115,7 +108,7 @@ public:
     *
     * @return selected date or invalid QDate if checkbox is selected.
     */
-  QDate initialPaymentDate(void) const;
+  QDate initialPaymentDate() const;
 
   bool validateCurrentPage();
 
@@ -129,32 +122,12 @@ public:
    */
   int nextId() const;
 
-protected:
-  /**
-    * This method returns the transaction that is stored within
-    * the schedule. See schedule().
-    *
-    * @return MyMoneyTransaction object to be used within the schedule
-    */
-  MyMoneyTransaction transaction(void) const;
-
-protected slots:
+protected Q_SLOTS:
 
   // void slotNewPayee(const QString&);
-  void slotReloadEditWidgets(void);
+  void slotReloadEditWidgets();
 
-protected:
-  void loadAccountList(void);
-  void resetCalculator(void);
-  void updateLoanAmount(void);
-  void updateInterestRate(void);
-  void updateDuration(void);
-  void updatePayment(void);
-  void updateFinalPayment(void);
-  void updateLoanInfo(void);
-  int calculateLoan(void);
-
-signals:
+Q_SIGNALS:
   /**
     * This signal is emitted, when a new category name has been
     * entered by the user and this name is not known as account
@@ -181,10 +154,16 @@ signals:
   void createPayee(const QString& txt, QString& id);
 
 protected:
-  MyMoneyAccountLoan  m_account;
-  MyMoneyTransaction  m_transaction;
-  MyMoneySplit        m_split;
-  QBitArray           m_pages;
+  const QScopedPointer<KNewLoanWizardPrivate> d_ptr;
+  KNewLoanWizard(KNewLoanWizardPrivate &dd, QWidget *parent);
+   
+private:
+  Q_DISABLE_COPY(KNewLoanWizard)
+  Q_DECLARE_PRIVATE(KNewLoanWizard)
+
+private Q_SLOTS:
+  void slotNewCategory(MyMoneyAccount &acc);
+  void slotNewPayee(const QString& newnameBase, QString& id);
 };
 
 #endif
