@@ -20,14 +20,7 @@
 // QT Includes
 
 #include <QFile>
-#include <QObject>
 #include <QString>
-#include <QStringList>
-#include <QLinkedList>
-#include <QEventLoop>
-#include <QByteArray>
-
-#include <vector>
 
 class QDateTime;
 // ----------------------------------------------------------------------------
@@ -81,8 +74,8 @@ public:
   ~KGPGFile();
 
   virtual bool open(OpenMode mode);
-  virtual void close(void);
-  virtual void flush(void);
+  virtual void close();
+  virtual void flush();
 
   virtual qint64 readData(char *data, qint64 maxlen);
   virtual qint64 writeData(const char *data, qint64 maxlen);
@@ -122,7 +115,7 @@ public:
     * @retval true GPG can be started and returns a version number
     * @retval false GPG is not available
     */
-  static bool GPGAvailable(void);
+  static bool GPGAvailable();
 
   /**
     * Checks whether a key for a given user-id @p name exists.
