@@ -16,12 +16,17 @@
 
 #include "../kmymoney/mymoney/mymoneyaccount.h"
 
+#include <QDir>
 #include <QFile>
 #include <QStringList>
+#include <QMap>
 #include <QTextStream>
 #include <QXmlStreamReader>
-#include <QtDebug>
+#include <QDebug>
 
+#include "mymoneyenums.h"
+
+using namespace eMyMoney;
 
 QDebug operator <<(QDebug out, const QXmlStreamNamespaceDeclaration &a)
 {
@@ -46,24 +51,45 @@ QDebug operator <<(QDebug out, const QXmlStreamAttribute &a)
 bool debug = false;
 bool withID = false;
 bool noLevel1Names = false;
+bool withTax = false;
+bool prefixNameWithCode = false;
+
+typedef QMap<QString,QString> DirNameMapType;
+
+/**
+ * map to hold differences from gnucash to kmymoney template directory
+ * @return directory name map
+ */
+DirNameMapType &getDirNameMap()
+{
+    static DirNameMapType dirNameMap;
+    dirNameMap["cs"] = "cs_CZ";
+    dirNameMap["da"] = "dk";
+    dirNameMap["ja"] = "ja_JP";
+    dirNameMap["ko"] = "ko_KR";
+    dirNameMap["nb"] = "nb_NO";
+    dirNameMap["nl"] = "nl_NL";
+    dirNameMap["ru"] = "ru_RU";
+    return dirNameMap;
+}
 
 int toKMyMoneyAccountType(const QString &type)
 {
-    if(type == "ROOT") return MyMoneyAccount::UnknownAccountType;
-    else if (type == "BANK") return MyMoneyAccount::Checkings;
-    else if (type == "CASH") return MyMoneyAccount::Cash;
-    else if (type == "CREDIT") return MyMoneyAccount::CreditCard;
-    else if (type == "INVEST") return MyMoneyAccount::Investment;
-    else if (type == "RECEIVABLE") return MyMoneyAccount::Asset;
-    else if (type == "ASSET") return MyMoneyAccount::Asset;
-    else if (type == "PAYABLE") return MyMoneyAccount::Liability;
-    else if (type == "LIABILITY") return MyMoneyAccount::Liability;
-    else if (type == "CURRENCY") return MyMoneyAccount::Currency;
-    else if (type == "INCOME") return MyMoneyAccount::Income;
-    else if (type == "EXPENSE") return MyMoneyAccount::Expense;
-    else if (type == "STOCK") return MyMoneyAccount::Stock;
-    else if (type == "MUTUAL") return MyMoneyAccount::Stock;
-    else if (type == "EQUITY") return MyMoneyAccount::Equity;
+    if(type == "ROOT")              return (int)Account::Type::Unknown;
+    else if (type == "BANK")        return (int)Account::Type::Checkings;
+    else if (type == "CASH")        return (int)Account::Type::Cash;
+    else if (type == "CREDIT")      return (int)Account::Type::CreditCard;
+    else if (type == "INVEST")      return (int)Account::Type::Investment;
+    else if (type == "RECEIVABLE")  return (int)Account::Type::Asset;
+    else if (type == "ASSET")       return (int)Account::Type::Asset;
+    else if (type == "PAYABLE")     return (int)Account::Type::Liability;
+    else if (type == "LIABILITY")   return (int)Account::Type::Liability;
+    else if (type == "CURRENCY")    return (int)Account::Type::Currency;
+    else if (type == "INCOME")      return (int)Account::Type::Income;
+    else if (type == "EXPENSE")     return (int)Account::Type::Expense;
+    else if (type == "STOCK")       return (int)Account::Type::Stock;
+    else if (type == "MUTUAL")      return (int)Account::Type::Stock;
+    else if (type == "EQUITY")      return (int)Account::Type::Equity;
     else return 99; // unknown
 }
 
@@ -71,12 +97,14 @@ class TemplateAccount {
 public:
     typedef QList<TemplateAccount> List;
     typedef QList<TemplateAccount*> PointerList;
+    typedef QMap<QString,QString> SlotList;
 
     QString id;
     QString type;
     QString name;
     QString code;
     QString parent;
+    SlotList slotList;
 
     TemplateAccount()
     {
@@ -87,7 +115,8 @@ public:
         type(b.type),
         name(b.name),
         code(b.code),
-        parent(b.parent)
+        parent(b.parent),
+        slotList(b.slotList)
     {
     }
 
@@ -98,6 +127,43 @@ public:
         name = "";
         code = "";
         parent = "";
+        slotList.clear();
+    }
+
+    bool readSlots(QXmlStreamReader &xml)
+    {
+        while (!xml.atEnd()) {
+            QXmlStreamReader::TokenType type = xml.readNext();
+            if (type == QXmlStreamReader::StartElement) {
+                QStringRef _name = xml.name();
+                if (_name == "slot") {
+                    type = xml.readNext();
+                    if (type == QXmlStreamReader::Characters)
+                        type = xml.readNext();
+                    if (type == QXmlStreamReader::StartElement) {
+                        QStringRef name = xml.name();
+                        QString key, value;
+                        if (name == "key")
+                            key = xml.readElementText(QXmlStreamReader::SkipChildElements).trimmed();
+                        type = xml.readNext();
+                        if (type == QXmlStreamReader::Characters)
+                            type = xml.readNext();
+                        if (type == QXmlStreamReader::StartElement) {
+                            name = xml.name();
+                            if (name == "value")
+                                value = xml.readElementText(QXmlStreamReader::SkipChildElements).trimmed();
+                        }
+                        if (!key.isEmpty() && !value.isEmpty())
+                            slotList[key] = value;
+                    }
+                }
+            } else if (type == QXmlStreamReader::EndElement) {
+                QStringRef _name = xml.name();
+                if (_name  == "slots")
+                    return true;
+            }
+        }
+        return true;
     }
 
     bool read(QXmlStreamReader &xml)
@@ -105,21 +171,25 @@ public:
         while (!xml.atEnd()) {
             xml.readNext();
             QStringRef _name = xml.name();
-            if (xml.isEndElement() && _name == "account")
+            if (xml.isEndElement() && _name == "account") {
+                if (prefixNameWithCode && !code.isEmpty() && !name.startsWith(code))
+                    name = code + ' ' + name;
                 return true;
+            }
             if (xml.isStartElement())
             {
-                QString value = xml.readElementText(QXmlStreamReader::SkipChildElements).trimmed();
                 if (_name == "name")
-                    name = value;
+                    name = xml.readElementText(QXmlStreamReader::SkipChildElements).trimmed();
                 else if (_name == "id")
-                    id = value;
+                    id = xml.readElementText(QXmlStreamReader::SkipChildElements).trimmed();
                 else if (_name == "type")
-                    type = value;
+                    type = xml.readElementText(QXmlStreamReader::SkipChildElements).trimmed();
                 else if (_name == "code")
-                    code = value;
+                    code = xml.readElementText(QXmlStreamReader::SkipChildElements).trimmed();
                 else if (_name == "parent")
-                    parent = value;
+                    parent = xml.readElementText(QXmlStreamReader::SkipChildElements).trimmed();
+                else if (_name == "slots")
+                    readSlots(xml);
                 else
                 {
                     if (debug)
@@ -139,6 +209,7 @@ QDebug operator <<(QDebug out, const TemplateAccount &a)
         << "type:" << a.type
         << "code:" << a.code
         << "parent:" << a.parent
+        << "slotList:" << a.slotList
         << ")\n";
     return out;
 }
@@ -222,6 +293,14 @@ public:
                 xml.writeAttribute("name", noLevel1Names && index < 2 ? "" : account->name);
                 if (withID)
                     xml.writeAttribute("id", account->id);
+                if (withTax) {
+                    if (account->slotList.contains("tax-related")) {
+                        xml.writeStartElement("flag");
+                        xml.writeAttribute("name","Tax");
+                        xml.writeAttribute("value",account->slotList["tax-related"]);
+                        xml.writeEndElement();
+                    }
+                }
             }
             index++;
             writeAccountsAsXml(xml, account->id, index);
@@ -393,13 +472,14 @@ protected:
 
     bool writeAsXml(QIODevice *device)
     {
-            QXmlStreamWriter xml(device);
+        QXmlStreamWriter xml(device);
         xml.setAutoFormatting(true);
         xml.setAutoFormattingIndent(1);
         xml.setCodec("utf-8");
         xml.writeStartDocument();
 
-        QString fileName = inFileName.replace(QRegExp(".*/accounts"),"accounts");
+        QString fileName = inFileName;
+        fileName.replace(QRegExp(".*/accounts"),"accounts");
         xml.writeComment(QString("\n"
             "     Converted using xea2kmt from GnuCash sources\n"
             "\n"
@@ -421,20 +501,113 @@ protected:
     QString inFileName;
 };
 
+void scanDir(QDir dir, QStringList &files)
+{
+    dir.setNameFilters(QStringList("*.gnucash-xea"));
+    dir.setFilter(QDir::Files | QDir::NoDotAndDotDot | QDir::NoSymLinks);
+
+    if (debug)
+        qDebug() << "Scanning: " << dir.path();
+
+    QStringList fileList = dir.entryList();
+    for (int i=0; i<fileList.count(); i++)
+    {
+        if (debug)
+            qDebug() << "Found file: " << fileList[i];
+        files.append(QString("%1/%2").arg(dir.absolutePath()).arg(fileList[i]));
+    }
+
+    dir.setFilter(QDir::AllDirs | QDir::NoDotAndDotDot | QDir::NoSymLinks);
+    QStringList dirList = dir.entryList();
+    for (int i=0; i<dirList.size(); ++i)
+    {
+        QString newPath = QString("%1/%2").arg(dir.absolutePath()).arg(dirList.at(i));
+        scanDir(QDir(newPath), files);
+    }
+}
+
+bool convertFile(const QString &inFile, const QString &outFile)
+{
+    GnuCashAccountTemplateReader reader;
+    if (!reader.read(inFile))
+        return false;
+    return reader.writeAsXml(outFile);
+}
+
+int convertFileStructure(const QString &indir, const QString &outdir)
+{
+    DirNameMapType &dirNameMap = getDirNameMap();
+    // get gnucash account files
+    QDir d(indir);
+    QStringList files;
+    scanDir(d, files);
+
+    QString inPath = d.absolutePath();
+    QDir outDir(outdir);
+    QString outPath = outDir.absolutePath();
+    QStringList mapKeys = dirNameMap.keys();
+    int result = 0;
+
+    // process templates
+    foreach (const QString &file, files)
+    {
+        if (debug)
+            qDebug() << "processing" << file;
+
+        // create output file dir
+        QFileInfo fi(file);
+        auto outFileName = fi.canonicalFilePath();
+        outFileName.replace(inPath, outPath);
+        outFileName.remove("acctchrt_");
+        outFileName.replace(".gnucash-xea", ".kmt");
+        foreach(const QString &key, mapKeys)
+        {
+            if (outFileName.contains('/' + key + '/'))
+                outFileName = outFileName.replace('/' + key + '/', '/' + dirNameMap[key] + '/');
+        }
+        fi.setFile(outFileName);
+
+        QDir d(fi.absolutePath());
+        if (!d.exists())
+        {
+            if  (debug)
+                qDebug() << "creating path " << fi.absolutePath();
+            d.mkpath(fi.absolutePath());
+        }
+        if (debug)
+            qDebug() << "writing to " << outFileName;
+        if (!convertFile(file, outFileName))
+        {
+            qWarning() << "could not create" << outFileName;
+            result = 1;
+        }
+    }
+    return result;
+}
+
 int main(int argc, char *argv[])
 {
-    if (argc < 2)
+    if (argc < 2 || (argc == 2 && QLatin1String(argv[1]) == "--help"))
     {
-        qWarning() << "convert gnucash template file to kmymoney template file";
+        qWarning() << "xea2kmt: convert gnucash template file to kmymoney template file";
         qWarning() << argv[0] << "<options> <gnucash-template-file> [<kmymoney-template-output-file>]";
+        qWarning() << argv[0] << "<options> --in-dir <gnucash-template-files-root> --out-dir <kmymoney-template-files-root>";
         qWarning() << "options:";
-        qWarning() << "          --debug   - output debug information";
-        qWarning() << "          --with-id - write account id attribute";
+        qWarning() << "          --debug                   - output debug information";
+        qWarning() << "          --help                    - this page";
+        qWarning() << "          --no-level1-names         - do not export account names for top level accounts";
+        qWarning() << "          --prefix-name-with-code   - prefix account name with account code if present";
+        qWarning() << "          --with-id                 - write account id attribute";
+        qWarning() << "          --with-tax-related        - parse and export gnucash 'tax-related' flag";
+        qWarning() << "          --in-dir <dir>            - search for gnucash templates files in <dir>";
+        qWarning() << "          --out-dir <dir>           - generate kmymoney templates below <dir";
         return -1;
     }
 
     QString inFileName;
     QString outFileName;
+    QString inDir;
+    QString outDir;
     for(int i = 1; i < argc; i++)
     {
         QString arg = QLatin1String(argv[i]);
@@ -444,6 +617,14 @@ int main(int argc, char *argv[])
             withID = true;
         else if (arg == "--no-level1-names")
             noLevel1Names = true;
+        else if (arg == "--with-tax-related")
+            withTax = true;
+        else if (arg == "--prefix-name-with-code")
+            prefixNameWithCode = true;
+        else if (arg == "--in-dir")
+            inDir = argv[++i];
+        else if (arg == "--out-dir")
+            outDir = argv[++i];
         else if (!arg.startsWith(QLatin1String("--")))
         {
             if (inFileName.isEmpty())
@@ -458,6 +639,11 @@ int main(int argc, char *argv[])
         }
     }
 
+    if (!inDir.isEmpty() && !outDir.isEmpty())
+    {
+        return convertFileStructure(inDir, outDir);
+    }
+
     GnuCashAccountTemplateReader reader;
     bool result = reader.read(inFileName);
     if (debug)
